@@ -875,9 +875,11 @@ native_inop_setattr(struct pnode *pno,
 		/*
 		 * Do the truncate last. It can't be undone.
 		 */
-		 (void )(fd < 0
+		 err = fd < 0
 			   ? syscall(SYSIO_SYS_truncate, path, stat->st_size)
-			   : syscall(SYSIO_SYS_ftruncate, fd, stat->st_size));
+			   : syscall(SYSIO_SYS_ftruncate, fd, stat->st_size);
+		if (err)
+			err = -errno;
 	}
 	if (!err)
 		goto out;
@@ -931,7 +933,7 @@ native_pos(int fd, _SYSIO_OFF_T *offset, int whence)
 	assert(*offset >= 0);
 
 	off = *offset;
-#if _LARGEFILE64_SOURCE && defined(SYSIO_SYS__llseek)
+#if defined(_LARGEFILE64_SOURCE) && defined(SYSIO_SYS__llseek)
 	{
 		int	err;
 		err =
@@ -967,7 +969,7 @@ native_ifilldirentries(struct native_inode *nino,
 	int	err;
 	ssize_t	cc;
 #if defined(SYSIO_SYS_getdirentries)
-	_SYSIO_OFF_T	waste;
+	_SYSIO_OFF_T	waste=*posp;
 #endif
 
 	if (*posp < 0)
