@@ -4,7 +4,9 @@
 #ifndef __LIBCFS_KP30_H__
 #define __LIBCFS_KP30_H__
 
-#define LIBCFS_DEBUG
+/* Controlled via configure key */
+/* #define LIBCFS_DEBUG */
+
 #include <libcfs/libcfs.h>
 #include <lnet/types.h>
 
@@ -216,11 +218,11 @@ do {                                                                           \
    do {                                         \
         (ptr) = malloc(size);                   \
         memset(ptr, 0, size);                   \
-   } while (0);
+   } while (0)
 # else
-#  define LIBCFS_ALLOC(ptr, size) do { (ptr) = calloc(1,size); } while (0);
+#  define LIBCFS_ALLOC(ptr, size) do { (ptr) = calloc(1,size); } while (0)
 # endif
-# define LIBCFS_FREE(a, b) do { free(a); } while (0);
+# define LIBCFS_FREE(a, b) do { free(a); } while (0)
 
 void libcfs_debug_dumplog(void);
 int libcfs_debug_init(unsigned long bufsize);
@@ -543,6 +545,7 @@ extern int libcfs_ioctl_popdata(void *arg, void *buf, int size);
 #define IOC_LIBCFS_TESTPROTOCOMPAT         _IOWR('e', 60, IOCTL_LIBCFS_TYPE)
 #define IOC_LIBCFS_PING                    _IOWR('e', 61, IOCTL_LIBCFS_TYPE)
 #define IOC_LIBCFS_DEBUG_PEER              _IOWR('e', 62, IOCTL_LIBCFS_TYPE)
+#define IOC_LIBCFS_LNETST                  _IOWR('e', 63, IOCTL_LIBCFS_TYPE)
 /* lnd ioctls */
 #define IOC_LIBCFS_REGISTER_MYNID          _IOWR('e', 70, IOCTL_LIBCFS_TYPE)
 #define IOC_LIBCFS_CLOSE_CONNECTION        _IOWR('e', 71, IOCTL_LIBCFS_TYPE)
