@@ -1,7 +1,8 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  Copyright (c) 2003 Los Alamos National Laboratory (LANL)
+ * Copyright (c) 2003 Los Alamos National Laboratory (LANL)
+ * Copyright (C) 2005 Cluster File Systems, Inc. All rights reserved.
  *
  *   This file is part of Lustre, http://www.lustre.org/
  *
@@ -191,7 +192,7 @@ gmnal_tx_done(gmnal_tx_t *tx, int rc)
 
         /* Delay finalize until tx is free */
         if (lnetmsg != NULL)
-                lnet_finalize(gmni->gmni_ni, lnetmsg, 0);
+                lnet_finalize(gmni->gmni_ni, lnetmsg, rc);
 }
 
 void 
