@@ -443,7 +443,7 @@ kibnal_handle_svcqry (struct socket *sock)
         }
 
         version = (msg->ibm_magic == IBNAL_MSG_MAGIC) ?
-                  msg->ibm_version : __swab32(msg->ibm_version);
+                  msg->ibm_version : __swab16(msg->ibm_version);
         /* Peer is a different protocol version: reply in my current protocol
          * to tell her I'm "old" */
         if (version != IBNAL_MSG_VERSION &&
@@ -786,6 +786,10 @@ kibnal_add_persistent_peer (lnet_nid_t nid, __u32 ip, int port)
 
         write_lock_irqsave (&kibnal_data.kib_global_lock, flags);
 
+        /* I'm always called with a reference on kibnal_data.kib_ni
+         * so shutdown can't have started */
+        LASSERT (kibnal_data.kib_nonewpeers == 0);
+
         peer2 = kibnal_find_peer_locked (nid);
         if (peer2 != NULL) {
                 kibnal_peer_decref(peer);
@@ -1746,7 +1750,7 @@ kibnal_startup (lnet_ni_t *ni)
         
         snprintf(ipif_name, sizeof(ipif_name), "%s%d",
                  *kibnal_tunables.kib_ipif_basename, i);
-        if (strlen(ipif_name) == sizeof(ipif_name - 1)) {
+        if (strlen(ipif_name) == sizeof(ipif_name) - 1) {
                 CERROR("IPoIB interface name %s truncated\n", ipif_name);
                 return -EINVAL;
         }
