@@ -36,7 +36,7 @@ static void lnet_drop_delayed_put(lnet_msg_t *msg, char *reason);
 
 #define LNET_MATCHMD_NONE     0   /* Didn't match */
 #define LNET_MATCHMD_OK       1   /* Matched OK */
-#define LNET_MATCHMD_DROP     2   /* Must be disarded */
+#define LNET_MATCHMD_DROP     2   /* Must be discarded */
 
 static int
 lnet_try_match_md (int index, int op_mask, lnet_process_id_t src,
@@ -485,7 +485,7 @@ lnet_copy_kiov2kiov (unsigned int ndiov, lnet_kiov_t *diov, unsigned int doffset
         LASSERT (!in_interrupt ());
 
         LASSERT (ndiov > 0);
-        while (doffset > diov->kiov_len) {
+        while (doffset >= diov->kiov_len) {
                 doffset -= diov->kiov_len;
                 diov++;
                 ndiov--;
@@ -493,7 +493,7 @@ lnet_copy_kiov2kiov (unsigned int ndiov, lnet_kiov_t *diov, unsigned int doffset
         }
 
         LASSERT (nsiov > 0);
-        while (soffset > siov->kiov_len) {
+        while (soffset >= siov->kiov_len) {
                 soffset -= siov->kiov_len;
                 siov++;
                 nsiov--;
@@ -565,7 +565,7 @@ lnet_copy_kiov2iov (unsigned int niov, struct iovec *iov, unsigned int iovoffset
         LASSERT (!in_interrupt ());
 
         LASSERT (niov > 0);
-        while (iovoffset > iov->iov_len) {
+        while (iovoffset >= iov->iov_len) {
                 iovoffset -= iov->iov_len;
                 iov++;
                 niov--;
@@ -573,7 +573,7 @@ lnet_copy_kiov2iov (unsigned int niov, struct iovec *iov, unsigned int iovoffset
         }
 
         LASSERT (nkiov > 0);
-        while (kiovoffset > kiov->kiov_len) {
+        while (kiovoffset >= kiov->kiov_len) {
                 kiovoffset -= kiov->kiov_len;
                 kiov++;
                 nkiov--;
@@ -634,7 +634,7 @@ lnet_copy_iov2kiov (unsigned int nkiov, lnet_kiov_t *kiov, unsigned int kiovoffs
         LASSERT (!in_interrupt ());
 
         LASSERT (nkiov > 0);
-        while (kiovoffset > kiov->kiov_len) {
+        while (kiovoffset >= kiov->kiov_len) {
                 kiovoffset -= kiov->kiov_len;
                 kiov++;
                 nkiov--;
@@ -642,7 +642,7 @@ lnet_copy_iov2kiov (unsigned int nkiov, lnet_kiov_t *kiov, unsigned int kiovoffs
         }
 
         LASSERT (niov > 0);
-        while (iovoffset > iov->iov_len) {
+        while (iovoffset >= iov->iov_len) {
                 iovoffset -= iov->iov_len;
                 iov++;
                 niov--;
@@ -1432,7 +1432,10 @@ LNetClearLazyPortal(int portal)
                 return 0;
         }
 
-        CDEBUG(D_NET, "clearing portal %d lazy\n", portal);
+        if (the_lnet.ln_shutdown)
+                CWARN ("Active lazy portal %d on exit\n", portal);
+        else
+                CDEBUG (D_NET, "clearing portal %d lazy\n", portal);
 
         /* grab all the blocked messages atomically */
         list_add(&zombies, &ptl->ptl_msgq);
@@ -1619,7 +1622,7 @@ lnet_parse_put(lnet_ni_t *ni, lnet_msg_t *msg)
                 
         case LNET_MATCHMD_NONE:
                 rc = lnet_eager_recv_locked(msg);
-                if (rc == 0) {
+                if (rc == 0 && !the_lnet.ln_shutdown) {
                         list_add_tail(&msg->msg_list, 
                                       &the_lnet.ln_portals[index].ptl_msgq);
 
@@ -1637,15 +1640,15 @@ lnet_parse_put(lnet_ni_t *ni, lnet_msg_t *msg)
                 /* fall through */
 
         case LNET_MATCHMD_DROP:
-                CWARN("Dropping PUT from %s portal %d match "LPU64
-                      " offset %d length %d: %d\n",
-                      libcfs_id2str(src), index, 
-                      hdr->msg.put.match_bits, 
-                      hdr->msg.put.offset, rlength, rc);
+                CDEBUG(D_NETERROR,
+                       "Dropping PUT from %s portal %d match "LPU64
+                       " offset %d length %d: %d\n",
+                       libcfs_id2str(src), index, 
+                       hdr->msg.put.match_bits, 
+                       hdr->msg.put.offset, rlength, rc);
                 LNET_UNLOCK();
 
                 return ENOENT;          /* +ve: OK but no match */
-
         }
 }
 
@@ -1674,13 +1677,14 @@ lnet_parse_get(lnet_ni_t *ni, lnet_msg_t *msg, int rdma_get)
                            hdr->msg.get.match_bits, msg,
                            &mlength, &offset, &md);
         if (rc == LNET_MATCHMD_DROP) {
-                CWARN("Dropping GET from %s portal %d match "LPU64
-                      " offset %d length %d\n",
-                      libcfs_id2str(src), 
-                      hdr->msg.get.ptl_index, 
-                      hdr->msg.get.match_bits, 
-                      hdr->msg.get.src_offset,
-                      hdr->msg.get.sink_length);
+                CDEBUG(D_NETERROR,
+                       "Dropping GET from %s portal %d match "LPU64
+                       " offset %d length %d\n",
+                       libcfs_id2str(src), 
+                       hdr->msg.get.ptl_index, 
+                       hdr->msg.get.match_bits, 
+                       hdr->msg.get.src_offset,
+                       hdr->msg.get.sink_length);
                 LNET_UNLOCK();
                 return ENOENT;                  /* +ve: OK but no match */
         }
@@ -1741,12 +1745,12 @@ lnet_parse_reply(lnet_ni_t *ni, lnet_msg_t *msg)
         /* NB handles only looked up by creator (no flips) */
         md = lnet_wire_handle2md(&hdr->msg.reply.dst_wmd);
         if (md == NULL || md->md_threshold == 0) {
-                CWARN("%s: Dropping REPLY from %s for %s "
-                      "MD "LPX64"."LPX64"\n", 
-                      libcfs_nid2str(ni->ni_nid), libcfs_id2str(src),
-                      (md == NULL) ? "invalid" : "inactive",
-                      hdr->msg.reply.dst_wmd.wh_interface_cookie,
-                      hdr->msg.reply.dst_wmd.wh_object_cookie);
+                CDEBUG(D_NETERROR, "%s: Dropping REPLY from %s for %s "
+                       "MD "LPX64"."LPX64"\n", 
+                       libcfs_nid2str(ni->ni_nid), libcfs_id2str(src),
+                       (md == NULL) ? "invalid" : "inactive",
+                       hdr->msg.reply.dst_wmd.wh_interface_cookie,
+                       hdr->msg.reply.dst_wmd.wh_object_cookie);
 
                 LNET_UNLOCK();
                 return ENOENT;                  /* +ve: OK but no match */
@@ -1759,9 +1763,9 @@ lnet_parse_reply(lnet_ni_t *ni, lnet_msg_t *msg)
 
         if (mlength < rlength &&
             (md->md_options & LNET_MD_TRUNCATE) == 0) {
-                CERROR ("%s: Dropping REPLY from %s length %d "
-                        "for MD "LPX64" would overflow (%d)\n",
-                        libcfs_nid2str(ni->ni_nid), libcfs_id2str(src),
+                CDEBUG(D_NETERROR, "%s: Dropping REPLY from %s length %d "
+                       "for MD "LPX64" would overflow (%d)\n",
+                       libcfs_nid2str(ni->ni_nid), libcfs_id2str(src),
                         rlength, hdr->msg.reply.dst_wmd.wh_object_cookie,
                         mlength);
                 LNET_UNLOCK();
@@ -1814,14 +1818,13 @@ lnet_parse_ack(lnet_ni_t *ni, lnet_msg_t *msg)
         /* NB handles only looked up by creator (no flips) */
         md = lnet_wire_handle2md(&hdr->msg.ack.dst_wmd);
         if (md == NULL || md->md_threshold == 0) {
-#if 0
                 /* Don't moan; this is expected */
-                CERROR ("%s: Dropping ACK from %s to %s MD "LPX64"."LPX64"\n",
-                        libcfs_nid2str(ni->ni_nid), libcfs_id2str(src),
-                        (md == NULL) ? "invalid" : "inactive",
-                        hdr->msg.ack.dst_wmd.wh_interface_cookie,
-                        hdr->msg.ack.dst_wmd.wh_object_cookie);
-#endif
+                CDEBUG(D_NET,
+                       "%s: Dropping ACK from %s to %s MD "LPX64"."LPX64"\n",
+                       libcfs_nid2str(ni->ni_nid), libcfs_id2str(src),
+                       (md == NULL) ? "invalid" : "inactive",
+                       hdr->msg.ack.dst_wmd.wh_interface_cookie,
+                       hdr->msg.ack.dst_wmd.wh_object_cookie);
                 LNET_UNLOCK();
                 return ENOENT;                  /* +ve! */
         }
@@ -2054,8 +2057,8 @@ lnet_parse(lnet_ni_t *ni, lnet_hdr_t *hdr, lnet_nid_t from_nid,
         msg = lnet_msg_alloc();
         if (msg == NULL) {
                 CERROR("%s, src %s: Dropping %s (out of memory)\n",
-                       libcfs_nid2str(from_nid), libcfs_nid2str(src_nid)
-                       , lnet_msgtyp2str(type));
+                       libcfs_nid2str(from_nid), libcfs_nid2str(src_nid), 
+                       lnet_msgtyp2str(type));
                 goto drop;
         }
 
@@ -2116,6 +2119,8 @@ lnet_parse(lnet_ni_t *ni, lnet_hdr_t *hdr, lnet_nid_t from_nid,
         msg->msg_hdr.dest_nid = dest_nid;
         msg->msg_hdr.dest_pid = le32_to_cpu(msg->msg_hdr.dest_pid);
         msg->msg_hdr.payload_length = payload_length;
+
+        msg->msg_ev.sender = from_nid;
         
         switch (type) {
         case LNET_MSG_ACK:
@@ -2220,6 +2225,7 @@ LNetPut(lnet_nid_t self, lnet_handle_md_t mdh, lnet_ack_req_t ack,
         msg->msg_ev.initiator.nid = LNET_NID_ANY;
         msg->msg_ev.initiator.pid = the_lnet.ln_pid;
         msg->msg_ev.target = target;
+        msg->msg_ev.sender = LNET_NID_ANY;
         msg->msg_ev.pt_index = portal;
         msg->msg_ev.match_bits = match_bits;
         msg->msg_ev.rlength = md->md_length;
@@ -2291,6 +2297,7 @@ lnet_create_reply_msg (lnet_ni_t *ni, lnet_msg_t *getmsg)
 
         msg->msg_ev.type = LNET_EVENT_REPLY;
         msg->msg_ev.initiator = peer_id;
+        msg->msg_ev.sender = peer_id.nid;  /* optimized GETs can't be routed */
         msg->msg_ev.rlength = msg->msg_ev.mlength = getmd->md_length;
         msg->msg_ev.offset = 0;
 
@@ -2391,6 +2398,7 @@ LNetGet(lnet_nid_t self, lnet_handle_md_t mdh,
         msg->msg_ev.initiator.nid = LNET_NID_ANY;
         msg->msg_ev.initiator.pid = the_lnet.ln_pid;
         msg->msg_ev.target = target;
+        msg->msg_ev.sender = LNET_NID_ANY;
         msg->msg_ev.pt_index = portal;
         msg->msg_ev.match_bits = match_bits;
         msg->msg_ev.rlength = md->md_length;
@@ -2495,3 +2503,76 @@ LNetDist (lnet_nid_t dstnid, lnet_nid_t *srcnidp, int *orderp)
         return -EHOSTUNREACH;
 }
 
+int
+LNetSetAsync(lnet_process_id_t id, int nasync)
+{
+#ifdef __KERNEL__
+        return 0;
+#else
+        lnet_ni_t        *ni;
+        lnet_remotenet_t *rnet;
+        struct list_head *tmp;
+        lnet_route_t     *route;
+        lnet_nid_t       *nids;
+        int               nnids;
+        int               maxnids = 256;
+        int               rc = 0;
+        int               rc2;
+        
+        /* Target on a local network? */ 
+        
+        ni = lnet_net2ni(LNET_NIDNET(id.nid));
+        if (ni != NULL) {
+                if (ni->ni_lnd->lnd_setasync != NULL) 
+                        rc = (ni->ni_lnd->lnd_setasync)(ni, id, nasync);
+                lnet_ni_decref(ni);
+                return rc;
+        }
+
+        /* Target on a remote network: apply to routers */
+ again:
+        LIBCFS_ALLOC(nids, maxnids * sizeof(*nids));
+        if (nids == NULL)
+                return -ENOMEM;
+        nnids = 0;
+
+        /* Snapshot all the router NIDs */
+        LNET_LOCK();
+        rnet = lnet_find_net_locked(LNET_NIDNET(id.nid));
+        if (rnet != NULL) {
+                list_for_each(tmp, &rnet->lrn_routes) {
+                        if (nnids == maxnids) {
+                                LNET_UNLOCK();
+                                LIBCFS_FREE(nids, maxnids * sizeof(*nids));
+                                maxnids *= 2;
+                                goto again;
+                        }
+                        
+                        route = list_entry(tmp, lnet_route_t, lr_list);
+                        nids[nnids++] = route->lr_gateway->lp_nid;
+                }
+        }
+        LNET_UNLOCK();
+
+        /* set async on all the routers */
+        while (nnids-- > 0) {
+                id.pid = LUSTRE_SRV_LNET_PID;
+                id.nid = nids[nnids];
+
+                ni = lnet_net2ni(LNET_NIDNET(id.nid));
+                if (ni == NULL)
+                        continue;
+                
+                if (ni->ni_lnd->lnd_setasync != NULL) {
+                        rc2 = (ni->ni_lnd->lnd_setasync)(ni, id, nasync);
+                        if (rc2 != 0)
+                                rc = rc2;
+                }
+                lnet_ni_decref(ni);
+        }
+
+        LIBCFS_FREE(nids, maxnids * sizeof(*nids));
+        return rc;
+#endif
+}
+
