@@ -570,7 +570,7 @@ jt_ptl_print_peers (int argc, char **argv)
         int                      index;
         int                      rc;
 
-        if (!g_net_is_compatible (argv[0], SOCKLND, RALND, PTLLND,
+        if (!g_net_is_compatible (argv[0], SOCKLND, RALND, PTLLND, MXLND,
                                   OPENIBLND, CIBLND, IIBLND, VIBLND, O2IBLND, 0))
                 return -1;
 
@@ -951,58 +951,6 @@ jt_ptl_print_active_txs (int argc, char **argv)
         return 0;
 }
 
-int jt_ptl_ping_test(int argc, char **argv)
-{
-        int       rc;
-        lnet_nid_t nid;
-        long      count   = 1;
-        long      size    = 4;
-        long      timeout = 1;
-        struct libcfs_ioctl_data data;
-
-        if (argc < 2) {
-                fprintf(stderr, "usage: %s nid [count] [size] [timeout (secs)]\n", argv[0]);
-                return 0;
-        }
-
-        nid = libcfs_str2nid(argv[1]);
-        if (nid == LNET_NID_ANY) {
-                fprintf (stderr, "Can't parse nid \"%s\"\n", argv[1]);
-                return (-1);
-        }
-        
-        if (argc > 2)
-        {
-                count = atol(argv[2]);
-
-                if (count < 0 || count > 20000) 
-                {
-                        fprintf(stderr, "are you insane?  %ld is a crazy count.\n", count);
-                        return -1;
-                }
-        }
-        
-        if (argc > 3)
-                size= atol(argv[3]);
-
-        if (argc > 4)
-                timeout = atol (argv[4]);
-        
-        LIBCFS_IOC_INIT (data);
-        data.ioc_count   = count;
-        data.ioc_nid     = nid;
-        data.ioc_u32[0]  = size;
-        data.ioc_u32[1]  = timeout;
-        
-        rc = l_ioctl(LNET_DEV_ID, IOC_LIBCFS_PING_TEST, &data);
-        if (rc) {
-                fprintf(stderr, "failed to start pinger: %s\n",
-                        strerror(errno));
-                return -1;
-        }
-        return 0;
-}
-
 int jt_ptl_ping(int argc, char **argv)
 {
         int                      rc;
@@ -1011,17 +959,38 @@ int jt_ptl_ping(int argc, char **argv)
         lnet_process_id_t        ids[16];
         int                      maxids = sizeof(ids)/sizeof(ids[0]);
         struct libcfs_ioctl_data data;
+        char                    *sep;
         int                      i;
 
         if (argc < 2) {
-                fprintf(stderr, "usage: %s nid [timeout (secs)] [pid]\n", argv[0]);
+                fprintf(stderr, "usage: %s id [timeout (secs)]\n", argv[0]);
                 return 0;
         }
 
-        id.nid = libcfs_str2nid(argv[1]);
-        if (id.nid == LNET_NID_ANY) {
-                fprintf (stderr, "Can't parse nid \"%s\"\n", argv[1]);
-                return -1;
+        sep = strchr(argv[1], '-');
+        if (sep == NULL) {
+                id.pid = LNET_PID_ANY;
+                id.nid = libcfs_str2nid(argv[1]);
+                if (id.nid == LNET_NID_ANY) {
+                        fprintf (stderr, "Can't parse nid \"%s\"\n", argv[1]);
+                        return -1;
+                }
+        } else {
+                char   *end;
+                
+                if (argv[1][0] == 'u' ||
+                    argv[1][0] == 'U')
+                        id.pid = strtoul(&argv[1][1], &end, 0) | LNET_PID_USERFLAG;
+                else
+                        id.pid = strtoul(argv[1], &end, 0);
+
+                id.nid = libcfs_str2nid(sep + 1);
+                
+                if (end != sep ||
+                    id.nid == LNET_NID_ANY) {
+                        fprintf(stderr, "Can't parse process id \"%s\"\n", argv[1]);
+                        return -1;
+                }
         }
 
         if (argc > 2)
@@ -1029,11 +998,6 @@ int jt_ptl_ping(int argc, char **argv)
         else
                 timeout = 1000;                 /* default 1 second timeout */
 
-        if (argc > 3)
-                id.pid = atol(argv[3]);
-        else
-                id.pid = LNET_PID_ANY;
-
         LIBCFS_IOC_INIT (data);
         data.ioc_nid     = id.nid;
         data.ioc_u32[0]  = id.pid;
