@@ -201,7 +201,7 @@ static inline int cleanup_group_info(void)
 
 #include <linux/proc_fs.h>
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
+#ifndef HAVE___D_REHASH
 #define __d_rehash(dentry, lock) d_rehash_cond(dentry, lock)
 #endif
 
@@ -213,6 +213,8 @@ static inline int cleanup_group_info(void)
         flock_lock_file_wait(file, lock)
 #endif
 
+#define CheckWriteback(page, cmd) \
+        (!(!PageWriteback(page) && cmd == OBD_BRW_WRITE))
 
 #else /* 2.4.. */
 
@@ -354,6 +356,7 @@ static inline void cond_resched(void)
 #define __set_page_ll_data(page, llap) set_page_private(page, (unsigned long)llap)
 #define __clear_page_ll_data(page) set_page_private(page, 0)
 #define PageWriteback(page) 0
+#define CheckWriteback(page, cmd) 1
 #define set_page_writeback(page) do {} while (0)
 #define end_page_writeback(page) do {} while (0)
 
@@ -494,6 +497,7 @@ ll_kern_mount(const char *fstype, int flags, const char *name, void *data)
         if (!type)
                 return ERR_PTR(-ENODEV);
         mnt = vfs_kern_mount(type, flags, name, data);
+        module_put(type->owner);
         return mnt;
 }
 #else
@@ -550,5 +554,22 @@ generic_file_write(struct file *filp, const char __user *buf, size_t len, loff_t
 #define p_pptr parent
 #endif
 
+#ifdef HAVE_UNREGISTER_BLKDEV_RETURN_INT
+#define ll_unregister_blkdev(a,b)       unregister_blkdev((a),(b))
+#else
+static inline 
+int ll_unregister_blkdev(unsigned int dev, const char *name)
+{
+        unregister_blkdev(dev, name);
+        return 0;
+}
+#endif
+
+#ifdef HAVE_INVALIDATE_BDEV_2ARG
+#define ll_invalidate_bdev(a,b)         invalidate_bdev((a),(b))
+#else
+#define ll_invalidate_bdev(a,b)         invalidate_bdev((a))
+#endif
+
 #endif /* __KERNEL__ */
 #endif /* _COMPAT25_H */
