@@ -48,11 +48,14 @@ struct lustre_md {
 struct mdc_op_data {
         struct ll_fid    fid1;
         struct ll_fid    fid2;
+        struct ll_fid    fid3;
+        struct ll_fid    fid4;
         __u64            mod_time;
         const char      *name;
         int              namelen;
         __u32            create_mode;
         __u32            suppgids[2];
+        void            *data;
 };
 
 struct mds_update_record {
@@ -74,6 +77,7 @@ struct mds_update_record {
         __u32 ur_mode;
         __u32 ur_flags;
         struct lvfs_grp_hash_entry *ur_grp_entry;
+        struct ldlm_request *ur_dlm;
 };
 
 /* file data for open files on MDS */
@@ -88,19 +92,26 @@ struct mds_file_data {
 
 /* ACL */
 #ifdef CONFIG_FS_POSIX_ACL
-#ifdef HAVE_XATTR_ACL
+# ifdef HAVE_XATTR_ACL
 #  define MDS_XATTR_NAME_ACL_ACCESS XATTR_NAME_ACL_ACCESS
 #  define mds_xattr_acl_size(entry) xattr_acl_size(entry)
-# else
+# else /* HAVE_XATTR_ACL */
 #  define MDS_XATTR_NAME_ACL_ACCESS POSIX_ACL_XATTR_ACCESS
 #  define mds_xattr_acl_size(entry) posix_acl_xattr_size(entry)
-# endif
+# endif /* HAVE_XATTR_ACL */
+
 # define LUSTRE_POSIX_ACL_MAX_ENTRIES   (32)
-# define LUSTRE_POSIX_ACL_MAX_SIZE      \
+
+# ifdef __KERNEL__
+#  define LUSTRE_POSIX_ACL_MAX_SIZE      \
                 (mds_xattr_acl_size(LUSTRE_POSIX_ACL_MAX_ENTRIES))
-#else
+# else /* __KERNEL__ */
+#  define LUSTRE_POSIX_ACL_MAX_SIZE	0
+# endif /* __KERNEL__ */
+
+#else /* CONFIG_FS_POSIX_ACL */
 # define LUSTRE_POSIX_ACL_MAX_SIZE      0
-#endif
+#endif /* CONFIG_FS_POSIX_ACL */
 
 /* mds/mds_reint.c */
 int mds_reint_rec(struct mds_update_record *r, int offset,
@@ -109,6 +120,8 @@ int mds_reint_rec(struct mds_update_record *r, int offset,
 /* mds/mds_lov.c */
 
 /* mdc/mdc_locks.c */
+struct md_enqueue_info;
+
 int it_disposition(struct lookup_intent *it, int flag);
 void it_set_disposition(struct lookup_intent *it, int flag);
 void it_clear_disposition(struct lookup_intent *it, int flag);
@@ -116,23 +129,22 @@ int it_open_error(int phase, struct lookup_intent *it);
 void mdc_set_lock_data(__u64 *lockh, void *data);
 int mdc_change_cbdata(struct obd_export *exp, struct ll_fid *fid,
                       ldlm_iterator_t it, void *data);
+int mdc_revalidate_lock(struct obd_export *exp,
+                        struct lookup_intent *it,
+                        struct ll_fid *fid);
 int mdc_intent_lock(struct obd_export *exp,
                     struct mdc_op_data *,
                     void *lmm, int lmmsize,
                     struct lookup_intent *, int,
                     struct ptlrpc_request **reqp,
                     ldlm_blocking_callback cb_blocking, int extra_lock_flags);
-int mdc_enqueue(struct obd_export *exp,
-                int lock_type,
-                struct lookup_intent *it,
-                int lock_mode,
-                struct mdc_op_data *data,
-                struct lustre_handle *lockh,
-                void *lmm,
-                int lmmlen,
-                ldlm_completion_callback cb_completion,
-                ldlm_blocking_callback cb_blocking,
-                void *cb_data, int extra_lock_flags);
+int mdc_enqueue(struct obd_export *exp, struct ldlm_enqueue_info *einfo,
+                struct lookup_intent *it, struct mdc_op_data *data,
+                struct lustre_handle *lockh, void *lmm, int lmmlen,
+                int extra_lock_flags);
+int mdc_intent_getattr_async(struct obd_export *exp,
+                             struct md_enqueue_info *minfo,
+                             struct ldlm_enqueue_info *einfo);
 
 /* mdc/mdc_request.c */
 int mdc_init_ea_size(struct obd_export *mdc_exp, struct obd_export *lov_exp);
@@ -182,6 +194,9 @@ int mdc_rename(struct obd_export *exp, struct mdc_op_data *data,
 int mdc_sync(struct obd_export *exp, struct ll_fid *fid,
              struct ptlrpc_request **);
 int mdc_create_client(struct obd_uuid uuid, struct ptlrpc_client *cl);
+int mdc_resource_get_unused(struct obd_export *exp, struct ll_fid *fid,
+                            struct list_head *cancels, ldlm_mode_t mode,
+                            __u64 bits);
 
 /* Store the generation of a newly-created inode in |req| for replay. */
 void mdc_store_inode_generation(struct ptlrpc_request *req, int reqoff,
@@ -197,6 +212,18 @@ static inline void mdc_pack_fid(struct ll_fid *fid, obd_id ino, __u32 gen,
         fid->f_type = type;
 }
 
+static inline int it_to_lock_mode(struct lookup_intent *it)
+{
+        /* CREAT needs to be tested before open (both could be set) */
+        if (it->it_op & IT_CREAT)
+                return LCK_CW;
+        else if (it->it_op & (IT_READDIR | IT_GETATTR | IT_OPEN | IT_LOOKUP))
+                return LCK_CR;
+
+        LBUG();
+        return -EINVAL;
+}
+
 /* ioctls for trying requests */
 #define IOC_REQUEST_TYPE                   'f'
 #define IOC_REQUEST_MIN_NR                 30
@@ -209,4 +236,25 @@ static inline void mdc_pack_fid(struct ll_fid *fid, obd_id ino, __u32 gen,
 #define IOC_REQUEST_CLOSE               _IOWR('f', 35, long)
 #define IOC_REQUEST_MAX_NR               35
 
+/* metadata stat-ahead */
+typedef int (* md_enqueue_cb_t)(struct obd_export *exp,
+                                struct ptlrpc_request *req,
+                                struct md_enqueue_info *minfo,
+                                int rc);
+
+struct md_enqueue_info {
+        struct obd_export      *mi_exp;
+        struct mdc_op_data      mi_data;
+        struct lookup_intent    mi_it;
+        struct lustre_handle    mi_lockh;
+        struct dentry          *mi_dentry;
+        md_enqueue_cb_t         mi_cb;
+        int                     mi_cbdata;
+};
+
+struct mdc_enqueue_args {
+        struct md_enqueue_info   *ma_mi;
+        struct ldlm_enqueue_info *ma_ei;
+};
+
 #endif
