@@ -37,10 +37,10 @@
 # include <sys/statfs.h>
 #endif
 
+#include <sysio.h>
 #ifdef HAVE_XTIO_H
 #include <xtio.h>
 #endif
-#include <sysio.h>
 #include <fs.h>
 #include <mount.h>
 #include <inode.h>
@@ -1112,7 +1112,7 @@ static int llu_statfs_internal(struct llu_sb_info *sbi,
                osfs->os_bavail, osfs->os_blocks, osfs->os_ffree,osfs->os_files);
 
         rc = obd_statfs_rqset(class_exp2obd(sbi->ll_osc_exp),
-                              &obd_statfs, max_age);
+                              &obd_osfs, max_age);
         if (rc) {
                 CERROR("obd_statfs fails: rc = %d\n", rc);
                 RETURN(rc);
@@ -1227,7 +1227,7 @@ static int llu_iop_mkdir_raw(struct pnode *pno, mode_t mode)
                 RETURN(err);
 
         llu_prepare_mdc_op_data(&op_data, dir, NULL, name, len, 0);
-        err = mdc_create(llu_i2sbi(dir)->ll_mdc_exp, &op_data, NULL, 0, mode,
+        err = mdc_create(llu_i2sbi(dir)->ll_mdc_exp, &op_data, NULL, 0, mode | S_IFDIR,
                          current->fsuid, current->fsgid, current->cap_effective,
                          0, &request);
         ptlrpc_req_finished(request);
@@ -1267,23 +1267,21 @@ static int llu_iop_rmdir_raw(struct pnode *pno)
 #define FCNTL_FLMASK_INVALID (O_NONBLOCK|O_ASYNC)
 
 /* refer to ll_file_flock() for details */
-static int llu_file_flock(struct inode *ino,
-                          int cmd,
-                          struct file_lock *file_lock)
+int llu_file_flock(struct inode *ino, int cmd, struct file_lock *file_lock)
 {
         struct llu_inode_info *lli = llu_i2info(ino);
         struct intnl_stat *st = llu_i2stat(ino);
         struct ldlm_res_id res_id =
-                { .name = {st->st_ino,
-                           lli->lli_st_generation, LDLM_FLOCK} };
+                { .name = {st->st_ino, lli->lli_st_generation, LDLM_FLOCK} };
+        struct ldlm_enqueue_info einfo = { LDLM_FLOCK, 0, NULL,
+                ldlm_flock_completion_ast, NULL, file_lock };
         struct lustre_handle lockh = {0};
         ldlm_policy_data_t flock;
-        ldlm_mode_t mode = 0;
         int flags = 0;
         int rc;
 
-        CDEBUG(D_VFSTRACE, "VFS Op:inode="LPU64" file_lock=%p\n",
-               st->st_ino, file_lock);
+        CDEBUG(D_VFSTRACE, "VFS Op:inode=%llu file_lock=%p\n",
+               (unsigned long long) st->st_ino, file_lock);
 
         flock.l_flock.pid = file_lock->fl_pid;
         flock.l_flock.start = file_lock->fl_start;
@@ -1291,13 +1289,13 @@ static int llu_file_flock(struct inode *ino,
 
         switch (file_lock->fl_type) {
         case F_RDLCK:
-                mode = LCK_PR;
+                einfo.ei_mode = LCK_PR;
                 break;
         case F_UNLCK:
-                mode = LCK_NL;
+                einfo.ei_mode = LCK_NL;
                 break;
         case F_WRLCK:
-                mode = LCK_PW;
+                einfo.ei_mode = LCK_PW;
                 break;
         default:
                 CERROR("unknown fcntl lock type: %d\n", file_lock->fl_type);
@@ -1328,21 +1326,21 @@ static int llu_file_flock(struct inode *ino,
 #endif
 #endif
                 flags = LDLM_FL_TEST_LOCK;
-                file_lock->fl_type = mode;
+                file_lock->fl_type = einfo.ei_mode;
                 break;
         default:
                 CERROR("unknown fcntl cmd: %d\n", cmd);
                 LBUG();
         }
 
-        CDEBUG(D_DLMTRACE, "inode="LPU64", pid=%u, flags=%#x, mode=%u, "
-               "start="LPU64", end="LPU64"\n", st->st_ino, flock.l_flock.pid,
-               flags, mode, flock.l_flock.start, flock.l_flock.end);
+        CDEBUG(D_DLMTRACE, "inode=%llu, pid=%u, flags=%#x, mode=%u, "
+               "start="LPU64", end="LPU64"\n",
+               (unsigned long long) st->st_ino, flock.l_flock.pid,
+               flags, einfo.ei_mode, flock.l_flock.start, flock.l_flock.end);
+
+        rc = ldlm_cli_enqueue(llu_i2mdcexp(ino), NULL, &einfo, res_id, 
+                              &flock, &flags, NULL, 0, NULL, &lockh, 0);
 
-        rc = ldlm_cli_enqueue(llu_i2mdcexp(ino), NULL, res_id, 
-                              LDLM_FLOCK, &flock, mode, &flags, NULL, 
-                              ldlm_flock_completion_ast, NULL, 
-                              file_lock, NULL, 0, NULL, &lockh, 0);
         RETURN(rc);
 }
 
@@ -1397,7 +1395,7 @@ static int llu_fcntl_getlk(struct inode *ino, struct flock *flock)
         struct file_lock fl;
         int error;
 
-        error = EINVAL;
+        error = -EINVAL;
         if ((flock->l_type != F_RDLCK) && (flock->l_type != F_WRLCK))
                 goto out;
 
@@ -1476,10 +1474,11 @@ static int llu_iop_fcntl(struct inode *ino, int cmd, va_list ap, int *rtn)
                 flags = va_arg(ap, long);
                 flags &= FCNTL_FLMASK;
                 if (flags & FCNTL_FLMASK_INVALID) {
-                        CERROR("liblustre don't support O_NONBLOCK, O_ASYNC, "
-                               "and O_DIRECT on file descriptor\n");
-                        *rtn = -EINVAL;
-                        err = EINVAL;
+                        LCONSOLE_ERROR_MSG(0x010, "liblustre does not support "
+                                           "the O_NONBLOCK or O_ASYNC flags. "
+                                           "Please fix your application.\n");
+                        *rtn = -1;
+                        err = -EINVAL;
                         break;
                 }
                 lli->lli_open_flags = (int)(flags & FCNTL_FLMASK) |
@@ -1514,8 +1513,8 @@ static int llu_iop_fcntl(struct inode *ino, int cmd, va_list ap, int *rtn)
                 break;
         default:
                 CERROR("unsupported fcntl cmd %x\n", cmd);
-                *rtn = -ENOSYS;
-                err = ENOSYS;
+                *rtn = -1;
+                err = -ENOSYS;
                 break;
         }
 
@@ -1628,6 +1627,9 @@ static int llu_lov_setstripe_ea_info(struct inode *ino, int flags,
         struct llu_inode_info *lli2 = NULL;
         struct lov_stripe_md *lsm;
         struct lookup_intent oit = {.it_op = IT_OPEN, .it_flags = flags};
+        struct ldlm_enqueue_info einfo = { LDLM_IBITS, LCK_CR,
+                llu_mdc_blocking_ast, ldlm_completion_ast, NULL, NULL };
+
         struct ptlrpc_request *req = NULL;
         struct lustre_md md;
         struct mdc_op_data data;
@@ -1656,9 +1658,8 @@ static int llu_lov_setstripe_ea_info(struct inode *ino, int flags,
 
         llu_prepare_mdc_op_data(&data, NULL, ino, NULL, 0, O_RDWR);
 
-        rc = mdc_enqueue(sbi->ll_mdc_exp, LDLM_IBITS, &oit, LCK_CR, &data,
-                         &lockh, lum, lum_size, ldlm_completion_ast,
-                         llu_mdc_blocking_ast, NULL, LDLM_FL_INTENT_ONLY);
+        rc = mdc_enqueue(sbi->ll_mdc_exp, &einfo, &oit, &data,
+                         &lockh, lum, lum_size, LDLM_FL_INTENT_ONLY);
         if (rc)
                 GOTO(out, rc);
         
@@ -1884,7 +1885,7 @@ llu_fsswop_mount(const char *source,
                 RETURN(-ENOMEM);
 
         INIT_LIST_HEAD(&sbi->ll_conn_chain);
-        generate_random_uuid(uuid);
+        ll_generate_random_uuid(uuid);
         class_uuid_unparse(uuid, &sbi->ll_sb_uuid);
 
         /* generate a string unique to this super, let's try
