@@ -1,33 +1,31 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright (C) 1992, 1993, 1994, 1995
- * Remy Card (card@masi.ibp.fr)
- * Laboratoire MASI - Institut Blaise Pascal
- * Universite Pierre et Marie Curie (Paris VI)
+ * Directory code for lustre client.
  *
- *  from
+ *  Copyright (C) 2002--2007 Cluster File Systems, Inc.
  *
- *  linux/fs/minix/dir.c
- *  linux/fs/ext2/dir.c
+ *   This file is part of the Lustre file system, http://www.lustre.org
+ *   Lustre is a trademark of Cluster File Systems, Inc.
  *
- *  Copyright (C) 1991, 1992  Linus Torvalds
+ *   You may have signed or agreed to another license before downloading
+ *   this software.  If so, you are bound by the terms and conditions
+ *   of that agreement, and the following does not apply to you.  See the
+ *   LICENSE file included with this distribution for more information.
  *
- *  ext2 directory handling functions
+ *   If you did not agree to a different license, then this copy of Lustre
+ *   is open source software; you can redistribute it and/or modify it
+ *   under the terms of version 2 of the GNU General Public License as
+ *   published by the Free Software Foundation.
  *
- *  Big-endian to little-endian byte-swapping/bitmaps by
- *        David S. Miller (davem@caip.rutgers.edu), 1995
- *
- *  All code that works with directory layout had been switched to pagecache
- *  and moved here. AV
- *
- *  Adapted for Lustre Light
- *  Copyright (C) 2002-2003, Cluster File Systems, Inc.
+ *   In either case, Lustre is distributed in the hope that it will be
+ *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+ *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   license text for more details.
  *
  */
 
 #include <linux/fs.h>
-#include <linux/ext2_fs.h>
 #include <linux/pagemap.h>
 #include <linux/mm.h>
 #include <linux/version.h>
@@ -49,10 +47,37 @@
 #include <lustre_dlm.h>
 #include "llite_internal.h"
 
-typedef struct ext2_dir_entry_2 ext2_dirent;
+/*
+ * Directory entries are currently in the same format as ext2/ext3, but will
+ * be changed in the future to accomodate FIDs
+ */
+#define LL_DIR_NAME_LEN (255)
 
-#define PageChecked(page)        test_bit(PG_checked, &(page)->flags)
-#define SetPageChecked(page)     set_bit(PG_checked, &(page)->flags)
+static const int LL_DIR_PAD = 4;
+
+struct ll_dir_entry {
+        /* number of inode, referenced by this entry */
+	__le32	lde_inode;
+        /* total record length, multiple of LL_DIR_PAD */
+	__le16	lde_rec_len;
+        /* length of name */
+	__u8	lde_name_len;
+        /* file type: regular, directory, device, etc. */
+	__u8	lde_file_type;
+        /* name. NOT NUL-terminated */
+	char	lde_name[LL_DIR_NAME_LEN];
+};
+
+static inline unsigned ll_dir_rec_len(unsigned name_len)
+{
+        return (name_len + 8 + LL_DIR_PAD - 1) & ~(LL_DIR_PAD - 1);
+}
+
+
+#ifdef HAVE_PG_FS_MISC
+#define PageChecked(page)        test_bit(PG_fs_misc, &(page)->flags)
+#define SetPageChecked(page)     set_bit(PG_fs_misc, &(page)->flags)
+#endif
 
 /* returns the page unlocked, but with a reference */
 static int ll_dir_readpage(struct file *file, struct page *page)
@@ -80,7 +105,12 @@ static int ll_dir_readpage(struct file *file, struct page *page)
                 /* swabbed by mdc_readpage() */
                 LASSERT_REPSWABBED(request, REPLY_REC_OFF);
 
-                inode->i_size = body->size;
+                if (body->size != i_size_read(inode)) {
+                        ll_inode_size_lock(inode, 0);
+                        i_size_write(inode, body->size);
+                        ll_inode_size_unlock(inode, 0);
+                }
+
                 SetPageUptodate(page);
         }
         ptlrpc_req_finished(request);
@@ -94,116 +124,109 @@ struct address_space_operations ll_dir_aops = {
         .readpage  = ll_dir_readpage,
 };
 
+static inline unsigned ll_dir_page_mask(struct inode *inode)
+{
+        return ~(inode->i_sb->s_blocksize - 1);
+}
+
 /*
- * ext2 uses block-sized chunks. Arguably, sector-sized ones would be
- * more robust, but we have what we have
+ * Check consistency of a single entry.
  */
-static inline unsigned ext2_chunk_size(struct inode *inode)
+static int ll_dir_check_entry(struct inode *dir, struct ll_dir_entry *ent,
+                              unsigned offset, unsigned rec_len, pgoff_t index)
 {
-        return inode->i_sb->s_blocksize;
-}
+        const char *msg;
 
-static inline void ext2_put_page(struct page *page)
-{
-        kunmap(page);
-        page_cache_release(page);
-}
+        /*
+         * Consider adding more checks.
+         */
 
-static inline unsigned long dir_pages(struct inode *inode)
-{
-        return (inode->i_size+CFS_PAGE_SIZE-1) >> CFS_PAGE_SHIFT;
-}
-
-
-static void ext2_check_page(struct inode *dir, struct page *page)
-{
-        unsigned chunk_size = ext2_chunk_size(dir);
-        char *kaddr = page_address(page);
-        //      u32 max_inumber = le32_to_cpu(sb->u.ext2_sb.s_es->s_inodes_count);
-        unsigned rec_len;
-        __u64 offs, limit = CFS_PAGE_SIZE;
-        ext2_dirent *p;
-        char *error;
-
-        if ((dir->i_size >> CFS_PAGE_SHIFT) == (__u64)page->index) {
-                limit = dir->i_size & ~CFS_PAGE_MASK;
-                if (limit & (chunk_size - 1)) {
-                        CERROR("limit "LPU64" dir size %lld index "LPU64"\n",
-                               limit, dir->i_size, (__u64)page->index);
-                        goto Ebadsize;
-                }
-                for (offs = limit; offs < CFS_PAGE_SIZE; offs += chunk_size) {
-                        ext2_dirent *p = (ext2_dirent*)(kaddr + offs);
-                        p->rec_len = cpu_to_le16(chunk_size);
-                        p->name_len = 0;
-                        p->inode = 0;
-                }
-                if (!limit)
-                        goto out;
-        }
-        for (offs = 0; offs <= limit - EXT2_DIR_REC_LEN(1); offs += rec_len) {
-                p = (ext2_dirent *)(kaddr + offs);
-                rec_len = le16_to_cpu(p->rec_len);
-
-                if (rec_len < EXT2_DIR_REC_LEN(1))
-                        goto Eshort;
-                if (rec_len & 3)
-                        goto Ealign;
-                if (rec_len < EXT2_DIR_REC_LEN(p->name_len))
-                        goto Enamelen;
-                if (((offs + rec_len - 1) ^ offs) & ~(chunk_size-1))
-                        goto Espan;
-                //              if (le32_to_cpu(p->inode) > max_inumber)
-                //goto Einumber;
-        }
-        if (offs != limit)
-                goto Eend;
-out:
-        SetPageChecked(page);
-        return;
-
-        /* Too bad, we had an error */
-
-Ebadsize:
-        CERROR("%s: directory %lu/%u size %llu is not a multiple of %u\n",
-               ll_i2mdcexp(dir)->exp_obd->obd_name, dir->i_ino,
-               dir->i_generation, dir->i_size, chunk_size);
-        goto fail;
-Eshort:
-        error = "rec_len is smaller than minimal";
-        goto bad_entry;
-Ealign:
-        error = "unaligned directory entry";
-        goto bad_entry;
-Enamelen:
-        error = "rec_len is too small for name_len";
-        goto bad_entry;
-Espan:
-        error = "directory entry across blocks";
-        goto bad_entry;
-        //Einumber:
-        // error = "inode out of bounds";
-bad_entry:
+        if (unlikely(rec_len < ll_dir_rec_len(1)))
+                msg = "entry is too short";
+        else if (unlikely(rec_len & 3))
+                msg = "wrong alignment";
+        else if (unlikely(rec_len < ll_dir_rec_len(ent->lde_name_len)))
+                msg = "rec_len doesn't match name_len";
+        else if (unlikely(((offset + rec_len - 1) ^ offset) &
+                          ll_dir_page_mask(dir)))
+                msg = "directory entry across blocks";
+        else
+                return 0;
         CERROR("%s: bad entry in directory %lu/%u: %s - "
-               "offset="LPU64"+"LPU64", inode=%lu, rec_len=%d, name_len=%d\n",
-               ll_i2mdcexp(dir)->exp_obd->obd_name, dir->i_ino,
-               dir->i_generation, error, (__u64)page->index << CFS_PAGE_SHIFT,
-               offs, (unsigned long)le32_to_cpu(p->inode),
-               rec_len, p->name_len);
-        goto fail;
-Eend:
-        p = (ext2_dirent *)(kaddr + offs);
-        CERROR("%s: entry in directory %lu/%u spans the page boundary "
-               "offset="LPU64"+"LPU64", inode=%lu\n",ll_i2mdcexp(dir)->exp_obd->obd_name,
-               dir->i_ino, dir->i_generation,
-               (__u64)page->index << CFS_PAGE_SHIFT,
-               offs, (unsigned long)le32_to_cpu(p->inode));
-fail:
-        SetPageChecked(page);
-        SetPageError(page);
+               "offset=%lu+%u, inode=%lu, rec_len=%d,"
+               " name_len=%d\n", ll_i2mdcexp(dir)->exp_obd->obd_name,
+               dir->i_ino, dir->i_generation, msg,
+               index << CFS_PAGE_SHIFT,
+               offset, (unsigned long)le32_to_cpu(ent->lde_inode),
+               rec_len, ent->lde_name_len);
+        return -EIO;
 }
 
-static struct page *ll_get_dir_page(struct inode *dir, unsigned long n)
+static inline struct ll_dir_entry *ll_entry_at(void *base, unsigned offset)
+{
+        return (struct ll_dir_entry *)(base + offset);
+}
+
+static void ll_dir_check_page(struct inode *dir, struct page *page)
+{
+        int      err;
+        unsigned size = dir->i_sb->s_blocksize;
+        char    *addr = page_address(page);
+        unsigned off;
+        unsigned limit;
+        unsigned reclen;
+
+        struct ll_dir_entry *ent;
+
+        err = 0;
+        if ((i_size_read(dir) >> CFS_PAGE_SHIFT) == (__u64)page->index) {
+                /*
+                 * Last page.
+                 */
+                limit = i_size_read(dir) & ~CFS_PAGE_MASK;
+                if (limit & (size - 1)) {
+                        CERROR("%s: dir %lu/%u size %llu doesn't match %u\n",
+                               ll_i2mdcexp(dir)->exp_obd->obd_name, dir->i_ino,
+                               dir->i_generation, i_size_read(dir), size);
+                        err++;
+                } else {
+                        /*
+                         * Place dummy forwarding entries to streamline
+                         * ll_readdir().
+                         */
+                        for (off = limit; off < CFS_PAGE_SIZE; off += size) {
+                                ent = ll_entry_at(addr, off);
+                                ent->lde_rec_len = cpu_to_le16(size);
+                                ent->lde_name_len = 0;
+                                ent->lde_inode = 0;
+                        }
+                }
+        } else
+                limit = CFS_PAGE_SIZE;
+
+        for (off = 0;
+             !err && off <= limit - ll_dir_rec_len(1); off += reclen) {
+                ent    = ll_entry_at(addr, off);
+                reclen = le16_to_cpu(ent->lde_rec_len);
+                err    = ll_dir_check_entry(dir, ent, off, reclen, page->index);
+        }
+
+        if (!err && off != limit) {
+                ent = ll_entry_at(addr, off);
+                CERROR("%s: entry in directory %lu/%u spans the page boundary "
+                       "offset="LPU64"+%u, inode=%lu\n",
+                       ll_i2mdcexp(dir)->exp_obd->obd_name,
+                       dir->i_ino, dir->i_generation,
+                       (__u64)page->index << CFS_PAGE_SHIFT,
+                       off, (unsigned long)le32_to_cpu(ent->lde_inode));
+                err++;
+        }
+        if (err)
+                SetPageError(page);
+        SetPageChecked(page);
+}
+
+struct page *ll_get_dir_page(struct inode *dir, unsigned long n)
 {
         struct ldlm_res_id res_id =
                 { .name = { dir->i_ino, (__u64)dir->i_generation} };
@@ -218,15 +241,15 @@ static struct page *ll_get_dir_page(struct inode *dir, unsigned long n)
                              &res_id, LDLM_IBITS, &policy, LCK_CR, &lockh);
         if (!rc) {
                 struct lookup_intent it = { .it_op = IT_READDIR };
+                struct ldlm_enqueue_info einfo = { LDLM_IBITS, LCK_CR,
+                       ll_mdc_blocking_ast, ldlm_completion_ast, NULL, dir };
                 struct ptlrpc_request *request;
                 struct mdc_op_data data;
 
-                ll_prepare_mdc_op_data(&data, dir, NULL, NULL, 0, 0);
+                ll_prepare_mdc_op_data(&data, dir, NULL, NULL, 0, 0, NULL);
 
-                rc = mdc_enqueue(ll_i2sbi(dir)->ll_mdc_exp, LDLM_IBITS, &it,
-                                 LCK_CR, &data, &lockh, NULL, 0,
-                                 ldlm_completion_ast, ll_mdc_blocking_ast, dir,
-                                 0);
+                rc = mdc_enqueue(ll_i2sbi(dir)->ll_mdc_exp, &einfo, &it,
+                                 &data, &lockh, NULL, 0, 0);
 
                 request = (struct ptlrpc_request *)it.d.lustre.it_data;
                 if (request)
@@ -248,7 +271,7 @@ static struct page *ll_get_dir_page(struct inode *dir, unsigned long n)
         if (!PageUptodate(page))
                 goto fail;
         if (!PageChecked(page))
-                ext2_check_page(dir, page);
+                ll_dir_check_page(dir, page);
         if (PageError(page))
                 goto fail;
 
@@ -257,7 +280,8 @@ out_unlock:
         return page;
 
 fail:
-        ext2_put_page(page);
+        kunmap(page);
+        page_cache_release(page);
         page = ERR_PTR(-EIO);
         goto out_unlock;
 }
@@ -265,105 +289,159 @@ fail:
 /*
  * p is at least 6 bytes before the end of page
  */
-static inline ext2_dirent *ext2_next_entry(ext2_dirent *p)
+static inline struct ll_dir_entry *ll_dir_next_entry(struct ll_dir_entry *p)
 {
-        return (ext2_dirent *)((char*)p + le16_to_cpu(p->rec_len));
+        return ll_entry_at(p, le16_to_cpu(p->lde_rec_len));
 }
 
-static inline unsigned
-ext2_validate_entry(char *base, unsigned offset, unsigned mask)
+static inline unsigned ll_dir_validate_entry(char *base, unsigned offset,
+                                             unsigned mask)
 {
-        ext2_dirent *de = (ext2_dirent*)(base + offset);
-        ext2_dirent *p = (ext2_dirent*)(base + (offset&mask));
-        while ((char*)p < (char*)de)
-                p = ext2_next_entry(p);
+        struct ll_dir_entry *de = ll_entry_at(base, offset);
+        struct ll_dir_entry *p  = ll_entry_at(base, offset & mask);
+        while (p < de && p->lde_rec_len > 0)
+                p = ll_dir_next_entry(p);
         return (char *)p - base;
 }
 
-static unsigned char ext2_filetype_table[EXT2_FT_MAX] = {
-        [EXT2_FT_UNKNOWN]       DT_UNKNOWN,
-        [EXT2_FT_REG_FILE]      DT_REG,
-        [EXT2_FT_DIR]           DT_DIR,
-        [EXT2_FT_CHRDEV]        DT_CHR,
-        [EXT2_FT_BLKDEV]        DT_BLK,
-        [EXT2_FT_FIFO]          DT_FIFO,
-        [EXT2_FT_SOCK]          DT_SOCK,
-        [EXT2_FT_SYMLINK]       DT_LNK,
+/*
+ * File type constants. The same as in ext2 for compatibility.
+ */
+
+enum {
+        LL_DIR_FT_UNKNOWN,
+        LL_DIR_FT_REG_FILE,
+        LL_DIR_FT_DIR,
+        LL_DIR_FT_CHRDEV,
+        LL_DIR_FT_BLKDEV,
+        LL_DIR_FT_FIFO,
+        LL_DIR_FT_SOCK,
+        LL_DIR_FT_SYMLINK,
+        LL_DIR_FT_MAX
 };
 
+static unsigned char ll_dir_filetype_table[LL_DIR_FT_MAX] = {
+        [LL_DIR_FT_UNKNOWN]  = DT_UNKNOWN,
+        [LL_DIR_FT_REG_FILE] = DT_REG,
+        [LL_DIR_FT_DIR]      = DT_DIR,
+        [LL_DIR_FT_CHRDEV]   = DT_CHR,
+        [LL_DIR_FT_BLKDEV]   = DT_BLK,
+        [LL_DIR_FT_FIFO]     = DT_FIFO,
+        [LL_DIR_FT_SOCK]     = DT_SOCK,
+        [LL_DIR_FT_SYMLINK]  = DT_LNK,
+};
+
+/*
+ * Process one page. Returns:
+ *
+ *     -ve: filldir commands readdir to stop.
+ *     +ve: number of entries submitted to filldir.
+ *       0: no live entries on this page.
+ */
+
+int ll_readdir_page(char *addr, __u64 base, unsigned *offset,
+                    filldir_t filldir, void *cookie)
+{
+        struct ll_dir_entry *de;
+        char *end;
+        int nr;
+
+        de = ll_entry_at(addr, *offset);
+        end = addr + CFS_PAGE_SIZE - ll_dir_rec_len(1);
+        for (nr = 0 ;(char*)de <= end; de = ll_dir_next_entry(de)) {
+                if (de->lde_inode != 0) {
+                        nr++;
+                        *offset = (char *)de - addr;
+                        if (filldir(cookie, de->lde_name, de->lde_name_len,
+                                    base | *offset, le32_to_cpu(de->lde_inode),
+                                    ll_dir_filetype_table[de->lde_file_type &
+                                                          (LL_DIR_FT_MAX - 1)]))
+                                return -1;
+                }
+        }
+        return nr;
+}
 
 int ll_readdir(struct file *filp, void *dirent, filldir_t filldir)
 {
         struct inode *inode = filp->f_dentry->d_inode;
-        loff_t pos = filp->f_pos;
-        // XXX struct super_block *sb = inode->i_sb;
-        __u64 offset = pos & ~CFS_PAGE_MASK;
-        __u64 n = pos >> CFS_PAGE_SHIFT;
-        unsigned long npages = dir_pages(inode);
-        unsigned chunk_mask = ~(ext2_chunk_size(inode)-1);
-        unsigned char *types = ext2_filetype_table;
+        loff_t pos          = filp->f_pos;
+        unsigned offset     = pos & ~CFS_PAGE_MASK;
+        pgoff_t idx         = pos >> CFS_PAGE_SHIFT;
+        pgoff_t npages      = dir_pages(inode);
+        unsigned chunk_mask = ll_dir_page_mask(inode);
         int need_revalidate = (filp->f_version != inode->i_version);
-        int rc = 0;
+        int rc              = 0;
+        int done; /* when this becomes negative --- stop iterating */
+
         ENTRY;
 
         CDEBUG(D_VFSTRACE, "VFS Op:inode=%lu/%u(%p) pos %llu/%llu\n",
-               inode->i_ino, inode->i_generation, inode, pos, inode->i_size);
+               inode->i_ino, inode->i_generation, inode,
+               pos, i_size_read(inode));
 
-        if (pos > inode->i_size - EXT2_DIR_REC_LEN(1))
+        /*
+         * Checking ->i_size without the lock. Should be harmless, as server
+         * re-checks.
+         */
+        if (pos > i_size_read(inode) - ll_dir_rec_len(1))
                 RETURN(0);
 
-        for ( ; n < npages; n++, offset = 0) {
-                char *kaddr, *limit;
-                ext2_dirent *de;
+        for (done = 0; idx < npages; idx++, offset = 0) {
+                /*
+                 * We can assume that all blocks on this page are filled with
+                 * entries, because ll_dir_check_page() placed special dummy
+                 * entries for us.
+                 */
+
+                char *kaddr;
                 struct page *page;
 
-                CDEBUG(D_EXT2,"read %lu of dir %lu/%u page "LPU64"/%lu "
+                CDEBUG(D_EXT2,"read %lu of dir %lu/%u page %lu/%lu "
                        "size %llu\n",
                        CFS_PAGE_SIZE, inode->i_ino, inode->i_generation,
-                       n, npages, inode->i_size);
-                page = ll_get_dir_page(inode, n);
+                       idx, npages, i_size_read(inode));
+                page = ll_get_dir_page(inode, idx);
 
                 /* size might have been updated by mdc_readpage */
                 npages = dir_pages(inode);
 
                 if (IS_ERR(page)) {
                         rc = PTR_ERR(page);
-                        CERROR("error reading dir %lu/%u page "LPU64": rc %d\n",
-                               inode->i_ino, inode->i_generation, n, rc);
+                        CERROR("error reading dir %lu/%u page %lu: rc %d\n",
+                               inode->i_ino, inode->i_generation, idx, rc);
                         continue;
                 }
 
                 kaddr = page_address(page);
                 if (need_revalidate) {
-                        /* page already checked from ll_get_dir_page() */
-                        offset = ext2_validate_entry(kaddr, offset, chunk_mask);
+                        /*
+                         * File offset was changed by lseek() and possibly
+                         * points in the middle of an entry. Re-scan from the
+                         * beginning of the chunk.
+                         */
+                        offset = ll_dir_validate_entry(kaddr, offset,
+                                                       chunk_mask);
                         need_revalidate = 0;
                 }
-                de = (ext2_dirent *)(kaddr+offset);
-                limit = kaddr + CFS_PAGE_SIZE - EXT2_DIR_REC_LEN(1);
-                for ( ;(char*)de <= limit; de = ext2_next_entry(de)) {
-                        if (de->inode) {
-                                int over;
-
-                                rc = 0; /* no error if we return something */
-
-                                offset = (char *)de - kaddr;
-                                over = filldir(dirent, de->name, de->name_len,
-                                               (n << CFS_PAGE_SHIFT) | offset,
-                                               le32_to_cpu(de->inode),
-                                               types[de->file_type &
-                                                     (EXT2_FT_MAX - 1)]);
-                                if (over) {
-                                        ext2_put_page(page);
-                                        GOTO(done, rc);
-                                }
-                        }
-                }
-                ext2_put_page(page);
+                done = ll_readdir_page(kaddr, idx << CFS_PAGE_SHIFT,
+                                       &offset, filldir, dirent);
+                kunmap(page);
+                page_cache_release(page);
+                if (done > 0)
+                        /*
+                         * Some entries were sent to the user space, return
+                         * success.
+                         */
+                        rc = 0;
+                else if (done < 0)
+                        /*
+                         * filldir is satisfied.
+                         */
+                        break;
         }
 
-done:
-        filp->f_pos = (n << CFS_PAGE_SHIFT) | offset;
+        filp->f_pos = (idx << CFS_PAGE_SHIFT) | offset;
         filp->f_version = inode->i_version;
         touch_atime(filp->f_vfsmnt, filp->f_dentry);
 
@@ -380,11 +458,51 @@ do {                                    \
         Q_COPY(out, in, qc_dqblk);      \
 } while (0)
 
-int ll_dir_setstripe(struct inode *inode, struct lov_user_md *lump)
+int ll_send_mgc_param(struct obd_export *mgc, char *string)
+{
+        struct mgs_send_param *msp;
+        int rc = 0;
+
+        OBD_ALLOC_PTR(msp);
+        if (!msp)
+                return -ENOMEM;
+
+        strncpy(msp->mgs_param, string, MGS_PARAM_MAXLEN);
+        rc = obd_set_info_async(mgc, strlen(KEY_SET_INFO), KEY_SET_INFO,
+                                sizeof(struct mgs_send_param), msp, NULL);
+        if (rc)
+                CERROR("Failed to set parameter: %d\n", rc);
+
+        OBD_FREE_PTR(msp);
+        return rc;
+}
+
+char *ll_get_fsname(struct inode *inode)
+{
+        struct lustre_sb_info *lsi = s2lsi(inode->i_sb);
+        char *ptr, *fsname;
+        int len;
+
+        OBD_ALLOC(fsname, MGS_PARAM_MAXLEN);
+        len = strlen(lsi->lsi_lmd->lmd_profile);
+        ptr = strrchr(lsi->lsi_lmd->lmd_profile, '-');
+        if (ptr && (strcmp(ptr, "-client") == 0))
+                len -= 7;
+        strncpy(fsname, lsi->lsi_lmd->lmd_profile, len);
+        fsname[len] = '\0';
+
+        return fsname;
+}
+
+int ll_dir_setstripe(struct inode *inode, struct lov_user_md *lump,
+                     int set_default)
 {
         struct ll_sb_info *sbi = ll_i2sbi(inode);
         struct mdc_op_data data;
         struct ptlrpc_request *req = NULL;
+        struct lustre_sb_info *lsi = s2lsi(inode->i_sb);
+        struct obd_device *mgc = lsi->lsi_mgc;
+        char *fsname = NULL, *param = NULL;
 
         struct iattr attr = { 0 };
         int rc = 0;
@@ -400,7 +518,7 @@ int ll_dir_setstripe(struct inode *inode, struct lov_user_md *lump)
         if (lump->lmm_magic != cpu_to_le32(LOV_USER_MAGIC))
                 lustre_swab_lov_user_md(lump);
 
-        ll_prepare_mdc_op_data(&data, inode, NULL, NULL, 0, 0);
+        ll_prepare_mdc_op_data(&data, inode, NULL, NULL, 0, 0, NULL);
 
         /* swabbing is done in lov_setstripe() on server side */
         rc = mdc_setattr(sbi->ll_mdc_exp, &data,
@@ -413,12 +531,42 @@ int ll_dir_setstripe(struct inode *inode, struct lov_user_md *lump)
         }
         ptlrpc_req_finished(req);
 
-        return rc;
+        if (set_default && mgc->u.cli.cl_mgc_mgsexp) {
+                OBD_ALLOC(param, MGS_PARAM_MAXLEN);
 
+                /* Get fsname and assume devname to be -MDT0000. */
+                fsname = ll_get_fsname(inode);
+                /* Set root stripesize */
+                sprintf(param, "%s-MDT0000.lov.stripesize=%u", fsname,
+                        lump->lmm_stripe_size);
+                rc = ll_send_mgc_param(mgc->u.cli.cl_mgc_mgsexp, param);
+                if (rc)
+                        goto end;
+
+                /* Set root stripecount */
+                sprintf(param, "%s-MDT0000.lov.stripecount=%u", fsname,
+                        lump->lmm_stripe_count);
+                rc = ll_send_mgc_param(mgc->u.cli.cl_mgc_mgsexp, param);
+                if (rc)
+                        goto end;
+
+                /* Set root stripeoffset */
+                sprintf(param, "%s-MDT0000.lov.stripeoffset=%u", fsname,
+                        lump->lmm_stripe_offset);
+                rc = ll_send_mgc_param(mgc->u.cli.cl_mgc_mgsexp, param);
+                if (rc)
+                        goto end;
+end:
+                if (fsname)
+                        OBD_FREE(fsname, MGS_PARAM_MAXLEN);
+                if (param)
+                        OBD_FREE(param, MGS_PARAM_MAXLEN);
+        }
+        return rc;
 }
 
-int ll_dir_getstripe(struct inode *inode, struct lov_mds_md **lmmp, 
-                     int *lmm_size, struct ptlrpc_request **request) 
+int ll_dir_getstripe(struct inode *inode, struct lov_mds_md **lmmp,
+                     int *lmm_size, struct ptlrpc_request **request)
 {
         struct ll_sb_info *sbi = ll_i2sbi(inode);
         struct ll_fid     fid;
@@ -489,7 +637,7 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
         if (_IOC_TYPE(cmd) == 'T' || _IOC_TYPE(cmd) == 't') /* tty ioctls */
                 return -ENOTTY;
 
-        lprocfs_counter_incr(ll_i2sbi(inode)->ll_stats, LPROC_LL_IOCTL);
+        ll_stats_ops_tally(ll_i2sbi(inode), LPROC_LL_IOCTL, 1);
         switch(cmd) {
         case EXT3_IOC_GETFLAGS:
         case EXT3_IOC_SETFLAGS:
@@ -541,6 +689,7 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
         case LL_IOC_LOV_SETSTRIPE: {
                 struct lov_user_md lum, *lump = (struct lov_user_md *)arg;
                 int rc = 0;
+                int set_default = 0;
 
                 LASSERT(sizeof(lum) == sizeof(*lump));
                 LASSERT(sizeof(lum.lmm_objects[0]) ==
@@ -549,7 +698,10 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
                 if (rc)
                         return(-EFAULT);
 
-                rc = ll_dir_setstripe(inode, &lum);
+                if (inode->i_sb->s_root == file->f_dentry)
+                        set_default = 1;
+
+                rc = ll_dir_setstripe(inode, &lum, set_default);
 
                 return rc;
         }
@@ -572,7 +724,7 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
                         if (IS_ERR(filename))
                                 RETURN(PTR_ERR(filename));
 
-                        rc = ll_lov_getstripe_ea_info(inode, filename, &lmm, 
+                        rc = ll_lov_getstripe_ea_info(inode, filename, &lmm,
                                                       &lmmsize, &request);
                 } else {
                         rc = ll_dir_getstripe(inode, &lmm, &lmmsize, &request);
@@ -589,7 +741,7 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
                 }
 
                 if (rc < 0) {
-                        if (rc == -ENODATA && (cmd == IOC_MDC_GETFILEINFO || 
+                        if (rc == -ENODATA && (cmd == IOC_MDC_GETFILEINFO ||
                                                cmd == LL_IOC_MDC_GETINFO))
                                 GOTO(skip_lmm, rc = 0);
                         else
@@ -792,7 +944,7 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
                 OBD_FREE_PTR(check);
                 RETURN(rc);
         }
-#if HAVE_QUOTA_SUPPORT
+#ifdef HAVE_QUOTA_SUPPORT
         case OBD_IOC_QUOTACTL: {
                 struct if_quotactl *qctl;
                 struct obd_quotactl *oqctl;
@@ -904,6 +1056,7 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
                 RETURN(rc);
         }
 #endif /* HAVE_QUOTA_SUPPORT */
+        case OBD_IOC_GETNAME_OLD:
         case OBD_IOC_GETNAME: {
                 struct obd_device *obd = class_exp2obd(sbi->ll_osc_exp);
                 if (!obd)
@@ -918,21 +1071,9 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
         }
 }
 
-int ll_dir_open(struct inode *inode, struct file *file)
-{
-        ENTRY;
-        RETURN(ll_file_open(inode, file));
-}
-
-int ll_dir_release(struct inode *inode, struct file *file)
-{
-        ENTRY;
-        RETURN(ll_file_release(inode, file));
-}
-
 struct file_operations ll_dir_operations = {
-        .open     = ll_dir_open,
-        .release  = ll_dir_release,
+        .open     = ll_file_open,
+        .release  = ll_file_release,
         .read     = generic_read_dir,
         .readdir  = ll_readdir,
         .ioctl    = ll_dir_ioctl
