@@ -18,7 +18,7 @@
  *   along with Lustre; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-#ifdef HAVE_KERNEL_CONFIG_H
+#ifndef AUTOCONF_INCLUDED
 #include <linux/config.h>
 #endif
 #include <linux/kernel.h>
@@ -379,6 +379,8 @@ struct page *ll_nopage(struct vm_area_struct *vma, unsigned long address,
                 RETURN(NULL);
         }
 
+        ll_clear_file_contended(inode);
+
         /* start and end the lock on the first and last bytes in the page */
         policy_from_vma(&policy, vma, address, CFS_PAGE_SIZE);
 
@@ -412,16 +414,22 @@ struct page *ll_nopage(struct vm_area_struct *vma, unsigned long address,
                 /* XXX change inode size without ll_inode_size_lock() held!
                  *     there is a race condition with truncate path. (see
                  *     ll_extent_lock) */
-               /* region is within kms and, hence, within real file size (A).
+                /* XXX i_size_write() is not used because it is not safe to
+                 *     take the ll_inode_size_lock() due to a potential lock
+                 *     inversion (bug 6077).  And since it's not safe to use
+                 *     i_size_write() without a covering mutex we do the
+                 *     assignment directly.  It is not critical that the
+                 *     size be correct. */
+                /* NOTE: region is within kms and, hence, within real file size (A).
                  * We need to increase i_size to cover the read region so that
                  * generic_file_read() will do its job, but that doesn't mean
                  * the kms size is _correct_, it is only the _minimum_ size.
                  * If someone does a stat they will get the correct size which
                  * will always be >= the kms value here.  b=11081 */
-                if (inode->i_size < kms) {
+                if (i_size_read(inode) < kms) {
                         inode->i_size = kms;
-			 CDEBUG(D_INODE, "ino=%lu, updating i_size %llu\n",
-                               inode->i_ino, inode->i_size);
+                        CDEBUG(D_INODE, "ino=%lu, updating i_size %llu\n",
+                               inode->i_ino, i_size_read(inode));
                 }
                 lov_stripe_unlock(lsm);
         }
@@ -433,8 +441,8 @@ struct page *ll_nopage(struct vm_area_struct *vma, unsigned long address,
         lov_stripe_lock(lsm);
         if (mode == LCK_PW)
                 obd_adjust_kms(ll_i2obdexp(inode), lsm,
-                               min_t(loff_t, policy.l_extent.end,inode->i_size),
-                               0);
+                               min_t(loff_t, policy.l_extent.end + 1,
+                               i_size_read(inode)), 0);
         lov_stripe_unlock(lsm);
 
         /* disable VM_SEQ_READ and use VM_RAND_READ to make sure that
@@ -628,7 +636,7 @@ int ll_file_mmap(struct file * file, struct vm_area_struct * vma)
         int rc;
         ENTRY;
 
-        ll_vfs_ops_tally(ll_i2sbi(file->f_dentry->d_inode), VFS_OPS_MMAP);
+        ll_stats_ops_tally(ll_i2sbi(file->f_dentry->d_inode), LPROC_LL_MAP, 1);
         rc = generic_file_mmap(file, vma);
         if (rc == 0) {
 #if !defined(HAVE_FILEMAP_POPULATE) && \
