@@ -73,12 +73,12 @@ static int ll_test_inode(struct inode *inode, void *opaque)
                 return 0;
 #endif
         if (inode->i_generation != md->body->generation) {
+#ifdef HAVE_EXPORT___IGET
                 if (inode->i_state & (I_FREEING | I_CLEAR))
                         return 0;
                 if (inode->i_nlink == 0)
                         return 0;
 
-#ifdef HAVE_EXPORT___IGET
                 /* add "duplicate" inode into deathrow for destroy */
                 spin_lock(&ll_i2sbi(inode)->ll_deathrow_lock);
                 if (list_empty(&ll_i2info(inode)->lli_dead_list)) {
@@ -150,6 +150,32 @@ struct inode *ll_iget(struct super_block *sb, ino_t hash,
 }
 #endif
 
+static void ll_drop_negative_dentry(struct inode *dir)
+{ 
+        struct dentry *dentry, *tmp_alias, *tmp_subdir;
+
+        spin_lock(&dcache_lock);
+restart:
+        list_for_each_entry_safe(dentry, tmp_alias,
+                                 &dir->i_dentry,d_alias) {
+                if (!list_empty(&dentry->d_subdirs)) {
+                        struct dentry *child;
+                        list_for_each_entry_safe(child, tmp_subdir,
+                                                 &dentry->d_subdirs,
+                                                 d_child) {
+                                /* XXX Print some debug here? */
+                                if (!child->d_inode)
+                                /* Negative dentry. If we were
+                                   dropping dcache lock, go
+                                   throught the list again */
+                                        if (ll_drop_dentry(child))
+                                                goto restart;
+                        }
+                }
+        }
+        spin_unlock(&dcache_lock);
+}
+
 int ll_mdc_blocking_ast(struct ldlm_lock *lock, struct ldlm_lock_desc *desc,
                         void *data, int flag)
 {
@@ -174,6 +200,17 @@ int ll_mdc_blocking_ast(struct ldlm_lock *lock, struct ldlm_lock_desc *desc,
                 if (inode == NULL)
                         break;
 
+                LASSERT(lock->l_flags & LDLM_FL_CANCELING);
+                if ((bits & MDS_INODELOCK_LOOKUP) &&
+                    ll_have_md_lock(inode, MDS_INODELOCK_LOOKUP))
+                        bits &= ~MDS_INODELOCK_LOOKUP;
+                if ((bits & MDS_INODELOCK_UPDATE) &&
+                    ll_have_md_lock(inode, MDS_INODELOCK_UPDATE))
+                        bits &= ~MDS_INODELOCK_UPDATE;
+                if ((bits & MDS_INODELOCK_OPEN) &&
+                    ll_have_md_lock(inode, MDS_INODELOCK_OPEN))
+                        bits &= ~MDS_INODELOCK_OPEN;
+                
                 if (lock->l_resource->lr_name.name[0] != inode->i_ino ||
                     lock->l_resource->lr_name.name[1] != inode->i_generation) {
                         LDLM_ERROR(lock, "data mismatch with ino %lu/%u (%p)",
@@ -208,48 +245,10 @@ int ll_mdc_blocking_ast(struct ldlm_lock *lock, struct ldlm_lock_desc *desc,
 
                 if (S_ISDIR(inode->i_mode) &&
                      (bits & MDS_INODELOCK_UPDATE)) {
-                        struct dentry *dentry, *tmp, *dir;
-                        struct list_head *list;
-                        int alias_counter = 0;
-
                         CDEBUG(D_INODE, "invalidating inode %lu\n",
                                inode->i_ino);
                         truncate_inode_pages(inode->i_mapping, 0);
-
-                        /* Drop possible cached negative dentries */
-                        list = &inode->i_dentry;
-                        dir = NULL;
-                        spin_lock(&dcache_lock);
-
-                        /* It is possible to have several dentries (with
-                           racer?) */
-                        while ((list = list->next) != &inode->i_dentry) {
-                                tmp = list_entry(list, struct dentry, d_alias);
-                                if (!list_empty(&tmp->d_subdirs))
-                                        dir = tmp;
-                                alias_counter ++;
-                        }
-                        
-                        if (alias_counter > 1)
-                                CWARN("More than 1 alias dir %lu alias %d\n",
-                                       inode->i_ino, alias_counter);
-
-                        if (dir) {
-restart:
-                                list_for_each_entry_safe(dentry, tmp,
-                                                         &dir->d_subdirs,
-                                                         d_child)
-                                {
-                                        /* XXX Print some debug here? */
-                                        if (!dentry->d_inode)
-                                                /* Negative dentry. If we were
-                                                   dropping dcache lock, go
-                                                   throught the list again */
-                                                if (ll_drop_dentry(dentry))
-                                                        goto restart;
-                                }
-                        }
-                        spin_unlock(&dcache_lock);
+                        ll_drop_negative_dentry(inode);
                 }
 
                 if (inode->i_sb->s_root &&
@@ -318,26 +317,27 @@ void ll_i2gids(__u32 *suppgids, struct inode *i1, struct inode *i2)
         }
 }
 
-int ll_prepare_mdc_op_data(struct mdc_op_data *data, struct inode *i1,
+int ll_prepare_mdc_op_data(struct mdc_op_data *op_data, struct inode *i1,
                             struct inode *i2, const char *name, int namelen,
-                            int mode)
+                            int mode, void *data)
 {
         LASSERT(i1);
 
         if (namelen > ll_i2sbi(i1)->ll_namelen)
                 return -ENAMETOOLONG;
-        ll_i2gids(data->suppgids, i1, i2);
-        ll_inode2fid(&data->fid1, i1);
+        ll_i2gids(op_data->suppgids, i1, i2);
+        ll_inode2fid(&op_data->fid1, i1);
 
         if (i2)
-                ll_inode2fid(&data->fid2, i2);
+                ll_inode2fid(&op_data->fid2, i2);
         else
-                memset(&data->fid2, 0, sizeof(data->fid2));
+                memset(&op_data->fid2, 0, sizeof(op_data->fid2));
 
-        data->name = name;
-        data->namelen = namelen;
-        data->create_mode = mode;
-        data->mod_time = CURRENT_SECONDS;
+        op_data->name = name;
+        op_data->namelen = namelen;
+        op_data->create_mode = mode;
+        op_data->mod_time = CURRENT_SECONDS;
+        op_data->data = data;
 
         return 0;
 }
@@ -367,13 +367,13 @@ static void ll_d_add(struct dentry *de, struct inode *inode)
         __d_rehash(de, 0);
 }
 
-/* Search "inode"'s alias list for a dentry that has the same name and parent as
- * de.  If found, return it.  If not found, return de.
- * Lustre can`t be use d_add_unique because don`t unhash aliases for directory in
- * ll_revalidate_it. After revaliadate inode will be have hashed aliases and it`s
- * trigered BUG_ON in d_instantiate_unique (bug #10954).
+/* Search "inode"'s alias list for a dentry that has the same name and parent
+ * as de.  If found, return it.  If not found, return de.
+ * Lustre can't use d_add_unique because don't unhash aliases for directory
+ * in ll_revalidate_it.  After revaliadate inode will be have hashed aliases
+ * and it triggers BUG_ON in d_instantiate_unique (bug #10954).
  */
-struct dentry *ll_find_alias(struct inode *inode, struct dentry *de)
+static struct dentry *ll_find_alias(struct inode *inode, struct dentry *de)
 {
         struct list_head *tmp;
         struct dentry *dentry;
@@ -411,7 +411,7 @@ struct dentry *ll_find_alias(struct inode *inode, struct dentry *de)
                 dget_locked(dentry);
                 lock_dentry(dentry);
                 __d_drop(dentry);
-#ifdef LUSTRE_KERNEL_VERSION
+#ifdef DCACHE_LUSTRE_INVALID
                 dentry->d_flags &= ~DCACHE_LUSTRE_INVALID;
 #endif
                 unlock_dentry(dentry);
@@ -424,8 +424,8 @@ struct dentry *ll_find_alias(struct inode *inode, struct dentry *de)
                 return dentry;
         }
         if (last_discon) {
-                 CDEBUG(D_DENTRY,"Reuse disconected dentry %p inode %p - refc %d\n",
-                        last_discon, last_discon->d_inode,
+                 CDEBUG(D_DENTRY, "Reuse disconnected dentry %p inode %p "
+                        "refc %d\n", last_discon, last_discon->d_inode,
                         atomic_read(&last_discon->d_count));
                  dget_locked(last_discon);
                  spin_unlock(&dcache_lock);
@@ -442,7 +442,7 @@ struct dentry *ll_find_alias(struct inode *inode, struct dentry *de)
         return de;
 }
 
-static int lookup_it_finish(struct ptlrpc_request *request, int offset,
+int lookup_it_finish(struct ptlrpc_request *request, int offset,
                             struct lookup_intent *it, void *data)
 {
         struct it_cb_data *icbd = data;
@@ -488,9 +488,8 @@ static int lookup_it_finish(struct ptlrpc_request *request, int offset,
                         /* We do not want to hash the dentry if don`t have a
                          * lock, but if this dentry is later used in d_move,
                          * we'd hit uninitialised list head d_hash, so we just
-                         * do this to init d_hash field but leave dentry unhashed.
-                         * (bug 10796).
-                         */
+                         * do this to init d_hash field but leave dentry
+                         * unhashed. (bug 10796). */
                         d_rehash(*de);
                         d_drop(*de);
                 }
@@ -531,11 +530,20 @@ static struct dentry *ll_lookup_it(struct inode *parent, struct dentry *dentry,
                         RETURN(ERR_PTR(rc));
         }
 
-        icbd.icbd_childp = &dentry;
+        if (it->it_op == IT_GETATTR) {
+                rc = ll_statahead_enter(parent, &dentry, 1);
+                if (rc >= 0) {
+                        ll_statahead_exit(dentry, rc);
+                        if (rc == 1)
+                                RETURN(retval = dentry);
+                }
+        }
+
         icbd.icbd_parent = parent;
+        icbd.icbd_childp = &dentry;
 
         rc = ll_prepare_mdc_op_data(&op_data, parent, NULL, dentry->d_name.name,
-                                    dentry->d_name.len, lookup_flags);
+                                    dentry->d_name.len, lookup_flags, NULL);
         if (rc)
                 RETURN(ERR_PTR(rc));
 
@@ -571,7 +579,7 @@ static struct dentry *ll_lookup_it(struct inode *parent, struct dentry *dentry,
 }
 
 #if (LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0))
-#ifdef LUSTRE_KERNEL_VERSION
+#ifdef HAVE_VFS_INTENT_PATCHES
 static struct dentry *ll_lookup_nd(struct inode *parent, struct dentry *dentry,
                                    struct nameidata *nd)
 {
@@ -811,7 +819,7 @@ static void ll_update_times(struct ptlrpc_request *request, int offset,
 }
 
 static int ll_new_node(struct inode *dir, struct qstr *name,
-                       char *tgt, int mode,
+                       const char *tgt, int mode,
                        int rdev, struct dentry *dchild)
 {
         struct ptlrpc_request *request = NULL;
@@ -826,7 +834,7 @@ static int ll_new_node(struct inode *dir, struct qstr *name,
                 tgt_len = strlen(tgt)+1;
 
         err = ll_prepare_mdc_op_data(&op_data, dir, NULL, name->name,
-                                     name->len, 0);
+                                     name->len, 0, NULL);
         if (err)
                 GOTO(err_exit, err);
 
@@ -887,7 +895,7 @@ static int ll_mknod_generic(struct inode *dir, struct qstr *name, int mode,
 }
 
 #if (LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0))
-#ifndef LUSTRE_KERNEL_VERSION
+#ifndef HAVE_VFS_INTENT_PATCHES
 static int ll_create_nd(struct inode *dir, struct dentry *dentry, int mode, struct nameidata *nd)
 {
         struct lookup_intent *it = ll_d2d(dentry)->lld_it;
@@ -963,7 +971,7 @@ static int ll_link_generic(struct inode *src,  struct inode *dir,
                dir->i_generation, dir, name->len, name->name);
 
         err = ll_prepare_mdc_op_data(&op_data, src, dir, name->name,
-                                     name->len, 0);
+                                     name->len, 0, NULL);
         if (err)
                 GOTO(out, err);
         err = mdc_link(sbi->ll_mdc_exp, &op_data, &request);
@@ -997,11 +1005,27 @@ static int ll_mkdir_generic(struct inode *dir, struct qstr *name, int mode,
         RETURN(err);
 }
 
+/* Try to find the child dentry by its name.
+   If found, put the result fid into @fid. */
+static void ll_get_child_fid(struct inode * dir, struct qstr *name,
+                             struct ll_fid *fid)
+{
+        struct dentry *parent, *child;
+        
+        parent = list_entry(dir->i_dentry.next, struct dentry, d_alias);
+        child = d_lookup(parent, name);
+        if (child) {
+                if (child->d_inode)
+                        ll_inode2fid(fid, child->d_inode);
+                dput(child);
+        }
+}
+
 static int ll_rmdir_generic(struct inode *dir, struct dentry *dparent,
                             struct qstr *name)
 {
         struct ptlrpc_request *request = NULL;
-        struct mdc_op_data op_data;
+        struct mdc_op_data op_data = {{0}};
         struct dentry *dentry;
         int rc;
         ENTRY;
@@ -1021,9 +1045,11 @@ static int ll_rmdir_generic(struct inode *dir, struct dentry *dparent,
         }
 
         rc = ll_prepare_mdc_op_data(&op_data, dir, NULL, name->name,
-                                    name->len, S_IFDIR);
+                                    name->len, S_IFDIR, NULL);
         if (rc)
                 GOTO(out, rc);
+        
+        ll_get_child_fid(dir, name, &op_data.fid3);
         rc = mdc_unlink(ll_i2sbi(dir)->ll_mdc_exp, &op_data, &request);
         if (rc)
                 GOTO(out, rc);
@@ -1079,7 +1105,7 @@ int ll_objects_destroy(struct ptlrpc_request *request, struct inode *dir)
         if (rc)
                 GOTO(out_free_memmd, rc);
 
-        oa = obdo_alloc();
+        OBDO_ALLOC(oa);
         if (oa == NULL)
                 GOTO(out_free_memmd, rc = -ENOMEM);
 
@@ -1100,7 +1126,7 @@ int ll_objects_destroy(struct ptlrpc_request *request, struct inode *dir)
         }
 
         rc = obd_destroy(ll_i2obdexp(dir), oa, lsm, &oti, ll_i2mdcexp(dir));
-        obdo_free(oa);
+        OBDO_FREE(oa);
         if (rc)
                 CERROR("obd destroy objid "LPX64" error %d\n",
                        lsm->lsm_object_id, rc);
@@ -1113,7 +1139,7 @@ int ll_objects_destroy(struct ptlrpc_request *request, struct inode *dir)
 static int ll_unlink_generic(struct inode * dir, struct qstr *name)
 {
         struct ptlrpc_request *request = NULL;
-        struct mdc_op_data op_data;
+        struct mdc_op_data op_data = {{0}};
         int rc;
         ENTRY;
 
@@ -1121,9 +1147,11 @@ static int ll_unlink_generic(struct inode * dir, struct qstr *name)
                name->len, name->name, dir->i_ino, dir->i_generation, dir);
 
         rc = ll_prepare_mdc_op_data(&op_data, dir, NULL, name->name,
-                                    name->len, 0);
+                                    name->len, 0, NULL);
         if (rc)
                 GOTO(out, rc);
+
+        ll_get_child_fid(dir, name, &op_data.fid3);
         rc = mdc_unlink(ll_i2sbi(dir)->ll_mdc_exp, &op_data, &request);
         if (rc)
                 GOTO(out, rc);
@@ -1144,7 +1172,7 @@ static int ll_rename_generic(struct inode *src, struct qstr *src_name,
 {
         struct ptlrpc_request *request = NULL;
         struct ll_sb_info *sbi = ll_i2sbi(src);
-        struct mdc_op_data op_data;
+        struct mdc_op_data op_data = {{0}};
         int err;
 
         ENTRY;
@@ -1153,9 +1181,12 @@ static int ll_rename_generic(struct inode *src, struct qstr *src_name,
                src->i_ino, src->i_generation, src, tgt_name->len,
                tgt_name->name, tgt->i_ino, tgt->i_generation, tgt);
 
-        err = ll_prepare_mdc_op_data(&op_data, src, tgt, NULL, 0, 0);
+        err = ll_prepare_mdc_op_data(&op_data, src, tgt, NULL, 0, 0, NULL);
         if (err)
                 GOTO(out, err);
+        
+        ll_get_child_fid(src, src_name, &op_data.fid3);
+        ll_get_child_fid(tgt, tgt_name, &op_data.fid4);
         err = mdc_rename(sbi->ll_mdc_exp, &op_data,
                          src_name->name, src_name->len,
                          tgt_name->name, tgt_name->len, &request);
@@ -1174,7 +1205,7 @@ out:
         return(err);
 }
 
-#ifdef LUSTRE_KERNEL_VERSION
+#ifdef HAVE_VFS_INTENT_PATCHES
 static int ll_mknod_raw(struct nameidata *nd, int mode, dev_t rdev)
 {
         return ll_mknod_generic(nd->dentry->d_inode, &nd->last, mode,rdev,NULL);
@@ -1247,7 +1278,7 @@ static int ll_rename(struct inode *old_dir, struct dentry *old_dentry,
 #endif
 
 struct inode_operations ll_dir_inode_operations = {
-#ifdef LUSTRE_KERNEL_VERSION
+#ifdef HAVE_VFS_INTENT_PATCHES
         .link_raw           = ll_link_raw,
         .unlink_raw         = ll_unlink_raw,
         .symlink_raw        = ll_symlink_raw,
@@ -1284,7 +1315,7 @@ struct inode_operations ll_dir_inode_operations = {
 };
 
 struct inode_operations ll_special_inode_operations = {
-#ifdef LUSTRE_KERNEL_VERSION
+#ifdef HAVE_VFS_INTENT_PATCHES
         .setattr_raw    = ll_setattr_raw,
 #endif
         .setattr        = ll_setattr,
