@@ -108,6 +108,7 @@ static
 int mdc_getattr_common(struct obd_export *exp, unsigned int ea_size, 
                        unsigned int acl_size, struct ptlrpc_request *req)
 {
+        struct obd_device *obddev = class_exp2obd(exp);
         struct mds_body *body;
         void *eadata;
         int size[4] = { sizeof(struct ptlrpc_body), sizeof(*body) };
@@ -128,7 +129,9 @@ int mdc_getattr_common(struct obd_export *exp, unsigned int ea_size,
 
         ptlrpc_req_set_repsize(req, bufcount, size);
 
+        mdc_enter_request(&obddev->u.cli);
         rc = ptlrpc_queue_wait(req);
+        mdc_exit_request(&obddev->u.cli);
         if (rc != 0)
                 RETURN (rc);
 
@@ -236,6 +239,7 @@ int mdc_xattr_common(struct obd_export *exp, struct ll_fid *fid,
                      const char *input, int input_size, int output_size,
                      int flags, struct ptlrpc_request **request)
 {
+        struct obd_device *obddev = class_exp2obd(exp);
         struct ptlrpc_request *req;
         int size[4] = { sizeof(struct ptlrpc_body), sizeof(struct mds_body) };
         // int size[3] = {sizeof(struct mds_body)}, bufcnt = 1;
@@ -287,11 +291,15 @@ int mdc_xattr_common(struct obd_export *exp, struct ll_fid *fid,
         /* make rpc */
         if (opcode == MDS_SETXATTR)
                 mdc_get_rpc_lock(exp->exp_obd->u.cli.cl_rpc_lock, NULL);
+        else
+                mdc_enter_request(&obddev->u.cli);
 
         rc = ptlrpc_queue_wait(req);
 
         if (opcode == MDS_SETXATTR)
                 mdc_put_rpc_lock(exp->exp_obd->u.cli.cl_rpc_lock, NULL);
+        else
+                mdc_exit_request(&obddev->u.cli);
 
         if (rc != 0)
                 GOTO(err_out, rc);
@@ -352,7 +360,7 @@ void mdc_store_inode_generation(struct ptlrpc_request *req, int reqoff,
                 LBUG();
         }
 
-        DEBUG_REQ(D_HA, req, "storing generation %u for ino "LPU64,
+        DEBUG_REQ(D_INODE, req, "storing generation %u for ino "LPU64,
                   rec->cr_replayfid.generation, rec->cr_replayfid.id);
 }
 
@@ -418,27 +426,47 @@ int mdc_req2lustre_md(struct ptlrpc_request *req, int offset,
                 int lmmsize;
                 struct lov_mds_md *lmm;
 
-                LASSERT(S_ISREG(md->body->mode));
+                if (!S_ISREG(md->body->mode)) {
+                        CERROR("OBD_MD_FLEASIZE set, should be a regular file, "
+                               "but is not\n");
+                        GOTO(err_out, rc = -EPROTO);
+                }
 
                 if (md->body->eadatasize == 0) {
                         CERROR ("OBD_MD_FLEASIZE set, but eadatasize 0\n");
-                        RETURN(-EPROTO);
+                        GOTO(err_out, rc = -EPROTO);
                 }
                 lmmsize = md->body->eadatasize;
                 lmm = lustre_msg_buf(req->rq_repmsg, offset, lmmsize);
-                LASSERT (lmm != NULL);
+                if (!lmm) {
+                        CERROR ("incorrect message: lmm == 0\n");
+                        GOTO(err_out, rc = -EPROTO);
+                }
                 LASSERT_REPSWABBED(req, offset);
 
                 rc = obd_unpackmd(exp, &md->lsm, lmm, lmmsize);
                 if (rc < 0)
-                        RETURN(rc);
+                        GOTO(err_out, rc);
 
-                LASSERT (rc >= sizeof (*md->lsm));
+                if (rc < sizeof(*md->lsm)) {
+                        CERROR ("lsm size too small:  rc < sizeof (*md->lsm) "
+                                "(%d < "LPSZ")\n", rc, sizeof(*md->lsm));
+                        GOTO(err_out, rc = -EPROTO);
+                }
                 rc = 0;
 
                 offset++;
         }
 
+        if (md->body->valid & OBD_MD_FLDIREA) {
+                if(!S_ISDIR(md->body->mode)) {
+                        CERROR("OBD_MD_FLDIREA set, should be a directory, but "
+                               "is not\n");
+                        GOTO(err_out, rc = -EPROTO);
+                }
+                offset++;
+        }
+
         /* for ACL, it's possible that FLACL is set but aclsize is zero.
          * only when aclsize != 0 there's an actual segment for ACL in
          * reply buffer.
@@ -512,7 +540,7 @@ static void mdc_replay_open(struct ptlrpc_request *req)
                 struct lustre_handle *file_fh;
                 LASSERT(och->och_magic == OBD_CLIENT_HANDLE_MAGIC);
                 file_fh = &och->och_fh;
-                CDEBUG(D_HA, "updating handle from "LPX64" to "LPX64"\n",
+                CDEBUG(D_RPCTRACE, "updating handle from "LPX64" to "LPX64"\n",
                        file_fh->cookie, body->handle.cookie);
                 memcpy(&old, file_fh, sizeof(old));
                 memcpy(file_fh, &body->handle, sizeof(*file_fh));
@@ -526,7 +554,7 @@ static void mdc_replay_open(struct ptlrpc_request *req)
                                             sizeof(*close_body));
                 if (och != NULL)
                         LASSERT(!memcmp(&old, &close_body->handle, sizeof old));
-                DEBUG_REQ(D_HA, close_req, "updating close body with new fh");
+                DEBUG_REQ(D_RPCTRACE, close_req, "updating close with new fh");
                 memcpy(&close_body->handle, &body->handle,
                        sizeof(close_body->handle));
         }
@@ -558,11 +586,19 @@ void mdc_set_open_replay_data(struct obd_client_handle *och,
                         return;
                 }
 
+                spin_lock(&open_req->rq_lock);
+                if (!open_req->rq_replay) {
+                        OBD_FREE(mod, sizeof(*mod));
+                        spin_unlock(&open_req->rq_lock);
+                        return;
+                }
+
                 och->och_mod = mod;
                 mod->mod_och = och;
                 mod->mod_open_req = open_req;
                 open_req->rq_cb_data = mod;
                 open_req->rq_commit_cb = mdc_commit_open;
+                spin_unlock(&open_req->rq_lock);
         }
 
         memcpy(&rec->cr_replayfid, &body->fid1, sizeof rec->cr_replayfid);
@@ -573,7 +609,7 @@ void mdc_set_open_replay_data(struct obd_client_handle *och,
                 LBUG();
         }
 
-        DEBUG_REQ(D_HA, open_req, "set up replay data");
+        DEBUG_REQ(D_RPCTRACE, open_req, "set up replay data");
 }
 
 void mdc_clear_open_replay_data(struct obd_client_handle *och)
@@ -596,7 +632,7 @@ static void mdc_commit_close(struct ptlrpc_request *req)
         struct ptlrpc_request *open_req;
         struct obd_import *imp = req->rq_import;
 
-        DEBUG_REQ(D_HA, req, "close req committed");
+        DEBUG_REQ(D_RPCTRACE, req, "close req committed");
         if (mod == NULL)
                 return;
 
@@ -609,7 +645,7 @@ static void mdc_commit_close(struct ptlrpc_request *req)
         LASSERT(open_req != LP_POISON);
         LASSERT(open_req->rq_type != LI_POISON);
 
-        DEBUG_REQ(D_HA, open_req, "open req balanced");
+        DEBUG_REQ(D_RPCTRACE, open_req, "open req balanced");
         LASSERT(open_req->rq_transno != 0);
         LASSERT(open_req->rq_import == imp);
 
@@ -651,18 +687,16 @@ int mdc_close(struct obd_export *exp, struct obdo *oa,
         mod = och->och_mod;
         if (likely(mod != NULL)) {
                 if (mod->mod_open_req->rq_type == LI_POISON) {
-                        /* FIXME This should be an ASSERT, but until we
-                           figure out why it can be poisoned here, give
-                           a reasonable return. bug 6155 */
                         CERROR("LBUG POISONED open %p!\n", mod->mod_open_req);
+                        LBUG();
                         ptlrpc_req_finished(req);
                         req = NULL;
                         GOTO(out, rc = -EIO);
                 }
                 mod->mod_close_req = req;
-                DEBUG_REQ(D_HA, mod->mod_open_req, "matched open");
+                DEBUG_REQ(D_RPCTRACE, mod->mod_open_req, "matched open");
         } else {
-                CDEBUG(D_HA, "couldn't find open req; expecting close error\n");
+                CDEBUG(D_RPCTRACE, "couldn't find open req; expecting error\n");
         }
 
         mdc_close_pack(req, REQ_REC_OFF, oa, oa->o_valid, och);
@@ -677,7 +711,7 @@ int mdc_close(struct obd_export *exp, struct obdo *oa,
         mdc_put_rpc_lock(obd->u.cli.cl_close_lock, NULL);
 
         if (req->rq_repmsg == NULL) {
-                CDEBUG(D_HA, "request failed to send: %p, %d\n", req,
+                CDEBUG(D_RPCTRACE, "request failed to send: %p, %d\n", req,
                        req->rq_status);
                 if (rc == 0)
                         rc = req->rq_status ? req->rq_status : -EIO;
@@ -822,6 +856,7 @@ static int mdc_iocontrol(unsigned int cmd, struct obd_export *exp, int len,
         case OBD_IOC_PARSE: {
                 ctxt = llog_get_context(exp->exp_obd, LLOG_CONFIG_REPL_CTXT);
                 rc = class_config_parse_llog(ctxt, data->ioc_inlbuf1, NULL);
+                llog_ctxt_put(ctxt);
                 GOTO(out, rc);
         }
 #ifdef __KERNEL__
@@ -829,7 +864,7 @@ static int mdc_iocontrol(unsigned int cmd, struct obd_export *exp, int len,
         case OBD_IOC_LLOG_PRINT: {
                 ctxt = llog_get_context(obd, LLOG_CONFIG_REPL_CTXT);
                 rc = llog_ioctl(ctxt, cmd, data);
-
+                llog_ctxt_put(ctxt);
                 GOTO(out, rc);
         }
 #endif
@@ -861,7 +896,10 @@ int mdc_set_info_async(struct obd_export *exp, obd_count keylen,
         if (KEY_IS(KEY_INIT_RECOV)) {
                 if (vallen != sizeof(int))
                         RETURN(-EINVAL);
+                spin_lock(&imp->imp_lock);
                 imp->imp_initial_recov = *(int *)val;
+                spin_unlock(&imp->imp_lock);
+
                 CDEBUG(D_HA, "%s: set imp_initial_recov = %d\n",
                        exp->exp_obd->obd_name, imp->imp_initial_recov);
                 RETURN(0);
@@ -870,9 +908,13 @@ int mdc_set_info_async(struct obd_export *exp, obd_count keylen,
         if (KEY_IS(KEY_INIT_RECOV_BACKUP)) {
                 if (vallen != sizeof(int))
                         RETURN(-EINVAL);
+
+                spin_lock(&imp->imp_lock);
                 imp->imp_initial_recov_bk = *(int *)val;
                 if (imp->imp_initial_recov_bk)
                         imp->imp_initial_recov = 1;
+                spin_unlock(&imp->imp_lock);
+
                 CDEBUG(D_HA, "%s: set imp_initial_recov_bk = %d\n",
                        exp->exp_obd->obd_name, imp->imp_initial_recov_bk);
                 RETURN(0);
@@ -1095,6 +1137,7 @@ static int mdc_import_event(struct obd_device *obd, struct obd_import *imp,
 
         switch (event) {
         case IMP_EVENT_DISCON: {
+                ptlrpc_import_setasync(imp, -obd->obd_namespace->ns_max_unused);
                 break;
         }
         case IMP_EVENT_INACTIVE: {
@@ -1113,6 +1156,7 @@ static int mdc_import_event(struct obd_device *obd, struct obd_import *imp,
                 break;
         }
         case IMP_EVENT_OCD:
+                ptlrpc_import_setasync(imp, obd->obd_namespace->ns_max_unused);
                 break;
 
         default:
@@ -1271,6 +1315,7 @@ static int mdc_llog_init(struct obd_device *obd, struct obd_device *tgt,
         if (rc == 0) {
                 ctxt = llog_get_context(obd, LLOG_CONFIG_REPL_CTXT);
                 ctxt->loc_imp = obd->u.cli.cl_import;
+                llog_ctxt_put(ctxt);
         }
 
         rc = llog_setup(obd, LLOG_LOVEA_REPL_CTXT, tgt, 0, NULL,
@@ -1278,6 +1323,7 @@ static int mdc_llog_init(struct obd_device *obd, struct obd_device *tgt,
         if (rc == 0) {
                 ctxt = llog_get_context(obd, LLOG_LOVEA_REPL_CTXT);
                 ctxt->loc_imp = obd->u.cli.cl_import;
+                llog_ctxt_put(ctxt);
         }
 
         RETURN(rc);
