@@ -55,11 +55,12 @@ static int mds_export_stats_init(struct obd_device *obd, struct obd_export *exp)
         int rc, num_stats;
 
         rc = lprocfs_exp_setup(exp);
-        if (rc) 
+        if (rc)
                 return rc;
         num_stats = (sizeof(*obd->obd_type->typ_ops) / sizeof(void *)) +
                      LPROC_MDS_LAST - 1;
-        exp->exp_ops_stats = lprocfs_alloc_stats(num_stats);
+        exp->exp_ops_stats = lprocfs_alloc_stats(num_stats,
+                                                 LPROCFS_STATS_FLAG_NOPERCPU);
         if (exp->exp_ops_stats == NULL)
                 return -ENOMEM;
         lprocfs_init_ops_stats(LPROC_MDS_LAST, exp->exp_ops_stats);
@@ -142,8 +143,11 @@ int mds_client_add(struct obd_device *obd, struct obd_export *exp,
                 } else {
                         rc = fsfilt_add_journal_cb(obd, 0, handle,
                                                    target_client_add_cb, exp);
-                        if (rc == 0)
+                        if (rc == 0) {
+                                spin_lock(&exp->exp_lock);
                                 exp->exp_need_sync = 1;
+                                spin_unlock(&exp->exp_lock);
+                        }
                         rc = fsfilt_write_record(obd, file, med->med_mcd,
                                                  sizeof(*med->med_mcd),
                                                  &off, rc /* sync if no cb */);
@@ -209,7 +213,8 @@ int mds_client_free(struct obd_export *exp)
                 push_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
                 rc = fsfilt_write_record(obd, mds->mds_rcvd_filp, &zero_mcd,
                                          sizeof(zero_mcd), &off,
-                                         !exp->exp_libclient);
+                                         (!exp->exp_libclient ||
+                                          exp->exp_need_sync));
                 pop_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
 
                 CDEBUG(rc == 0 ? D_INFO : D_ERROR,
@@ -252,7 +257,7 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
         struct lr_server_data *lsd;
         struct mds_client_data *mcd = NULL;
         loff_t off = 0;
-        unsigned long last_rcvd_size = file->f_dentry->d_inode->i_size;
+        unsigned long last_rcvd_size = i_size_read(file->f_dentry->d_inode);
         __u64 mount_count;
         int cl_idx, rc = 0;
         ENTRY;
@@ -293,15 +298,15 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
                         GOTO(err_msd, rc);
                 }
                 if (strcmp(lsd->lsd_uuid, obd->obd_uuid.uuid) != 0) {
-                        LCONSOLE_ERROR("Trying to start OBD %s using the wrong"
-                                       " disk %s. Were the /dev/ assignments "
-                                       "rearranged?\n",
-                                       obd->obd_uuid.uuid, lsd->lsd_uuid);
+                        LCONSOLE_ERROR_MSG(0x157, "Trying to start OBD %s using"
+                                           " the wrong disk %s. Were the /dev/ "
+                                           "assignments rearranged?\n",
+                                           obd->obd_uuid.uuid, lsd->lsd_uuid);
                         GOTO(err_msd, rc = -EINVAL);
                 }
                 /* COMPAT_146 */
                 /* Assume old last_rcvd format unless I_C_LR is set */
-                if (!(lsd->lsd_feature_incompat & 
+                if (!(lsd->lsd_feature_incompat &
                       cpu_to_le32(OBD_INCOMPAT_COMMON_LR)))
                         lsd->lsd_mount_count = lsd->lsd_compat14;
                 /* end COMPAT_146 */
@@ -323,7 +328,7 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
         }
 
         lsd->lsd_feature_compat = cpu_to_le32(OBD_COMPAT_MDT);
-        
+
         mds->mds_last_transno = le64_to_cpu(lsd->lsd_last_transno);
 
         CDEBUG(D_INODE, "%s: server last_transno: "LPU64"\n",
@@ -396,22 +401,33 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
                        le64_to_cpu(mcd->mcd_last_xid));
 
                 exp = class_new_export(obd, (struct obd_uuid *)mcd->mcd_uuid);
-                if (IS_ERR(exp))
-                        GOTO(err_client, rc = PTR_ERR(exp));
+                if (IS_ERR(exp)) {
+                        if (PTR_ERR(exp) == -EALREADY) {
+                                /* export already exists, zero out this one */
+                                mcd->mcd_uuid[0] = '\0';
+                        } else {
+                                GOTO(err_client, rc = PTR_ERR(exp));
+                        }
+                } else {
+                        med = &exp->exp_mds_data;
+                        med->med_mcd = mcd;
+                        rc = mds_client_add(obd, exp, cl_idx);
+                        /* can't fail for existing client */
+                        LASSERTF(rc == 0, "rc = %d\n", rc);
 
-                med = &exp->exp_mds_data;
-                med->med_mcd = mcd;
-                rc = mds_client_add(obd, exp, cl_idx);
-                LASSERTF(rc == 0, "rc = %d\n", rc); /* can't fail existing */
+                        mcd = NULL;
 
+                        spin_lock(&exp->exp_lock);
+                        exp->exp_replay_needed = 1;
+                        exp->exp_connecting = 0;
+                        spin_unlock(&exp->exp_lock);
 
-                mcd = NULL;
-                exp->exp_replay_needed = 1;
-                exp->exp_connecting = 0;
-                obd->obd_recoverable_clients++;
-                obd->obd_max_recoverable_clients++;
-                class_export_put(exp);
+                        obd->obd_recoverable_clients++;
+                        obd->obd_max_recoverable_clients++;
+                        class_export_put(exp);
+                }
 
+                /* Need to check last_rcvd even for duplicated exports. */
                 CDEBUG(D_OTHER, "client at idx %d has last_transno = "LPU64"\n",
                        cl_idx, last_transno);
 
@@ -437,7 +453,7 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
         }
 
         mds->mds_mount_count = mount_count + 1;
-        lsd->lsd_mount_count = lsd->lsd_compat14 = 
+        lsd->lsd_mount_count = lsd->lsd_compat14 =
                 cpu_to_le64(mds->mds_mount_count);
 
         /* save it, so mount count and last_transno is current */
@@ -576,7 +592,7 @@ int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
         file = filp_open(HEALTH_CHECK, O_RDWR | O_CREAT, 0644);
         if (IS_ERR(file)) {
                 rc = PTR_ERR(file);
-                CERROR("cannot open/create %s file: rc = %d\n", HEALTH_CHECK, rc);
+                CERROR("cannot open/create %s file: rc = %d\n",HEALTH_CHECK,rc);
                 GOTO(err_lov_objid, rc = PTR_ERR(file));
         }
         mds->mds_health_check_filp = file;
@@ -594,7 +610,7 @@ err_pop:
         return rc;
 
 err_health_check:
-        if (mds->mds_health_check_filp && 
+        if (mds->mds_health_check_filp &&
             filp_close(mds->mds_health_check_filp, 0))
                 CERROR("can't close %s after error\n", HEALTH_CHECK);
 err_lov_objid:
@@ -647,7 +663,7 @@ int mds_fs_cleanup(struct obd_device *obd)
                 rc = filp_close(mds->mds_health_check_filp, 0);
                 mds->mds_health_check_filp = NULL;
                 if (rc)
-                        CERROR("%s file won't close, rc=%d\n", HEALTH_CHECK, rc);
+                        CERROR("%s file won't close, rc=%d\n", HEALTH_CHECK,rc);
         }
         if (mds->mds_objects_dir != NULL) {
                 l_dput(mds->mds_objects_dir);
@@ -691,7 +707,7 @@ int mds_obd_create(struct obd_export *exp, struct obdo *oa,
         ENTRY;
 
         /* the owner of object file should always be root */
-        ucred.luc_cap = current->cap_effective | CAP_SYS_RESOURCE;
+        cap_raise(ucred.luc_cap, CAP_SYS_RESOURCE);
 
         push_ctxt(&saved, &exp->exp_obd->obd_lvfs_ctxt, &ucred);
 
@@ -777,7 +793,7 @@ int mds_obd_destroy(struct obd_export *exp, struct obdo *oa,
         int err, namelen, rc = 0;
         ENTRY;
 
-        ucred.luc_cap = current->cap_effective | CAP_SYS_RESOURCE;
+        cap_raise(ucred.luc_cap, CAP_SYS_RESOURCE);
         push_ctxt(&saved, &obd->obd_lvfs_ctxt, &ucred);
 
         namelen = ll_fid2str(fidname, oa->o_id, oa->o_generation);
