@@ -138,11 +138,10 @@ static int mgs_setup(struct obd_device *obd, obd_count len, void *buf)
                 GOTO(err_put, rc = PTR_ERR(obd->obd_fsops));
 
         /* namespace for mgs llog */
-        obd->obd_namespace = ldlm_namespace_new("MGS", LDLM_NAMESPACE_SERVER);
-        if (obd->obd_namespace == NULL) {
-                mgs_cleanup(obd);
+        obd->obd_namespace = ldlm_namespace_new("MGS", LDLM_NAMESPACE_SERVER, 
+                                                LDLM_NAMESPACE_MODEST);
+        if (obd->obd_namespace == NULL)
                 GOTO(err_ops, rc = -ENOMEM);
-        }
 
         /* ldlm setup */
         ptlrpc_init_client(LDLM_CB_REQUEST_PORTAL, LDLM_CB_REPLY_PORTAL,
@@ -157,10 +156,6 @@ static int mgs_setup(struct obd_device *obd, obd_count len, void *buf)
                 GOTO(err_ns, rc);
         }
 
-        rc = llog_start_commit_thread();
-        if (rc < 0)
-                GOTO(err_fs, rc);
-
         rc = llog_setup(obd, LLOG_CONFIG_ORIG_CTXT, obd, 0, NULL,
                         &llog_lvfs_ops);
         if (rc)
@@ -255,11 +250,11 @@ static int mgs_cleanup(struct obd_device *obd)
         struct mgs_obd *mgs = &obd->u.mgs;
         ENTRY;
 
-        ping_evictor_stop();
-
         if (mgs->mgs_sb == NULL)
                 RETURN(0);
         
+        ping_evictor_stop();
+
         ptlrpc_unregister_service(mgs->mgs_service);
 
         mgs_cleanup_fsdb_list(obd);
@@ -322,15 +317,15 @@ static int mgs_check_target(struct obd_device *obd, struct mgs_target_info *mti)
 
         rc = mgs_check_index(obd, mti);
         if (rc == 0) {
-                LCONSOLE_ERROR("%s claims to have registered, but this MGS "
-                               "does not know about it.  Assuming writeconf.\n",
-                               mti->mti_svname);
+                LCONSOLE_ERROR_MSG(0x13b, "%s claims to have registered, but "
+                                  "this MGS does not know about it.  Assuming "
+                                  "writeconf.\n", mti->mti_svname);
                 mti->mti_flags |= LDD_F_WRITECONF;
                 rc = 1;
         } else if (rc == -1) {
-                LCONSOLE_ERROR("Client log %s-client has disappeared! "
-                               "Regenerating all logs.\n",
-                               mti->mti_fsname);
+                LCONSOLE_ERROR_MSG(0x13c, "Client log %s-client has "
+                                   "disappeared! Regenerating all logs.\n",
+                                   mti->mti_fsname);
                 mti->mti_flags |= LDD_F_WRITECONF;
                 rc = 1;
         } else {
@@ -375,14 +370,15 @@ static int mgs_handle_target_reg(struct ptlrpc_request *req)
            revoke at the end they will just update from where they left off. */
         lockrc = mgs_get_cfg_lock(obd, mti->mti_fsname, &lockh);
         if (lockrc != ELDLM_OK) {
-                LCONSOLE_ERROR("%s: Can't signal other nodes to update "
-                               "their configuration (%d). Updating local logs "
-                               "anyhow; you might have to manually restart "
-                               "other nodes to get the latest configuration.\n",
-                               obd->obd_name, lockrc);
+                LCONSOLE_ERROR_MSG(0x13d, "%s: Can't signal other nodes to "
+                                   "update their configuration (%d). Updating "
+                                   "local logs anyhow; you might have to "
+                                   "manually restart other nodes to get the "
+                                   "latest configuration.\n",
+                                   obd->obd_name, lockrc);
         }
 
-        OBD_FAIL_TIMEOUT(OBD_FAIL_MGS_SLOW_TARGET_REG, 10);
+        OBD_FAIL_TIMEOUT(OBD_FAIL_MGS_PAUSE_TARGET_REG, 10);
 
         /* Log writing contention is handled by the fsdb_sem */
 
@@ -454,14 +450,62 @@ out_nolock:
         RETURN(rc);
 }
 
+static int mgs_set_info_rpc(struct ptlrpc_request *req)
+{
+        struct obd_device *obd = req->rq_export->exp_obd;
+        struct mgs_send_param *msp, *rep_msp;
+        struct lustre_handle lockh;
+        int rep_size[] = { sizeof(struct ptlrpc_body), sizeof(*msp) };
+        int lockrc, rc;
+        struct lustre_cfg_bufs bufs;
+        struct lustre_cfg *lcfg;
+        char fsname[MTI_NAME_MAXLEN];
+        ENTRY;
+
+        msp = lustre_swab_reqbuf(req, REQ_REC_OFF, sizeof(*msp), NULL);
+
+        /* Construct lustre_cfg structure to pass to function mgs_setparam */
+        lustre_cfg_bufs_reset(&bufs, NULL);
+        lustre_cfg_bufs_set_string(&bufs, 1, msp->mgs_param);
+        lcfg = lustre_cfg_new(LCFG_PARAM, &bufs);
+        rc = mgs_setparam(obd, lcfg, fsname);
+        if (rc) {
+                CERROR("Error %d in setting the parameter %s for fs %s\n",
+                       rc, msp->mgs_param, fsname);
+                RETURN(rc);
+        }
+
+        /* Revoke lock so everyone updates.  Should be alright if
+         * someone was already reading while we were updating the logs,
+         * so we don't really need to hold the lock while we're
+         * writing.
+         */
+        if (fsname[0]) {
+                lockrc = mgs_get_cfg_lock(obd, fsname, &lockh);
+                if (lockrc != ELDLM_OK)
+                        CERROR("lock error %d for fs %s\n", lockrc,
+                               fsname);
+                else
+                        mgs_put_cfg_lock(&lockh);
+        }
+        lustre_cfg_free(lcfg);
+
+        lustre_pack_reply(req, 2, rep_size, NULL);
+        rep_msp = lustre_msg_buf(req->rq_repmsg, REPLY_REC_OFF,
+                                 sizeof(*rep_msp));
+        memcpy(rep_msp, msp, sizeof(*rep_msp));
+
+        RETURN(rc);
+}
+
 int mgs_handle(struct ptlrpc_request *req)
 {
         int fail = OBD_FAIL_MGS_ALL_REPLY_NET;
         int opc, rc = 0;
         ENTRY;
 
-        OBD_FAIL_TIMEOUT(OBD_FAIL_MGS_SLOW_REQUEST_NET, 2);
-        OBD_FAIL_RETURN(OBD_FAIL_MGS_ALL_REQUEST_NET | OBD_FAIL_ONCE, 0);
+        OBD_FAIL_TIMEOUT_MS(OBD_FAIL_MGS_PAUSE_REQ, obd_fail_val);
+        OBD_FAIL_RETURN(OBD_FAIL_MGS_ALL_REQUEST_NET, 0);
 
         LASSERT(current->journal_info == NULL);
         opc = lustre_msg_get_opc(req->rq_reqmsg);
@@ -490,13 +534,16 @@ int mgs_handle(struct ptlrpc_request *req)
                 req->rq_status = rc;            /* superfluous? */
                 break;
         case MGS_TARGET_REG:
-                DEBUG_REQ(D_MGS, req, "target add\n");
+                DEBUG_REQ(D_MGS, req, "target add");
                 rc = mgs_handle_target_reg(req);
                 break;
         case MGS_TARGET_DEL:
-                DEBUG_REQ(D_MGS, req, "target del\n");
+                DEBUG_REQ(D_MGS, req, "target del");
                 //rc = mgs_handle_target_del(req);
                 break;
+        case MGS_SET_INFO:
+                rc = mgs_set_info_rpc(req);
+                break;
 
         case LDLM_ENQUEUE:
                 DEBUG_REQ(D_MGS, req, "enqueue");
@@ -515,7 +562,7 @@ int mgs_handle(struct ptlrpc_request *req)
                 rc = target_handle_ping(req);
                 break;
         case OBD_LOG_CANCEL:
-                DEBUG_REQ(D_MGS, req, "log cancel\n");
+                DEBUG_REQ(D_MGS, req, "log cancel");
                 rc = -ENOTSUPP; /* la la la */
                 break;
 
@@ -634,6 +681,7 @@ out_free:
                 push_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
                 rc = class_config_dump_llog(ctxt, data->ioc_inlbuf1, NULL);
                 pop_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
+                llog_ctxt_put(ctxt);
                 if (rc)
                         RETURN(rc);
 
@@ -649,6 +697,7 @@ out_free:
                 push_ctxt(&saved, &ctxt->loc_exp->exp_obd->obd_lvfs_ctxt, NULL);
                 rc = llog_ioctl(ctxt, cmd, data);
                 pop_ctxt(&saved, &ctxt->loc_exp->exp_obd->obd_lvfs_ctxt, NULL);
+                llog_ctxt_put(ctxt);
 
                 RETURN(rc);
         }
