@@ -39,9 +39,7 @@
 #include <obd_class.h>
 #include <lustre_debug.h>
 #include <lprocfs_status.h>
-#ifdef __KERNEL__
-#include <linux/lustre_build_version.h>
-#endif
+#include <lustre/lustre_build_version.h>
 #include <libcfs/list.h>
 #include "llog_internal.h"
 
@@ -56,16 +54,18 @@ spinlock_t obd_dev_lock = SPIN_LOCK_UNLOCKED;
 #ifndef __KERNEL__
 atomic_t obd_memory;
 int obd_memmax;
+unsigned int obd_fail_val;
+unsigned int obd_fail_loc;
+unsigned int obd_alloc_fail_rate;
 #endif
 
 /* The following are visible and mutable through /proc/sys/lustre/. */
-unsigned int obd_fail_loc;
 unsigned int obd_debug_peer_on_timeout;
 unsigned int obd_dump_on_timeout;
 unsigned int obd_dump_on_eviction;
-unsigned int obd_timeout = 100; /* seconds */
+unsigned int obd_timeout = OBD_TIMEOUT_DEFAULT; /* seconds */
 unsigned int ldlm_timeout = LDLM_TIMEOUT_DEFAULT; /* seconds */
-unsigned int obd_health_check_timeout = 120; /* seconds */
+unsigned int obd_health_check_timeout = HEALTH_CHECK_TIMEOUT_DEFAULT; /* seconds */
 unsigned int obd_max_dirty_pages = 256;
 atomic_t obd_dirty_pages;
 
@@ -116,7 +116,7 @@ static inline void obd_conn2data(struct obd_ioctl_data *data,
         data->ioc_cookie = conn->cookie;
 }
 
-int class_resolve_dev_name(uint32_t len, const char *name)
+int class_resolve_dev_name(__u32 len, const char *name)
 {
         int rc;
         int dev;
@@ -376,7 +376,6 @@ void *obd_psdev = NULL;
 #endif
 
 EXPORT_SYMBOL(obd_devs);
-EXPORT_SYMBOL(obd_fail_loc);
 EXPORT_SYMBOL(obd_print_fail_loc);
 EXPORT_SYMBOL(obd_race_waitq);
 EXPORT_SYMBOL(obd_race_state);
@@ -412,13 +411,13 @@ EXPORT_SYMBOL(class_disconnect);
 EXPORT_SYMBOL(class_num2obd);
 
 /* uuid.c */
-EXPORT_SYMBOL(class_generate_random_uuid);
 EXPORT_SYMBOL(class_uuid_unparse);
 EXPORT_SYMBOL(lustre_uuid_to_peer);
 
 EXPORT_SYMBOL(class_handle_hash);
 EXPORT_SYMBOL(class_handle_unhash);
 EXPORT_SYMBOL(class_handle2object);
+EXPORT_SYMBOL(class_handle_free_cb);
 
 /* obd_config.c */
 EXPORT_SYMBOL(class_incref);
@@ -511,7 +510,6 @@ int obd_init_checks(void)
 #endif
 
 extern spinlock_t obd_types_lock;
-extern spinlock_t handle_lock;
 extern int class_procfs_init(void);
 extern int class_procfs_clean(void);
 
@@ -535,7 +533,6 @@ int init_obdclass(void)
 #endif
 
         spin_lock_init(&obd_types_lock);
-        spin_lock_init(&handle_lock);
         cfs_waitq_init(&obd_race_waitq);
         obd_zombie_impexp_init();
 
