@@ -40,6 +40,7 @@
 #include <lustre_param.h>
 
 static int (*client_fill_super)(struct super_block *sb) = NULL;
+static void (*kill_super_cb)(struct super_block *sb) = NULL;
 
 /*********** mount lookup *********/
 
@@ -188,8 +189,10 @@ int server_put_mount(char *name, struct vfsmount *mnt)
                 CERROR("Can't find mount for %s\n", name);
                 RETURN(-ENOENT);
         }
-        lsi = s2lsi(lmi->lmi_sb);
         LASSERT(lmi->lmi_mnt == mnt);
+        /* Note if lsi is poisoned, it's probably because server_wait_finished
+           didn't wait long enough and the sb was freed at the umount. */
+        lsi = s2lsi(lmi->lmi_sb);
 
         CDEBUG(D_MOUNT, "put_mnt %p from %s, refs=%d, vfscount=%d\n",
                lmi->lmi_mnt, name, atomic_read(&lsi->lsi_mounts), count);
@@ -245,7 +248,7 @@ static int ldd_parse(struct lvfs_run_ctxt *mount_ctxt,
                 GOTO(out, rc);
         }
 
-        len = file->f_dentry->d_inode->i_size;
+        len = i_size_read(file->f_dentry->d_inode);
         CDEBUG(D_MOUNT, "Have %s, size %lu\n", MOUNT_DATA_FILE, len);
         if (len != sizeof(*ldd)) {
                 CERROR("disk data size does not match: see %lu expect "LPSZ"\n",
@@ -362,19 +365,20 @@ int lustre_process_log(struct super_block *sb, char *logname,
         lustre_cfg_free(lcfg);
 
         if (rc == -EINVAL)
-                LCONSOLE_ERROR("%s: The configuration from log '%s' failed "
-                               "(%d). Make sure this client and "
-                               "the MGS are running compatible versions of "
-                               "Lustre.\n",
-                               mgc->obd_name, logname, rc);
+                LCONSOLE_ERROR_MSG(0x15b, "%s: The configuration from log '%s' "
+                                   "failed (%d). Make sure this client and "
+                                   "the MGS are running compatible versions of "
+                                   "Lustre.\n",
+                                   mgc->obd_name, logname, rc);
 
         if (rc)
-                LCONSOLE_ERROR("%s: The configuration from log '%s' failed "
-                               "(%d). This may be the result of "
-                               "communication errors between this node and "
-                               "the MGS, a bad configuration, or other errors."
-                               " See the syslog for more information.\n",
-                               mgc->obd_name, logname, rc);
+                LCONSOLE_ERROR_MSG(0x15c, "%s: The configuration from log '%s' "
+                                   "failed (%d). This may be the result of "
+                                   "communication errors between this node and "
+                                   "the MGS, a bad configuration, or other "
+                                   "errors. See the syslog for more "
+                                   "information.\n", mgc->obd_name, logname, 
+                                   rc);
 
         /* class_obd_list(); */
         RETURN(rc);
@@ -468,8 +472,8 @@ static int server_start_mgs(struct super_block *sb)
         lmi = server_find_mount(LUSTRE_MGS_OBDNAME);
         if (lmi) {
                 lsi = s2lsi(lmi->lmi_sb);
-                LCONSOLE_ERROR("The MGS service was already started from "
-                               "server %s\n", lsi->lsi_ldd->ldd_svname);
+                LCONSOLE_ERROR_MSG(0x15d, "The MGS service was already started "
+                                   "from server %s\n", lsi->lsi_ldd->ldd_svname);
                 RETURN(-EALREADY);
         }
 
@@ -483,8 +487,9 @@ static int server_start_mgs(struct super_block *sb)
                 server_deregister_mount(LUSTRE_MGS_OBDNAME);
 
         if (rc)
-                LCONSOLE_ERROR("Failed to start MGS '%s' (%d).  Is the 'mgs' "
-                               "module loaded?\n", LUSTRE_MGS_OBDNAME, rc);
+                LCONSOLE_ERROR_MSG(0x15e, "Failed to start MGS '%s' (%d).  Is "
+                                   "the 'mgs' module loaded?\n", 
+                                   LUSTRE_MGS_OBDNAME, rc);
 
         RETURN(rc);
 }
@@ -640,7 +645,7 @@ static int lustre_start_mgc(struct super_block *sb)
 
         /* Random uuid for MGC allows easier reconnects */
         OBD_ALLOC_PTR(uuid);
-        class_generate_random_uuid(uuidc);
+        ll_generate_random_uuid(uuidc);
         class_uuid_unparse(uuidc, uuid);
 
         /* Start the MGC */
@@ -724,8 +729,8 @@ static int lustre_stop_mgc(struct super_block *sb)
 {
         struct lustre_sb_info *lsi = s2lsi(sb);
         struct obd_device *obd;
-        char *niduuid, *ptr = 0;
-        int i, rc = 0, len;
+        char *niduuid = 0, *ptr = 0;
+        int i, rc = 0, len = 0;
         ENTRY;
 
         if (!lsi)
@@ -744,11 +749,9 @@ static int lustre_stop_mgc(struct super_block *sb)
                 GOTO(out, rc = -EBUSY);
         }
 
-        /* MGC must always stop */
-        obd->obd_force = 1;
-        /* client_disconnect_export uses the no_recov flag to decide whether it
-           should disconnect or just invalidate.  (The MGC has no
-           recoverable data in any case.) */
+        /* MGC should disconnect nicely so MGS won't print eviction messages */
+        obd->obd_force = (lsi->lsi_flags & LSI_UMOUNT_FORCE) != 0;
+        /* The MGC has no recoverable data in any case. */
         obd->obd_no_recov = 1;
 
         if (obd->u.cli.cl_mgc_mgsexp)
@@ -769,7 +772,8 @@ static int lustre_stop_mgc(struct super_block *sb)
 
         /* Clean the nid uuids */
         if (!niduuid)
-                RETURN(-ENOMEM);
+                GOTO(out, rc = -ENOMEM);
+
         for (i = 0; i < lsi->lsi_lmd->lmd_mgs_failnodes; i++) {
                 sprintf(ptr, "_%x", i);
                 rc = do_lcfg(LUSTRE_MGC_OBDNAME, 0, LCFG_DEL_UUID,
@@ -778,10 +782,11 @@ static int lustre_stop_mgc(struct super_block *sb)
                         CERROR("del MDC UUID %s failed: rc = %d\n",
                                niduuid, rc);
         }
-        OBD_FREE(niduuid, len);
-        /* class_import_put will get rid of the additional connections */
-
 out:
+        if (niduuid)
+                OBD_FREE(niduuid, len);
+
+        /* class_import_put will get rid of the additional connections */
         mutex_up(&mgc_start_lock);
         RETURN(rc);
 }
@@ -1048,15 +1053,16 @@ static int server_start_targets(struct super_block *sb, struct vfsmount *mnt)
                 CERROR("Required registration failed for %s: %d\n",
                        lsi->lsi_ldd->ldd_svname, rc);
                 if (rc == -EIO) {
-                        LCONSOLE_ERROR("Communication error with the MGS.  Is "
-                                       "the MGS running?\n");
+                        LCONSOLE_ERROR_MSG(0x15f, "Communication error with the"
+                                           " MGS.  Is the MGS running?\n");
                 }
                 GOTO(out_mgc, rc);
         }
         if (rc == -EINVAL) {
-                LCONSOLE_ERROR("The MGS is refusing to allow this server (%s) "
-                               "to start.  Please see messages on the MGS node."
-                               "\n", lsi->lsi_ldd->ldd_svname);
+                LCONSOLE_ERROR_MSG(0x160, "The MGS is refusing to allow this "
+                                   "server (%s) to start.  Please see messages "
+                                   "on the MGS node.\n", 
+                                   lsi->lsi_ldd->ldd_svname);
                 GOTO(out_mgc, rc);
         }
 
@@ -1092,6 +1098,9 @@ out_mgc:
                         obd_iocontrol(OBD_IOC_ABORT_RECOVERY,
                                       obd->obd_self_export, 0, NULL, NULL);
                 }
+
+                /* log has been fully processed */
+                obd_notify(obd, NULL, OBD_NOTIFY_CONFIG, (void *)CONFIG_LOG);
         }
 
         RETURN(rc);
@@ -1211,11 +1220,17 @@ static struct vfsmount *server_kernel_mount(struct super_block *sb)
         if (IS_ERR(mnt)) {
                 rc = PTR_ERR(mnt);
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0))
-                CERROR("premount %s:%#lx ldiskfs failed (%d), is the ldiskfs "
-                       "module available?\n", lmd->lmd_dev, s_flags, rc);
-                GOTO(out_free, rc);
+                /* 2.6 kernels: if ldiskfs fails, try ldiskfs2 */
+                mnt = ll_kern_mount("ldiskfs2", s_flags, lmd->lmd_dev, 0);
+                if (IS_ERR(mnt)) {
+                        int rc2 = PTR_ERR(mnt);
+                        CERROR("premount %s:%#lx ldiskfs failed: %d, ldiskfs2 "
+                               "failed: %d.  Is the ldiskfs module available?\n",
+                               lmd->lmd_dev, s_flags, rc, rc2);
+                        GOTO(out_free, rc);
+                }
 #else
-                /* If ldisk fails, try ext3 */
+                /* 2.4 kernels: if ldiskfs fails, try ext3 */
                 mnt = ll_kern_mount("ext3", s_flags, lmd->lmd_dev, 0);
                 if (IS_ERR(mnt)) {
                         rc = PTR_ERR(mnt);
@@ -1286,7 +1301,7 @@ static void server_wait_finished(struct vfsmount *mnt)
 {
         wait_queue_head_t   waitq;
         struct l_wait_info  lwi;
-        int                 retries = 120;
+        int                 retries = 330;
 
         init_waitqueue_head(&waitq);
 
@@ -1297,13 +1312,12 @@ static void server_wait_finished(struct vfsmount *mnt)
 
                 /* Wait for a bit */
                 retries -= 5;
-                lwi = LWI_TIMEOUT(5 * HZ, NULL, NULL);
+                lwi = LWI_TIMEOUT(cfs_time_seconds(5), NULL, NULL);
                 l_wait_event(waitq, 0, &lwi);
         }
-        if (atomic_read(&mnt->mnt_count) > 1) {
+        if (atomic_read(&mnt->mnt_count) > 1)
                 CERROR("Mount %p is still busy (%d refs), giving up.\n",
                        mnt, atomic_read(&mnt->mnt_count));
-        }
 }
 
 static void server_put_super(struct super_block *sb)
@@ -1344,8 +1358,6 @@ static void server_put_super(struct super_block *sb)
                 obd = class_name2obd(lsi->lsi_ldd->ldd_svname);
                 if (obd) {
                         CDEBUG(D_MOUNT, "stopping %s\n", obd->obd_name);
-                        if (lsi->lsi_flags & LSI_UMOUNT_FORCE)
-                                obd->obd_force = 1;
                         if (lsi->lsi_flags & LSI_UMOUNT_FAILOVER)
                                 obd->obd_fail = 1;
                         /* We can't seem to give an error return code
@@ -1530,9 +1542,10 @@ static int server_fill_super(struct super_block *sb)
                lsi->lsi_lmd->lmd_dev);
 
         if (class_name2obd(lsi->lsi_ldd->ldd_svname)) {
-                LCONSOLE_ERROR("The target named %s is already running. "
-                               "Double-mount may have compromised the disk "
-                               "journal.\n", lsi->lsi_ldd->ldd_svname);
+                LCONSOLE_ERROR_MSG(0x161, "The target named %s is already "
+                                   "running. Double-mount may have compromised "
+                                   "the disk journal.\n", 
+                                   lsi->lsi_ldd->ldd_svname);
                 unlock_mntput(mnt);
                 lustre_put_lsi(sb);
                 GOTO(out, rc = -EALREADY);
@@ -1742,16 +1755,16 @@ static int lmd_parse(char *options, struct lustre_mount_data *lmd)
 
         LASSERT(lmd);
         if (!options) {
-                LCONSOLE_ERROR("Missing mount data: check that "
-                               "/sbin/mount.lustre is installed.\n");
+                LCONSOLE_ERROR_MSG(0x162, "Missing mount data: check that "
+                                   "/sbin/mount.lustre is installed.\n");
                 RETURN(-EINVAL);
         }
 
         /* Options should be a string - try to detect old lmd data */
         if ((raw->lmd_magic & 0xffffff00) == (LMD_MAGIC & 0xffffff00)) {
-                LCONSOLE_ERROR("You're using an old version of "
-                               "/sbin/mount.lustre.  Please install version "
-                               "%s\n", LUSTRE_VERSION_STRING);
+                LCONSOLE_ERROR_MSG(0x163, "You're using an old version of "
+                                   "/sbin/mount.lustre.  Please install "
+                                   "version %s\n", LUSTRE_VERSION_STRING);
                 RETURN(-EINVAL);
         }
         lmd->lmd_magic = LMD_MAGIC;
@@ -1809,8 +1822,8 @@ static int lmd_parse(char *options, struct lustre_mount_data *lmd)
         }
 
         if (!devname) {
-                LCONSOLE_ERROR("Can't find the device name "
-                               "(need mount option 'device=...')\n");
+                LCONSOLE_ERROR_MSG(0x164, "Can't find the device name "
+                                   "(need mount option 'device=...')\n");
                 goto invalid;
         }
 
@@ -1879,8 +1892,9 @@ int lustre_fill_super(struct super_block *sb, void *data, int silent)
         if (lmd_is_client(lmd)) {
                 CDEBUG(D_MOUNT, "Mounting client %s\n", lmd->lmd_profile);
                 if (!client_fill_super) {
-                        LCONSOLE_ERROR("Nothing registered for client mount!"
-                               " Is the 'lustre' module loaded?\n");
+                        LCONSOLE_ERROR_MSG(0x165, "Nothing registered for "
+                                           "client mount! Is the 'lustre' "
+                                           "module loaded?\n");
                         rc = -ENODEV;
                 } else {
                         rc = lustre_start_mgc(sb);
@@ -1922,6 +1936,11 @@ void lustre_register_client_fill_super(int (*cfs)(struct super_block *sb))
         client_fill_super = cfs;
 }
 
+void lustre_register_kill_super_cb(void (*cfs)(struct super_block *sb))
+{
+        kill_super_cb = cfs;
+}
+
 /***************** FS registration ******************/
 
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0))
@@ -1947,12 +1966,22 @@ int lustre_get_sb(struct file_system_type *fs_type,
 }
 #endif
 
+void lustre_kill_super(struct super_block *sb)
+{
+        struct lustre_sb_info *lsi = s2lsi(sb);
+
+        if (kill_super_cb && lsi && !(lsi->lsi_flags & LSI_SERVER))
+                (*kill_super_cb)(sb);
+
+        kill_anon_super(sb);
+}
+
 struct file_system_type lustre_fs_type = {
         .owner        = THIS_MODULE,
         .name         = "lustre",
         .get_sb       = lustre_get_sb,
-        .kill_sb      = kill_anon_super,
-        .fs_flags     = FS_BINARY_MOUNTDATA,
+        .kill_sb      = lustre_kill_super,
+        .fs_flags     = FS_BINARY_MOUNTDATA | FS_REQUIRES_DEV,
 };
 
 #else
@@ -1988,6 +2017,7 @@ int lustre_unregister_fs(void)
 }
 
 EXPORT_SYMBOL(lustre_register_client_fill_super);
+EXPORT_SYMBOL(lustre_register_kill_super_cb);
 EXPORT_SYMBOL(lustre_common_put_super);
 EXPORT_SYMBOL(lustre_process_log);
 EXPORT_SYMBOL(lustre_end_log);
