@@ -465,7 +465,8 @@ static int echo_setup(struct obd_device *obd, obd_count len, void *buf)
         obd->u.echo.eo_lastino = ECHO_INIT_OBJID;
 
         obd->obd_namespace = ldlm_namespace_new("echo-tgt",
-                                                LDLM_NAMESPACE_SERVER);
+                                                LDLM_NAMESPACE_SERVER,
+                                                LDLM_NAMESPACE_GREEDY);
         if (obd->obd_namespace == NULL) {
                 LBUG();
                 RETURN(-ENOMEM);
