@@ -27,7 +27,7 @@
  *   license text for more details.
  */
 
-#ifdef HAVE_KERNEL_CONFIG_H
+#ifndef AUTOCONF_INCLUDED
 #include <linux/config.h>
 #endif
 #include <linux/module.h>
@@ -170,7 +170,7 @@ int filter_clear_truncated_page(struct inode *inode)
         int rc;
 
         /* Truncate on page boundary, so nothing to flush? */
-        if (!(inode->i_size & ~CFS_PAGE_MASK))
+        if (!(i_size_read(inode) & ~CFS_PAGE_MASK))
                 return 0;
 
         rc = filter_sync_inode_data(inode);
@@ -180,7 +180,7 @@ int filter_clear_truncated_page(struct inode *inode)
         /* be careful to call this after fsync_inode_data_buffers has waited
          * for IO to complete before we evict it from the cache */
         page = find_lock_page(inode->i_mapping,
-                              inode->i_size >> CFS_PAGE_SHIFT);
+                              i_size_read(inode) >> CFS_PAGE_SHIFT);
         if (page) {
                 if (page->mapping != NULL)
                         ll_truncate_complete_page(page);
@@ -240,11 +240,11 @@ int filter_direct_io(int rw, struct dentry *dchild, struct filter_iobuf *buf,
 
         if (rw == OBD_BRW_WRITE) {
                 if (rc == 0) {
-                        filter_tally_write(exp, iobuf->maplist, iobuf->nr_pages,
-                                           KIOBUF_GET_BLOCKS(iobuf),
-                                           blocks_per_page);
+                        filter_tally(exp, iobuf->maplist, iobuf->nr_pages,
+                                     KIOBUF_GET_BLOCKS(iobuf), blocks_per_page,
+                                     1);
 
-                        if (attr->ia_size > inode->i_size)
+                        if (attr->ia_size > i_size_read(inode))
                                 attr->ia_valid |= ATTR_SIZE;
                         rc = fsfilt_setattr(obd, dchild,
                                             oti->oti_handle, attr, 0);
@@ -255,7 +255,7 @@ int filter_direct_io(int rw, struct dentry *dchild, struct filter_iobuf *buf,
                 up(&inode->i_sem);
                 cleanup_phase = 3;
 
-                rc = filter_finish_transno(exp, oti, 0);
+                rc = filter_finish_transno(exp, oti, 0, 0);
                 if (rc)
                         GOTO(cleanup, rc);
 
@@ -264,8 +264,8 @@ int filter_direct_io(int rw, struct dentry *dchild, struct filter_iobuf *buf,
                 if (rc)
                         GOTO(cleanup, rc);
         } else {
-                filter_tally_read(exp, iobuf->maplist, iobuf->nr_pages,
-                                  KIOBUF_GET_BLOCKS(iobuf), blocks_per_page);
+                filter_tally(exp, iobuf->maplist, iobuf->nr_pages,
+                             KIOBUF_GET_BLOCKS(iobuf), blocks_per_page, 0);
         }
 
         rc = filter_clear_page_cache(inode, iobuf);
