@@ -5,17 +5,11 @@
 #ifndef OST_INTERNAL_H
 #define OST_INTERNAL_H
 
-#ifdef LPROCFS
-extern void ost_print_req(void *seq_file, struct ptlrpc_request *req);
-#else
-# define ost_print_req NULL
-#endif
-
 /*
  * tunables for per-thread page pool (bug 5137)
  */
 #define OST_THREAD_POOL_SIZE PTLRPC_MAX_BRW_PAGES  /* pool size in pages */
-#define OST_THREAD_POOL_GFP  GFP_HIGHUSER          /* GFP mask for pool pages */
+#define OST_THREAD_POOL_GFP  CFS_ALLOC_HIGHUSER    /* GFP mask for pool pages */
 
 struct page;
 struct niobuf_local;
@@ -37,6 +31,9 @@ struct ost_thread_local_cache {
 
 struct ost_thread_local_cache *ost_tls(struct ptlrpc_request *r);
 
+#define OSS_DEF_CREATE_THREADS  1UL
+#define OSS_MAX_CREATE_THREADS 16UL
+
 /* Quota stuff */
 extern quota_interface_t *quota_interface;
 
