@@ -132,10 +132,10 @@ int ptlrpc_set_import_discon(struct obd_import *imp, __u32 conn_cnt)
                                imp->imp_obd->obd_name, target_len, target_start,
                                libcfs_nid2str(imp->imp_connection->c_peer.nid));
                 } else {
-                        LCONSOLE_ERROR("%s: Connection to service %.*s via nid "
-                               "%s was lost; in progress operations using this "
-                               "service will fail.\n",
-                               imp->imp_obd->obd_name, target_len, target_start,
+                        LCONSOLE_ERROR_MSG(0x166, "%s: Connection to service "
+                               "%.*s via nid %s was lost; in progress "
+                               "operations using this service will fail.\n",
+                               imp->imp_obd->obd_name, target_len, target_start, 
                                libcfs_nid2str(imp->imp_connection->c_peer.nid));
                 }
                 IMPORT_SET_STATE_NOLOCK(imp, LUSTRE_IMP_DISCON);
@@ -188,6 +188,8 @@ void ptlrpc_invalidate_import(struct obd_import *imp)
         struct l_wait_info lwi;
         int rc;
 
+        atomic_inc(&imp->imp_inval_count);
+
         if (!imp->imp_invalid)
                 ptlrpc_deactivate_import(imp);
 
@@ -206,6 +208,9 @@ void ptlrpc_invalidate_import(struct obd_import *imp)
                        atomic_read(&imp->imp_inflight));
 
         obd_import_event(imp->imp_obd, imp, IMP_EVENT_INVALIDATE);
+
+        atomic_dec(&imp->imp_inval_count);
+        cfs_waitq_signal(&imp->imp_recovery_waitq);
 }
 
 /* unset imp_invalid */
@@ -274,7 +279,7 @@ static int import_select_connection(struct obd_import *imp)
                                        cfs_time_current_64())) {
                         /* If we have never tried this connection since the
                            the last successful attempt, go with this one */
-                        if (cfs_time_before_64(conn->oic_last_attempt,
+                        if (cfs_time_beforeq_64(conn->oic_last_attempt,
                                                imp->imp_last_success_conn)) {
                                 imp_conn = conn;
                                 break;
@@ -288,6 +293,14 @@ static int import_select_connection(struct obd_import *imp)
                         else if (cfs_time_before_64(conn->oic_last_attempt,
                                                     imp_conn->oic_last_attempt))
                                 imp_conn = conn;
+                } else {
+                        /* Exceptionally unlikely case caused by the node
+                         * booting and attempting to mount lustre faster than
+                         * than RECONNECT_INTERVAL seconds. */
+                        if (unlikely(conn->oic_last_attempt == 0)) {
+                                imp_conn = conn;
+                                break;
+                        }
                 }
         }
 
@@ -405,7 +418,9 @@ int ptlrpc_connect_import(struct obd_import *imp, char *new_uuid)
                 if (imp->imp_recon_bk) {
                         CDEBUG(D_HA, "Last reconnection attempt (%d) for %s\n",
                                imp->imp_conn_cnt, obd2cli_tgt(imp->imp_obd));
+                        spin_lock(&imp->imp_lock);
                         imp->imp_last_recon = 1;
+                        spin_unlock(&imp->imp_lock);
                 }
         }
 
@@ -442,13 +457,21 @@ int ptlrpc_connect_import(struct obd_import *imp, char *new_uuid)
         aa->pcaa_initial_connect = initial_connect;
 
         if (aa->pcaa_initial_connect) {
+                spin_lock(&imp->imp_lock);
                 imp->imp_replayable = 1;
+                spin_unlock(&imp->imp_lock);
                 /* On an initial connect, we don't know which one of a
                    failover server pair is up.  Don't wait long. */
+#ifdef CRAY_XT3
+                request->rq_timeout = max((int)(obd_timeout / 2), 5);
+#else
                 request->rq_timeout = max((int)(obd_timeout / 20), 5);
+#endif
         }
 
-        DEBUG_REQ(D_RPCTRACE, request, "(re)connect request");
+        DEBUG_REQ(D_RPCTRACE, request, "%sconnect request %d",
+                  aa->pcaa_initial_connect ? "initial " : "re", 
+                  imp->imp_conn_cnt);
         ptlrpcd_add_req(request);
         rc = 0;
 out:
@@ -521,15 +544,18 @@ static int ptlrpc_connect_interpret(struct ptlrpc_request *request,
         msg_flags = lustre_msg_get_op_flags(request->rq_repmsg);
 
         /* All imports are pingable */
+        spin_lock(&imp->imp_lock);
         imp->imp_pingable = 1;
 
         if (aa->pcaa_initial_connect) {
                 if (msg_flags & MSG_CONNECT_REPLAYABLE) {
+                        imp->imp_replayable = 1;
+                        spin_unlock(&imp->imp_lock);
                         CDEBUG(D_HA, "connected to replayable target: %s\n",
                                obd2cli_tgt(imp->imp_obd));
-                        imp->imp_replayable = 1;
                 } else {
                         imp->imp_replayable = 0;
+                        spin_unlock(&imp->imp_lock);
                 }
 
                 if (msg_flags & MSG_CONNECT_NEXT_VER) {
@@ -546,6 +572,8 @@ static int ptlrpc_connect_interpret(struct ptlrpc_request *request,
 
                 IMPORT_SET_STATE(imp, LUSTRE_IMP_FULL);
                 GOTO(finish, rc = 0);
+        } else {
+                spin_unlock(&imp->imp_lock);
         }
 
         /* Determine what recovery state to move the import to. */
@@ -591,7 +619,11 @@ static int ptlrpc_connect_interpret(struct ptlrpc_request *request,
                         CDEBUG(D_HA, "%s: reconnected to %s during replay\n",
                                imp->imp_obd->obd_name,
                                obd2cli_tgt(imp->imp_obd));
+
+                        spin_lock(&imp->imp_lock);
                         imp->imp_resend_replay = 1;
+                        spin_unlock(&imp->imp_lock);
+
                         IMPORT_SET_STATE(imp, LUSTRE_IMP_REPLAY);
                 } else {
                         IMPORT_SET_STATE(imp, LUSTRE_IMP_RECOVER);
@@ -603,6 +635,7 @@ static int ptlrpc_connect_interpret(struct ptlrpc_request *request,
                 imp->imp_last_replay_transno = 0;
                 IMPORT_SET_STATE(imp, LUSTRE_IMP_REPLAY);
         } else {
+                DEBUG_REQ(D_HA, request, "evicting, flags=%x", msg_flags);
                 imp->imp_remote_handle =
                                 *lustre_msg_get_handle(request->rq_repmsg);
                 IMPORT_SET_STATE(imp, LUSTRE_IMP_EVICTED);
@@ -641,7 +674,6 @@ finish:
 
                 ocd = lustre_swab_repbuf(request, REPLY_REC_OFF, sizeof(*ocd),
                                          lustre_swab_connect);
-
                 spin_lock(&imp->imp_lock);
                 list_del(&imp->imp_conn_current->oic_item);
                 list_add(&imp->imp_conn_current->oic_item, &imp->imp_conn_list);
@@ -669,12 +701,12 @@ finish:
                 if (!exp) {
                         /* This could happen if export is cleaned during the 
                            connect attempt */
-                        spin_unlock(&imp->imp_lock);
                         CERROR("Missing export for %s\n", 
                                imp->imp_obd->obd_name);
                         GOTO(out, rc = -ENODEV);
                 }
                 exp->exp_connect_flags = ocd->ocd_connect_flags;
+                imp->imp_obd->obd_self_export->exp_connect_flags = ocd->ocd_connect_flags;
                 class_export_put(exp);
 
                 obd_import_event(imp->imp_obd, imp, IMP_EVENT_OCD);
@@ -686,23 +718,29 @@ finish:
 
                 if ((ocd->ocd_connect_flags & OBD_CONNECT_VERSION) &&
                     (ocd->ocd_version > LUSTRE_VERSION_CODE +
-                    LUSTRE_VERSION_OFFSET_WARN)) {
+                                        LUSTRE_VERSION_OFFSET_WARN ||
+                     ocd->ocd_version < LUSTRE_VERSION_CODE -
+                                        LUSTRE_VERSION_OFFSET_WARN)) {
                         /* Sigh, some compilers do not like #ifdef in the middle
                            of macro arguments */
 #ifdef __KERNEL__
-                        const char *action = "upgrading this client";
+                        const char *older =
+                                "older.  Consider upgrading this client";
 #else
-                        const char *action = "recompiling this application";
+                        const char *older =
+                                "older.  Consider recompiling this application";
 #endif
+                        const char *newer = "newer than client version";
 
-                        CWARN("Server %s version (%d.%d.%d.%d) is much newer. "
-                              "Consider %s (%s).\n",
-                              obd2cli_tgt(imp->imp_obd),
-                              OBD_OCD_VERSION_MAJOR(ocd->ocd_version),
-                              OBD_OCD_VERSION_MINOR(ocd->ocd_version),
-                              OBD_OCD_VERSION_PATCH(ocd->ocd_version),
-                              OBD_OCD_VERSION_FIX(ocd->ocd_version),
-                              action, LUSTRE_VERSION_STRING);
+                        LCONSOLE_WARN("Server %s version (%d.%d.%d.%d) "
+                                      "is much %s (%s)\n",
+                                      obd2cli_tgt(imp->imp_obd),
+                                      OBD_OCD_VERSION_MAJOR(ocd->ocd_version),
+                                      OBD_OCD_VERSION_MINOR(ocd->ocd_version),
+                                      OBD_OCD_VERSION_PATCH(ocd->ocd_version),
+                                      OBD_OCD_VERSION_FIX(ocd->ocd_version),
+                                      ocd->ocd_version > LUSTRE_VERSION_CODE ?
+                                      newer : older, LUSTRE_VERSION_STRING);
                 }
 
                 if (ocd->ocd_connect_flags & OBD_CONNECT_BRW_SIZE) {
@@ -710,6 +748,8 @@ finish:
                                 ocd->ocd_brw_size >> CFS_PAGE_SHIFT;
                 }
 
+                imp->imp_obd->obd_namespace->ns_connect_flags = ocd->ocd_connect_flags;
+
                 LASSERT((cli->cl_max_pages_per_rpc <= PTLRPC_MAX_BRW_PAGES) &&
                         (cli->cl_max_pages_per_rpc > 0));
         }
@@ -737,16 +777,17 @@ finish:
                            /* Actually servers are only supposed to refuse
                               connection from liblustre clients, so we should
                               never see this from VFS context */
-                                CERROR("Server %s version (%d.%d.%d.%d) "
-                                       "refused connection from this client "
-                                       "as too old version (%s).  Client must "
-                                       "be recompiled\n",
-                                      obd2cli_tgt(imp->imp_obd),
-                                      OBD_OCD_VERSION_MAJOR(ocd->ocd_version),
-                                      OBD_OCD_VERSION_MINOR(ocd->ocd_version),
-                                      OBD_OCD_VERSION_PATCH(ocd->ocd_version),
-                                      OBD_OCD_VERSION_FIX(ocd->ocd_version),
-                                      LUSTRE_VERSION_STRING);
+                                LCONSOLE_ERROR_MSG(0x16a, "Server %s version "
+                                        "(%d.%d.%d.%d)"
+                                        " refused connection from this client "
+                                        "with an incompatible version (%s).  "
+                                        "Client must be recompiled\n",
+                                        obd2cli_tgt(imp->imp_obd),
+                                        OBD_OCD_VERSION_MAJOR(ocd->ocd_version),
+                                        OBD_OCD_VERSION_MINOR(ocd->ocd_version),
+                                        OBD_OCD_VERSION_PATCH(ocd->ocd_version),
+                                        OBD_OCD_VERSION_FIX(ocd->ocd_version),
+                                        LUSTRE_VERSION_STRING);
                                 ptlrpc_deactivate_import(imp);
                                 IMPORT_SET_STATE(imp, LUSTRE_IMP_CLOSED);
                         }
@@ -760,7 +801,9 @@ finish:
                        (char *)imp->imp_connection->c_remote_uuid.uuid, rc);
         }
         
+        spin_lock(&imp->imp_lock);
         imp->imp_last_recon = 0;
+        spin_unlock(&imp->imp_lock);
 
         cfs_waitq_signal(&imp->imp_recovery_waitq);
         RETURN(rc);
@@ -848,10 +891,10 @@ int ptlrpc_import_recovery_state_machine(struct obd_import *imp)
                 /* Don't care about MGC eviction */
                 if (strcmp(imp->imp_obd->obd_type->typ_name,
                            LUSTRE_MGC_NAME) != 0) {
-                        LCONSOLE_ERROR("This client was evicted by %.*s; "
-                                       "in progress operations using this "
-                                       "service will fail.\n",
-                                       target_len, target_start);
+                        LCONSOLE_ERROR_MSG(0x167, "This client was evicted by "
+                                           "%.*s; in progress operations using "
+                                           "this service will fail.\n",
+                                           target_len, target_start);
                 }
                 CDEBUG(D_HA, "evicted from %s@%s; invalidating\n",
                        obd2cli_tgt(imp->imp_obd),
@@ -974,7 +1017,11 @@ int ptlrpc_disconnect_import(struct obd_import *imp, int noclose)
                  * it fails.  We can get through the above with a down server
                  * if the client doesn't know the server is gone yet. */
                 req->rq_no_resend = 1;
+#ifdef CRAY_XT3
+                req->rq_timeout = obd_timeout / 3;
+#else
                 req->rq_timeout = 5;
+#endif
                 IMPORT_SET_STATE(imp, LUSTRE_IMP_CONNECTING);
                 req->rq_send_state =  LUSTRE_IMP_CONNECTING;
                 ptlrpc_req_set_repsize(req, 1, NULL);
@@ -994,3 +1041,11 @@ out:
         RETURN(rc);
 }
 
+/* Sets maximal number of RPCs possible originating from other side of this
+   import (server) to us and number of async RPC replies that we are not waiting
+   for arriving */
+void ptlrpc_import_setasync(struct obd_import *imp, int count)
+{
+        LNetSetAsync(imp->imp_connection->c_peer, count);
+}
+
