@@ -281,6 +281,37 @@ int lprocfs_wr_type(struct file *file, const char *buffer,
         return count;
 }
 EXPORT_SYMBOL(lprocfs_wr_type);
+
+int lprocfs_filter_rd_limit(char *page, char **start, off_t off, int count, 
+                                   int *eof, void *data)
+{
+        struct obd_device *obd = (struct obd_device *)data;
+        LASSERT(obd != NULL);
+        
+        return snprintf(page, count, "%lu\n", 
+                        obd->u.obt.obt_qctxt.lqc_limit_sz);
+}
+EXPORT_SYMBOL(lprocfs_filter_rd_limit);
+
+int lprocfs_filter_wr_limit(struct file *file, const char *buffer,
+                                   unsigned long count, void *data)
+{
+        struct obd_device *obd = (struct obd_device *)data;
+        int val, rc;
+        LASSERT(obd != NULL);
+        
+        rc = lprocfs_write_helper(buffer, count, &val);
+        if (rc)
+                return rc;
+        
+        if (val <= 1 << 20)
+                return -EINVAL;
+        
+        obd->u.obt.obt_qctxt.lqc_limit_sz = val;
+        return count;
+} 
+EXPORT_SYMBOL(lprocfs_filter_wr_limit);
+
 #endif /* LPROCFS */
 
 static int filter_quota_setup(struct obd_device *obd)
@@ -309,7 +340,9 @@ static int filter_quota_setinfo(struct obd_export *exp, struct obd_device *obd)
         struct obd_import *imp;
 
         /* setup the quota context import */
+        spin_lock(&obd->u.obt.obt_qctxt.lqc_lock);
         obd->u.obt.obt_qctxt.lqc_import = exp->exp_imp_reverse;
+        spin_unlock(&obd->u.obt.obt_qctxt.lqc_lock);
 
         /* make imp's connect flags equal relative exp's connect flags 
          * adding it to avoid the scan export list
@@ -323,6 +356,22 @@ static int filter_quota_setinfo(struct obd_export *exp, struct obd_device *obd)
         qslave_start_recovery(obd, &obd->u.obt.obt_qctxt);
         return 0;
 }
+
+static int filter_quota_clearinfo(struct obd_export *exp, struct obd_device *obd)
+{
+        struct lustre_quota_ctxt *qctxt = &obd->u.obt.obt_qctxt;
+
+        /* when exp->exp_imp_reverse is destroyed, the corresponding lqc_import
+         * should be invalid b=12374 */
+        if (qctxt->lqc_import == exp->exp_imp_reverse) {
+                spin_lock(&qctxt->lqc_lock);
+                qctxt->lqc_import = NULL;
+                spin_unlock(&qctxt->lqc_lock);
+        }
+
+        return 0;
+}
+
 static int filter_quota_enforce(struct obd_device *obd, unsigned int ignore)
 {
         ENTRY;
@@ -393,6 +442,42 @@ static int filter_quota_acquire(struct obd_device *obd, unsigned int uid,
         RETURN(rc == -EAGAIN);
 }
 
+/* check whether the left quota of certain uid and uid can satisfy a write rpc
+ * when need to acquire quota, return QUOTA_RET_ACQUOTA */
+static int filter_quota_check(struct obd_device *obd, unsigned int uid, 
+                              unsigned int gid, int npage)
+{
+        struct lustre_quota_ctxt *qctxt = &obd->u.obt.obt_qctxt;
+        int i;
+        __u32 id[MAXQUOTAS] = { uid, gid };
+        __u64 limit;
+        struct qunit_data qdata[MAXQUOTAS];
+        int rc;
+        ENTRY;
+
+        CLASSERT(MAXQUOTAS < 4);
+        if (!sb_any_quota_enabled(qctxt->lqc_sb))
+                RETURN(0);
+
+        for (i = 0; i < MAXQUOTAS; i++) {
+                qdata[i].qd_id = id[i];
+                qdata[i].qd_flags = i;
+                qdata[i].qd_flags |= QUOTA_IS_BLOCK;
+                qdata[i].qd_count = 0;
+
+                qctxt_wait_pending_dqacq(qctxt, id[i], i, 1);
+                rc = compute_remquota(obd, qctxt, &qdata[i]);
+                limit = npage * CFS_PAGE_SIZE;
+                if (limit < qctxt->lqc_limit_sz )
+                        limit =  qctxt->lqc_limit_sz;
+                if (rc == QUOTA_RET_OK && 
+                    qdata[i].qd_count < limit)
+                        RETURN(QUOTA_RET_ACQUOTA);
+        }
+
+        RETURN(rc);
+}
+
 static int mds_quota_init(void)
 {
         return lustre_dquot_init();
@@ -624,8 +709,8 @@ int osc_quota_init(void)
 
         LASSERT(qinfo_cachep == NULL);
         qinfo_cachep = cfs_mem_cache_create("osc_quota_info",
-                                         sizeof(struct osc_quota_info),
-                                         0, 0);
+                                            sizeof(struct osc_quota_info),
+                                            0, 0);
         if (!qinfo_cachep)
                 RETURN(-ENOMEM);
 
@@ -676,10 +761,12 @@ quota_interface_t filter_quota_interface = {
         .quota_check    = target_quota_check,
         .quota_ctl      = filter_quota_ctl,
         .quota_setinfo  = filter_quota_setinfo,
+        .quota_clearinfo = filter_quota_clearinfo,
         .quota_enforce  = filter_quota_enforce,
         .quota_getflag  = filter_quota_getflag,
         .quota_acquire  = filter_quota_acquire,
         .quota_adjust   = filter_quota_adjust,
+        .quota_chkquota = filter_quota_check,
 };
 #endif /* __KERNEL__ */
 
