@@ -6,6 +6,7 @@ set -e
 
 PATH=`dirname $0`/../utils:$PATH
 
+[ -z "$CONFIG" -a "$NAME" ] && CONFIGS=$NAME
 [ "$CONFIGS" ] || CONFIGS="local"  #"local lov"
 [ "$MAX_THREADS" ] || MAX_THREADS=20
 RAMKB=`awk '/MemTotal:/ { print $2 }' /proc/meminfo`
@@ -21,12 +22,21 @@ fi
 [ "$TMP" ] || TMP=/tmp
 [ "$COUNT" ] || COUNT=1000
 [ "$DEBUG_LVL" ] || DEBUG_LVL=0
-[ "$DEBUG_OFF" ] || DEBUG_OFF="sysctl -w lnet.debug=$DEBUG_LVL"
-[ "$DEBUG_ON" ] || DEBUG_ON="sysctl -w lnet.debug=0x33f0484"
+[ "$DEBUG_OFF" ] || DEBUG_OFF="eval sysctl -w lnet.debug=\"$DEBUG_LVL\""
+[ "$DEBUG_ON" ] || DEBUG_ON="eval sysctl -w lnet.debug=0x33f0484"
 
+if [ "$ACC_SM_ONLY" ]; then
+    export RUNTESTS="no" SANITY="no" DBENCH="no" BONNIE="no" IOZONE="no" FSX="no" SANITYN="no" LFSCK="no" LIBLUSTRE="no" REPLAY_SINGLE="no" CONF_SANITY="no" RECOVERY_SMALL="no" REPLAY_OST_SINGLE="no" REPLAY_DUAL="no" INSANITY="no" SANITY_QUOTA="no"
+    for O in $ACC_SM_ONLY; do
+	O=`echo $O | tr "[:lower:]" "[:upper:]"`
+	export ${O}="yes"
+    done
+fi
 
-LIBLUSTRE=${LIBLUSTRE:-../liblustre}
-LIBLUSTRETESTS=${LIBLUSTRETESTS:-$LIBLUSTRE/tests}
+LIBLUSTRETESTS=${LIBLUSTRETESTS:-../liblustre/tests}
+
+STARTTIME=`date +%s`
+RANTEST=""
 
 LUSTRE=${LUSTRE:-`dirname $0`/..}
 . $LUSTRE/tests/test-framework.sh
@@ -42,7 +52,8 @@ setup_if_needed() {
 }
 
 title() {
-    echo "-----============= acceptance-small: "$*" ============-----"
+    log "-----============= acceptance-small: "$*" ============----- `date`"
+    RANTEST=${RANTEST}$*", "
 }
 
 for NAME in $CONFIGS; do
@@ -57,14 +68,14 @@ for NAME in $CONFIGS; do
 
 	if [ "$RUNTESTS" != "no" ]; then
 	        title runtests
-		sh runtests
+		bash runtests
 		$CLEANUP
 		$SETUP
 	fi
 
 	if [ "$SANITY" != "no" ]; then
 	        title sanity
-		SANITYLOG=/tmp/sanity.log sh sanity.sh
+		bash sanity.sh
 		$CLEANUP
 		$SETUP
 	fi
@@ -77,13 +88,13 @@ for NAME in $CONFIGS; do
 		[ $THREADS -lt $DB_THREADS ] && DB_THREADS=$THREADS
 
 		$DEBUG_OFF
-		sh rundbench 1
+		bash rundbench 1
 		$DEBUG_ON
 		$CLEANUP
 		$SETUP
 		if [ $DB_THREADS -gt 1 ]; then
 			$DEBUG_OFF
-			sh rundbench $DB_THREADS
+			bash rundbench $DB_THREADS
 			$DEBUG_ON
 			$CLEANUP
 			$SETUP
@@ -91,14 +102,20 @@ for NAME in $CONFIGS; do
 		rm -f /mnt/lustre/`hostname`/client.txt
 	fi
 
-	chown $UID $MOUNT && chmod 700 $MOUNT
+	chown $UID $MOUNT
 	which bonnie++ > /dev/null 2>&1 || BONNIE=no
 	if [ "$BONNIE" != "no" ]; then
 	        title bonnie
-		SPACE=`df -P $MOUNT | tail -n 1 | awk '{ print $4 }'`
+		mkdir -p $MOUNT/d0.bonnie
+		$LFS setstripe -c -1 $MOUNT/d0.bonnie
+		sync
+		MIN=`cat /proc/fs/lustre/osc/*/kbytesavail | sort -n | head -n1`
+		SPACE=$(( OSTCOUNT * MIN ))
 		[ $SPACE -lt $SIZE ] && SIZE=$((SPACE * 3 / 4))
+		log "min OST has ${MIN}kB available, using ${SIZE}kB file size"
 		$DEBUG_OFF
-		bonnie++ -f -r 0 -s $((SIZE / 1024)) -n 10 -u $UID -d $MOUNT
+		BONFILE=$MOUNT/d0.bonnie
+		bonnie++ -f -r 0 -s$((SIZE / 1024)) -n 10 -u$UID -d$BONFILE
 		$DEBUG_ON
 		$CLEANUP
 		$SETUP
@@ -108,12 +125,18 @@ for NAME in $CONFIGS; do
 	which iozone > /dev/null 2>&1 || IOZONE=no
 	if [ "$IOZONE" != "no" ]; then
 	        title iozone
-		SPACE=`df -P $MOUNT | tail -n 1 | awk '{ print $4 }'`
+		mkdir -p $MOUNT/d0.iozone
+		$LFS setstripe -c -1 $MOUNT/d0.iozone
+		sync
+		MIN=`cat /proc/fs/lustre/osc/*/kbytesavail | sort -n | head -n1`
+		SPACE=$(( OSTCOUNT * MIN ))
 		[ $SPACE -lt $SIZE ] && SIZE=$((SPACE * 3 / 4))
+		log "min OST has ${MIN}kB available, using ${SIZE}kB file size"
 		IOZONE_OPTS="-i 0 -i 1 -i 2 -e -+d -r $RSIZE -s $SIZE"
-		IOZFILE="-f $MOUNT/iozone"
+		IOZFILE="$MOUNT/d0.iozone/iozone"
+		# $SPACE was calculated with all OSTs
 		$DEBUG_OFF
-		iozone $IOZONE_OPTS $IOZFILE
+		iozone $IOZONE_OPTS -f $IOZFILE
 		$DEBUG_ON
 		$CLEANUP
 		$SETUP
@@ -139,6 +162,7 @@ for NAME in $CONFIGS; do
 		[ $THREADS -lt $IOZ_THREADS ] && IOZ_THREADS=$THREADS
 		IOZVER=`iozone -v | awk '/Revision:/ {print $3}' | tr -d .`
 		if [ "$IOZ_THREADS" -gt 1 -a "$IOZVER" -ge 3145 ]; then
+			$LFS setstripe -c 1 $MOUNT/d0.iozone
 			$DEBUG_OFF
 			THREAD=1
 			IOZFILE="-F "
@@ -175,7 +199,7 @@ for NAME in $CONFIGS; do
 		mkdir -p $MOUNT2
 		mount_client $MOUNT2
 		#echo "can't mount2 for '$NAME', skipping sanityN.sh"
-		SANITYLOG=$TMP/sanity.log START=: CLEAN=: sh sanityN.sh
+		START=: CLEAN=: bash sanityN.sh
 		umount $MOUNT2
 
 		$DEBUG_ON
@@ -186,8 +210,17 @@ for NAME in $CONFIGS; do
 	if [ "$LFSCK" != "no" -a -x /usr/sbin/lfsck ]; then
 	        title lfsck
 		E2VER=`e2fsck -V 2>&1 | head -n 1 | cut -d' ' -f 2`
-		[ `echo $E2VER | cut -d. -f2` -ge 39 ] && \
-			[ "`echo $E2VER | grep cfs`" ] && sh lfscktest.sh
+		if grep -q obdfilter /proc/fs/lustre/devices; then
+			if [ `echo $E2VER | cut -d. -f2` -ge 39 ] && \
+			   [ "`echo $E2VER | grep cfs`" ]; then
+			   	bash lfscktest.sh
+			else
+				e2fsck -V
+				echo "e2fsck does not support lfsck, skipping"
+			fi
+		else
+			echo "remote OST, skipping test"
+		fi
 	fi
 
 	if [ "$LIBLUSTRE" != "no" ]; then
@@ -196,13 +229,17 @@ for NAME in $CONFIGS; do
 		$CLEANUP
 		unload_modules
 		# Liblustre needs accept=all, noacl
-		LNETOPTS="accept=all" MDS_MOUNT_OPTS="${MDS_MOUNT_OPTS},noacl" $SETUP
+		[ -f /etc/modprobe.conf ] && MODPROBECONF=/etc/modprobe.conf
+		[ -f /etc/modprobe.d/Lustre ] && MODPROBECONF=/etc/modprobe.d/Lustre
+
+		LNETOPTS="$(awk '/^options lnet/ { print $0}' $MODPROBECONF | sed 's/^options lnet //g') accept=all" MDS_MOUNT_OPTS="${MDS_MOUNT_OPTS},noacl" $SETUP
 		export LIBLUSTRE_MOUNT_POINT=$MOUNT2
 		export LIBLUSTRE_MOUNT_TARGET=$MGSNID:/$FSNAME
 		export LIBLUSTRE_TIMEOUT=`cat /proc/sys/lustre/timeout`
 		#export LIBLUSTRE_DEBUG_MASK=`cat /proc/sys/lnet/debug`
 		if [ -x $LIBLUSTRETESTS/sanity ]; then
 			mkdir -p $MOUNT2
+			echo $LIBLUSTRETESTS/sanity --target=$LIBLUSTRE_MOUNT_TARGET
 			$LIBLUSTRETESTS/sanity --target=$LIBLUSTRE_MOUNT_TARGET
 		fi
 		$CLEANUP
@@ -214,40 +251,43 @@ done
 
 if [ "$REPLAY_SINGLE" != "no" ]; then
         title replay-single
-	sh replay-single.sh
+	bash replay-single.sh
 fi
 
 if [ "$CONF_SANITY" != "no" ]; then
         title conf-sanity
-        sh conf-sanity.sh
+        bash conf-sanity.sh
 fi
 
 if [ "$RECOVERY_SMALL" != "no" ]; then
         title recovery-small
-        sh recovery-small.sh
+        bash recovery-small.sh
 fi
 
 if [ "$REPLAY_OST_SINGLE" != "no" ]; then
         title replay-ost-single
-        sh replay-ost-single.sh
+        bash replay-ost-single.sh
 fi
 
 if [ "$REPLAY_DUAL" != "no" ]; then
         title replay-dual
-        sh replay-dual.sh
+        bash replay-dual.sh
 fi
 
 if [ "$INSANITY" != "no" ]; then
         title insanity
-        sh insanity.sh -r
+        bash insanity.sh -r
 fi
 
 if [ "$SANITY_QUOTA" != "no" ]; then
         title sanity-quota
-        sh sanity-quota.sh
+        bash sanity-quota.sh
 fi
 
-title FINISHED
 
 RC=$?
+title FINISHED
+echo "Finished at `date` in $((`date +%s` - $STARTTIME))s"
+echo "Tests ran: $RANTEST"
+
 echo "$0: completed with rc $RC" && exit $RC
