@@ -1,12 +1,11 @@
-
 #!/bin/bash
 
 set -e
 
-#         bug  11190 5494 7288 5493
-ALWAYS_EXCEPT="19b   24   27   52 $RECOVERY_SMALL_EXCEPT"
+#         bug  5493
+ALWAYS_EXCEPT="52 $RECOVERY_SMALL_EXCEPT"
 
-PTLDEBUG=${PTLDEBUG:--1}
+#PTLDEBUG=${PTLDEBUG:--1}
 LUSTRE=${LUSTRE:-`dirname $0`/..}
 . $LUSTRE/tests/test-framework.sh
 init_test_env $@
@@ -16,32 +15,10 @@ build_test_filter
 
 # Allow us to override the setup if we already have a mounted system by
 # setting SETUP=" " and CLEANUP=" "
-SETUP=${SETUP:-"setup"}
-CLEANUP=${CLEANUP:-"cleanup"}
+SETUP=${SETUP:-""}
+CLEANUP=${CLEANUP:-""}
 
-setup() {
-    formatall
-    setupall
-}
-
-cleanup() {
-	cleanupall || { echo "FAILed to clean up"; exit 20; }
-}
-
-if [ ! -z "$EVAL" ]; then
-    eval "$EVAL"
-    exit $?
-fi
-
-if [ "$ONLY" == "cleanup" ]; then
-    sysctl -w lnet.debug=0 || true
-    cleanup
-    exit
-fi
-
-$SETUP
-
-[ "$ONLY" == "setup" ] && exit
+cleanup_and_setup_lustre
 
 test_1() {
     drop_request "mcreate $MOUNT/1"  || return 1
@@ -61,26 +38,34 @@ test_3() {
 }
 run_test 3 "stat: drop req, drop rep"
 
+SAMPLE_NAME=recovery-small.junk
+SAMPLE_FILE=$TMP/$SAMPLE_NAME
+# make this big, else test 9 doesn't wait for bulk -- bz 5595
+dd if=/dev/urandom of=$SAMPLE_FILE bs=1M count=4
+
 test_4() {
-    do_facet client "cp /etc/inittab $MOUNT/inittab" || return 1
-    drop_request "cat $MOUNT/inittab > /dev/null"   || return 2
-    drop_reply "cat $MOUNT/inittab > /dev/null"     || return 3
+    do_facet client "cp $SAMPLE_FILE $MOUNT/$SAMPLE_NAME" || return 1
+    drop_request "cat $MOUNT/$SAMPLE_NAME > /dev/null"   || return 2
+    drop_reply "cat $MOUNT/$SAMPLE_NAME > /dev/null"     || return 3
 }
 run_test 4 "open: drop req, drop rep"
 
 test_5() {
-    drop_request "mv $MOUNT/inittab $MOUNT/renamed" || return 1
+    drop_request "mv $MOUNT/$SAMPLE_NAME $MOUNT/renamed" || return 1
     drop_reint_reply "mv $MOUNT/renamed $MOUNT/renamed-again" || return 2
     do_facet client "checkstat -v $MOUNT/renamed-again"  || return 3
 }
 run_test 5 "rename: drop req, drop rep"
 
+[ ! -e $MOUNT/renamed-again ] && cp $SAMPLE_FILE $MOUNT/renamed-again
 test_6() {
     drop_request "mlink $MOUNT/renamed-again $MOUNT/link1" || return 1
     drop_reint_reply "mlink $MOUNT/renamed-again $MOUNT/link2"   || return 2
 }
 run_test 6 "link: drop req, drop rep"
 
+[ ! -e $MOUNT/link1 ] && mlink $MOUNT/renamed-again $MOUNT/link1
+[ ! -e $MOUNT/link2 ] && mlink $MOUNT/renamed-again $MOUNT/link2
 test_7() {
     drop_request "munlink $MOUNT/link1"   || return 1
     drop_reint_reply "munlink $MOUNT/link2"     || return 2
@@ -89,16 +74,19 @@ run_test 7 "unlink: drop req, drop rep"
 
 #bug 1423
 test_8() {
-    drop_reint_reply "touch $MOUNT/renamed"    || return 1
+    drop_reint_reply "touch $MOUNT/$tfile"    || return 1
 }
 run_test 8 "touch: drop rep (bug 1423)"
 
+SAMPLE_FILE=$TMP/recovery-small.junk
+dd if=/dev/urandom of=$SAMPLE_FILE bs=1M count=4
+
 #bug 1420
 test_9() {
-    pause_bulk "cp /etc/profile $MOUNT"       || return 1
-    do_facet client "cp /etc/termcap $MOUNT"  || return 2
+    pause_bulk "cp /etc/profile $MOUNT/$tfile"       || return 1
+    do_facet client "cp $SAMPLE_FILE $MOUNT/${tfile}.2"  || return 2
     do_facet client "sync"
-    do_facet client "rm $MOUNT/termcap $MOUNT/profile" || return 3
+    do_facet client "rm $MOUNT/$tfile $MOUNT/${tfile}.2" || return 3
 }
 run_test 9 "pause bulk on OST (bug 1420)"
 
@@ -190,7 +178,7 @@ start_read_ahead() {
 }
 
 test_16() {
-    do_facet client cp /etc/termcap $MOUNT
+    do_facet client cp $SAMPLE_FILE $MOUNT
     sync
     stop_read_ahead
 
@@ -198,11 +186,11 @@ test_16() {
     do_facet ost1 sysctl -w lustre.fail_loc=0x80000504
     cancel_lru_locks osc
     # OST bulk will time out here, client resends
-    do_facet client "cmp /etc/termcap $MOUNT/termcap" || return 1
-    sysctl -w lustre.fail_loc=0
+    do_facet client "cmp $SAMPLE_FILE $MOUNT/${SAMPLE_FILE##*/}" || return 1
+    do_facet ost1 sysctl -w lustre.fail_loc=0
     # give recovery a chance to finish (shouldn't take long)
     sleep $TIMEOUT
-    do_facet client "cmp /etc/termcap $MOUNT/termcap" || return 2
+    do_facet client "cmp $SAMPLE_FILE $MOUNT/${SAMPLE_FILE##*/}" || return 2
     start_read_ahead
 }
 run_test 16 "timeout bulk put, don't evict client (2732)"
@@ -210,23 +198,23 @@ run_test 16 "timeout bulk put, don't evict client (2732)"
 test_17() {
     # OBD_FAIL_PTLRPC_BULK_GET_NET 0x0503 | OBD_FAIL_ONCE
     # OST bulk will time out here, client retries
-    sysctl -w lustre.fail_loc=0x80000503
+    do_facet ost1 sysctl -w lustre.fail_loc=0x80000503
     # need to ensure we send an RPC
-    do_facet client cp /etc/termcap $DIR/$tfile
+    do_facet client cp $SAMPLE_FILE $DIR/$tfile
     sync
 
     sleep $TIMEOUT
-    sysctl -w lustre.fail_loc=0
+    do_facet ost1 sysctl -w lustre.fail_loc=0
     do_facet client "df $DIR"
     # expect cmp to succeed, client resent bulk
-    do_facet client "cmp /etc/termcap $DIR/$tfile" || return 3
+    do_facet client "cmp $SAMPLE_FILE $DIR/$tfile" || return 3
     do_facet client "rm $DIR/$tfile" || return 4
     return 0
 }
 run_test 17 "timeout bulk get, don't evict client (2732)"
 
 test_18a() {
-    [ -z ${ost2_svc} ] && echo Skipping, needs 2 osts && return 0
+    [ -z ${ost2_svc} ] && skip "needs 2 osts" && return 0
 
     do_facet client mkdir -p $MOUNT/$tdir
     f=$MOUNT/$tdir/$tfile
@@ -237,7 +225,7 @@ test_18a() {
     # 1 stripe on ost2
     lfs setstripe $f $((128 * 1024)) 1 1
 
-    do_facet client cp /etc/termcap $f
+    do_facet client cp $SAMPLE_FILE $f
     sync
     local osc2dev=`grep ${ost2_svc}-osc- $LPROC/devices | awk '{print $1}'`
     $LCTL --device $osc2dev deactivate || return 3
@@ -263,11 +251,12 @@ test_18b() {
     lfs setstripe $f $((128 * 1024)) 0 1
     lfs setstripe $f2 $((128 * 1024)) 0 1
 
-    do_facet client cp /etc/termcap $f
+    do_facet client cp $SAMPLE_FILE $f
     sync
     ost_evict_client
-    # allow recovery to complete
-    sleep $((TIMEOUT + 2))
+    # force reconnect
+    df $MOUNT > /dev/null 2>&1
+    sleep 2
     # my understanding is that there should be nothing in the page
     # cache after the client reconnects?     
     rc=0
@@ -562,24 +551,22 @@ test_24() {	# bug 2248 - eviction fails writeback but app doesn't see it
 	cancel_lru_locks osc
 	multiop $DIR/$tdir/$tfile Owy_wyc &
 	MULTI_PID=$!
-	usleep 500
+	sleep 0.500s
 	ost_evict_client
-	usleep 500
+	sleep 0.500s
 	kill -USR1 $MULTI_PID
 	wait $MULTI_PID
 	rc=$?
 	sysctl -w lustre.fail_loc=0x0
 	client_reconnect
-	[ $rc -eq 0 ] && error "multiop didn't fail fsync: rc $rc" || true
+	[ $rc -eq 0 ] && error_ignore 5494 "multiop didn't fail fsync: rc $rc" || true
 }
 run_test 24 "fsync error (should return error)"
 
 test_26() {      # bug 5921 - evict dead exports by pinger
 # this test can only run from a client on a separate node.
-	[ "`lsmod | grep obdfilter`" ] && \
-	    echo "skipping test 26 (local OST)" && return
-	[ "`lsmod | grep mds`" ] && \
-	    echo "skipping test 26 (local MDS)" && return
+	remote_ost || skip "local OST" && return
+	remote_mds || skip "local MDS" && return
 	OST_FILE=$LPROC/obdfilter/${ost1_svc}/num_exports
         OST_EXP="`do_facet ost1 cat $OST_FILE`"
 	OST_NEXP1=`echo $OST_EXP | cut -d' ' -f2`
@@ -601,18 +588,20 @@ test_26() {      # bug 5921 - evict dead exports by pinger
 run_test 26 "evict dead exports"
 
 test_26b() {      # bug 10140 - evict dead exports by pinger
-	zconf_mount `hostname` $MOUNT2
+	client_df
+	zconf_mount `hostname` $MOUNT2 || error "Failed to mount $MOUNT2"
 	MDS_FILE=$LPROC/mds/${mds_svc}/num_exports
         MDS_NEXP1="`do_facet mds cat $MDS_FILE | cut -d' ' -f2`"
 	OST_FILE=$LPROC/obdfilter/${ost1_svc}/num_exports
         OST_NEXP1="`do_facet ost1 cat $OST_FILE | cut -d' ' -f2`"
 	echo starting with $OST_NEXP1 OST and $MDS_NEXP1 MDS exports
+	#force umount a client; exports should get evicted
 	zconf_umount `hostname` $MOUNT2 -f
-	# evictor takes up to 2.25x to evict.  But if there's a 
-	# race to start the evictor from various obds, the loser
-	# might have to wait for the next ping.
-	echo Waiting for $(($TIMEOUT * 4)) secs
-	sleep $(($TIMEOUT * 4))
+   # evictor takes up to 2.25x to evict.  But if there's a 
+   # race to start the evictor from various obds, the loser
+   # might have to wait for the next ping.
+   echo Waiting for $(($TIMEOUT * 4)) secs
+   sleep $(($TIMEOUT * 4))
         OST_NEXP2="`do_facet ost1 cat $OST_FILE | cut -d' ' -f2`"
         MDS_NEXP2="`do_facet mds cat $MDS_FILE | cut -d' ' -f2`"
 	echo ending with $OST_NEXP2 OST and $MDS_NEXP2 MDS exports
@@ -623,8 +612,7 @@ test_26b() {      # bug 10140 - evict dead exports by pinger
 run_test 26b "evict dead exports"
 
 test_27() {
-	[ "`lsmod | grep mds`" ] || \
-	    { echo "skipping test 27 (non-local MDS)" && return 0; }
+	remote_mds && { skip "remote MDS" && return 0; }
 	mkdir -p $DIR/$tdir
 	writemany -q -a $DIR/$tdir/$tfile 0 5 &
 	CLIENT_PID=$!
@@ -661,17 +649,22 @@ run_test 28 "handle error adding new clients (bug 6086)"
 
 test_50() {
 	mkdir -p $DIR/$tdir
+	debugsave
+	sysctl -w lnet.debug="-dlmtrace -ha"
 	# put a load of file creates/writes/deletes
 	writemany -q $DIR/$tdir/$tfile 0 5 &
 	CLIENT_PID=$!
 	echo writemany pid $CLIENT_PID
 	sleep 10
 	FAILURE_MODE="SOFT"
+	$LCTL mark "$TESTNAME fail mds 1"
 	fail mds
 	# wait for client to reconnect to MDS
 	sleep 60
+	$LCTL mark "$TESTNAME fail mds 2"
 	fail mds
 	sleep 60
+	$LCTL mark "$TESTNAME fail mds 3"
 	fail mds
 	# client process should see no problems even though MDS went down
 	sleep $TIMEOUT
@@ -680,6 +673,7 @@ test_50() {
 	rc=$?
 	echo writemany returned $rc
 	#these may fail because of eviction due to slow AST response.
+	debugrestore
 	return $rc
 }
 run_test 50 "failover MDS under load"
@@ -699,6 +693,7 @@ test_51() {
           do
           echo failover in $i sec
           sleep $i
+	  $LCTL mark "$TESTNAME fail mds $i"
           facet_failover mds
         done
 	# client process should see no problems even though MDS went down
@@ -718,6 +713,7 @@ test_52_guts() {
 	echo writemany pid $CLIENT_PID
 	sleep 10
 	FAILURE_MODE="SOFT"
+	$LCTL mark "$TESTNAME fail ost $1"
 	fail ost1
 	rc=0
 	wait $CLIENT_PID || rc=$?
@@ -731,16 +727,16 @@ test_52_guts() {
 
 test_52() {
 	mkdir -p $DIR/$tdir
-	test_52_guts
+	test_52_guts 1
 	rc=$?
 	[ $rc -ne 0 ] && { return $rc; }
 	# wait for client to reconnect to OST
 	sleep 30
-	test_52_guts
+	test_52_guts 2
 	rc=$?
 	[ $rc -ne 0 ] && { return $rc; }
 	sleep 30
-	test_52_guts
+	test_52_guts 3
 	rc=$?
 	client_reconnect
 	#return $rc
@@ -750,7 +746,7 @@ run_test 52 "failover OST under load"
 # test of open reconstruct
 test_53() {
 	touch $DIR/$tfile
-	drop_ldlm_reply "./openfile -f O_RDWR:O_CREAT -m 0755 $MOUNT/$tfile" ||\
+	drop_ldlm_reply "openfile -f O_RDWR:O_CREAT -m 0755 $DIR/$tfile" ||\
 		return 2
 }
 run_test 53 "touch: drop rep"
@@ -770,8 +766,7 @@ run_test 54 "back in time"
 
 # bug 11330 - liblustre application death during I/O locks up OST
 test_55() {
-	[ "`lsmod | grep obdfilter`" ] || \
-	    { echo "skipping test 55 (non-local OST)" && return 0; }	
+	remote_ost && { skip "remote OST" && return 0; }
 
 	mkdir -p $DIR/$tdir
 
@@ -827,15 +822,15 @@ test_55() {
 }
 run_test 55 "ost_brw_read/write drops timed-out read/write request"
 
-test_56() {
+test_56() { # b=11277
 #define OBD_FAIL_MDS_RESEND      0x136
-        touch $MOUNT/$tfile
+        touch $DIR/$tfile
         do_facet mds sysctl -w lustre.fail_loc=0x80000136
-        stat $MOUNT/$tfile
+        stat $DIR/$tfile
         do_facet mds sysctl -w lustre.fail_loc=0
-        rm -f $MOUNT/$tfile
+        rm -f $DIR/$tfile
 }
-run_test 56 "replace lock race"
+run_test 56 "do not allow reconnect to busy exports"
 
 test_57_helper() {
         # no oscs means no client or mdt 
@@ -862,5 +857,33 @@ test_57() { # bug 10866
 }
 run_test 57 "read procfs entries causes kernel crash"
 
-$CLEANUP
-echo "$0: completed"
+test_58() { # bug 11546
+#define OBD_FAIL_MDC_ENQUEUE_PAUSE        0x801
+        touch $MOUNT/$tfile
+        ls -la $MOUNT/$tfile
+        sysctl -w lustre.fail_loc=0x80000801
+        cp $MOUNT/$tfile /dev/null &
+        pid=$!
+        sleep 1
+        sysctl -w lustre.fail_loc=0
+        drop_bl_callback rm -f $MOUNT/$tfile
+        wait $pid
+        do_facet client "df $DIR"
+}
+run_test 58 "Eviction in the middle of open RPC reply processing"
+
+test_59() { # bug 10589
+	zconf_mount `hostname` $MOUNT2 || error "Failed to mount $MOUNT2"
+	sysctl -w lustre.fail_loc=0x311
+	writes=`dd if=/dev/zero of=$DIR2/$tfile count=1 2>&1 | awk 'BEGIN { FS="+" } /out/ {print $1}'`
+	sysctl -w lustre.fail_loc=0
+	sync
+	zconf_umount `hostname` $DIR2 -f
+	reads=`dd if=$DIR/$tfile of=/dev/null 2>&1 | awk 'BEGIN { FS="+" } /in/ {print $1}'`
+	[ $reads -eq $writes ] || error "read" $reads "blocks, must be" $writes
+}
+run_test 59 "Read cancel race on client eviction"
+
+equals_msg `basename $0`: test complete, cleaning up
+check_and_cleanup_lustre
+[ -f "$TESTSUITELOG" ] && cat $TESTSUITELOG || true
