@@ -6,52 +6,23 @@ set -e
 #
 # This test needs to be run on the client
 #
-
+SAVE_PWD=$PWD
 LUSTRE=${LUSTRE:-`dirname $0`/..}
+SETUP=${SETUP:-}
+CLEANUP=${CLEANUP:-}
 . $LUSTRE/tests/test-framework.sh
 init_test_env $@
 . ${CONFIG:=$LUSTRE/tests/cfg/$NAME.sh}
-
+CHECK_GRANT=${CHECK_GRANT:-"yes"}
+GRANT_CHECK_LIST=${GRANT_CHECK_LIST:-""}
 
 # Skip these tests
-# bug number:  4176 
-ALWAYS_EXCEPT="39   $REPLAY_SINGLE_EXCEPT"
-
-gen_config() {
-    rm -f $XMLCONFIG
-    add_mds mds --dev $MDSDEV --size $MDSSIZE
-    if [ ! -z "$mdsfailover_HOST" ]; then
-	 add_mdsfailover mds --dev $MDSDEV --size $MDSSIZE
-    fi
-    
-    add_lov lov1 mds --stripe_sz $STRIPE_BYTES \
-	--stripe_cnt $STRIPES_PER_OBJ --stripe_pattern 0
-    add_ost ost --lov lov1 --dev $OSTDEV --size $OSTSIZE
-    add_ost ost2 --lov lov1 --dev ${OSTDEV}-2 --size $OSTSIZE
-    add_client client mds --lov lov1 --path $MOUNT
-}
+# bug number: 
+ALWAYS_EXCEPT="$REPLAY_SINGLE_EXCEPT"
 
 build_test_filter
 
-SETUP=${SETUP:-"setup"}
-CLEANUP=${CLEANUP:-"cleanupall"}
-
-if [ "$ONLY" == "cleanup" ]; then
-    sysctl -w lnet.debug=0 || true
-    $CLEANUP
-    exit 0
-fi
-
-setup() {
-    formatall
-    setupall
-}
-
-$SETUP
-
-if [ "$ONLY" == "setup" ]; then
-    exit 0
-fi
+cleanup_and_setup_lustre
 
 mkdir -p $DIR
 
@@ -133,7 +104,7 @@ test_2a() {
 run_test 2a "touch"
 
 test_2b() {
-    ./mcreate $DIR/$tfile
+    mcreate $DIR/$tfile
     replay_barrier mds
     touch $DIR/$tfile
     fail mds
@@ -462,7 +433,7 @@ test_20b() { # bug 10480
     dd if=/dev/zero of=$DIR/$tfile bs=4k count=10000 &
     pid=$!
     while [ ! -e $DIR/$tfile ] ; do
-        usleep 60                           # give dd a chance to start
+        sleep 0.060s                           # give dd a chance to start
     done
 
     lfs getstripe $DIR/$tfile || return 1
@@ -472,7 +443,7 @@ test_20b() { # bug 10480
 
     fail mds                            # start orphan recovery
     df -P $DIR || df -P $DIR || true    # reconnect
-    sleep 2
+    wait_mds_recovery_done || error "MDS recovery not done"
 
     AFTERUSED=`df -P $DIR | tail -1 | awk '{ print $3 }'`
     log "before $BEFOREUSED, after $AFTERUSED"
@@ -483,9 +454,10 @@ test_20b() { # bug 10480
 run_test 20b "write, unlink, eviction, replay, (test mds_cleanup_orphans)"
 
 test_20c() { # bug 10480
-    dd if=/dev/zero of=$DIR/$tfile bs=4k count=10000
-
-    exec 100< $DIR/$tfile
+    multiop $DIR/$tfile Ow_c &
+    pid=$!
+    # give multiop a chance to open
+    sleep 1
 
     ls -la $DIR/$tfile
 
@@ -493,8 +465,7 @@ test_20c() { # bug 10480
 
     df -P $DIR || df -P $DIR || true    # reconnect
 
-    exec 100<&-
-
+    kill -USR1 $pid
     test -s $DIR/$tfile || error "File was truncated"
 
     return 0
@@ -740,10 +711,11 @@ run_test 32 "close() notices client eviction; close() after client eviction"
 # Abort recovery before client complete
 test_33() {
     replay_barrier mds
-    touch $DIR/$tfile
+    createmany -o $DIR/$tfile-%d 100 
     fail_abort mds
     # this file should be gone, because the replay was aborted
-    $CHECKSTAT -t file $DIR/$tfile && return 3
+    $CHECKSTAT -t file $DIR/$tfile-* && return 3 
+    unlinkmany $DIR/$tfile-%d 0 100
     return 0
 }
 run_test 33 "abort recovery before client does replay"
@@ -887,6 +859,10 @@ run_test 40 "cause recovery in ptlrpc, ensure IO continues"
 # the page, guarnateeing that the unlock from the RPC completion would
 # assert on trying to unlock the unlocked page.
 test_41() {
+    [ $OSTCOUNT -lt 2 ] && \
+	skip "skipping test 41: we don't have a second OST to test with" && \
+	return
+
     local f=$MOUNT/$tfile
     # make sure the start of the file is ost1
     lfs setstripe $f $((128 * 1024)) 0 0 
@@ -908,7 +884,7 @@ test_42() {
     createmany -o $DIR/$tfile-%d 800
     replay_barrier ost1
     unlinkmany $DIR/$tfile-%d 0 400
-    DEBUG42="`sysctl -n lnet.debug`"
+    debugsave
     sysctl -w lnet.debug=-1
     facet_failover ost1
     
@@ -917,7 +893,7 @@ test_42() {
     #[ $blocks_after -lt $blocks ] || return 1
     echo wait for MDS to timeout and recover
     sleep $((TIMEOUT * 2))
-    sysctl -w lnet.debug=$DEBUG42
+    debugrestore
     unlinkmany $DIR/$tfile-%d 400 400
     $CHECKSTAT -t file $DIR/$tfile-* && return 2 || true
 }
@@ -955,6 +931,7 @@ test_44b() {
     mdcdev=`awk '/-mdc-/ {print $1}' $LPROC/devices`
     [ "$mdcdev" ] || exit 2
     for i in `seq 1 10`; do
+	echo "$i of 10 ($(date +%s))"
 	#define OBD_FAIL_TGT_DELAY_RECONNECT 0x704
 	do_facet mds "sysctl -w lustre.fail_loc=0x80000704"
 	$LCTL --device $mdcdev recover
@@ -1122,5 +1099,78 @@ test_58() {
 }
 run_test 58 "test recovery from llog for setattr op (test llog_gen_rec)"
 
+# log_commit_thread vs filter_destroy race used to lead to import use after free
+# bug 11658
+test_59() {
+    mkdir $DIR/$tdir
+    createmany -o $DIR/$tdir/$tfile-%d 200
+    sync
+    unlinkmany $DIR/$tdir/$tfile-%d 200
+#define OBD_FAIL_PTLRPC_DELAY_RECOV       0x507
+    do_facet ost1 "sysctl -w lustre.fail_loc=0x507"
+    fail ost1
+    fail mds
+    do_facet ost1 "sysctl -w lustre.fail_loc=0x0"
+    sleep 20
+    rmdir $DIR/$tdir
+}
+run_test 59 "test log_commit_thread vs filter_destroy race"
+
+# race between add unlink llog vs cat log init in post_recovery (only for b1_6)
+# bug 12086: should no oops and No ctxt error for this test
+test_60() {
+    mkdir $DIR/$tdir
+    createmany -o $DIR/$tdir/$tfile-%d 200
+    replay_barrier mds
+    unlinkmany $DIR/$tdir/$tfile-%d 0 100
+    fail mds
+    unlinkmany $DIR/$tdir/$tfile-%d 100 100
+    local no_ctxt=`dmesg | grep "No ctxt"`
+    [ -z "$no_ctxt" ] || error "ctxt is not initialized in recovery" 
+}
+run_test 60 "test llog post recovery init vs llog unlink"
+
+#test race  llog recovery thread vs llog cleanup
+test_61a() {
+    mkdir $DIR/$tdir
+    createmany -o $DIR/$tdir/$tfile-%d 800
+    replay_barrier ost1 
+#   OBD_FAIL_OST_LLOG_RECOVERY_TIMEOUT 0x221 
+    unlinkmany $DIR/$tdir/$tfile-%d 800 
+    do_facet ost "sysctl -w lustre.fail_loc=0x80000221"
+    facet_failover ost1
+    sleep 10 
+    fail ost1
+    sleep 30
+    do_facet ost "sysctl -w lustre.fail_loc=0x0"
+    $CHECKSTAT -t file $DIR/$tdir/$tfile-* && return 1
+    rmdir $DIR/$tdir
+}
+run_test 61a "test race llog recovery vs llog cleanup"
+
+#test race  mds llog sync vs llog cleanup
+test_61b() {
+#   OBD_FAIL_MDS_LLOG_SYNC_TIMEOUT 0x13a 
+    do_facet mds "sysctl -w lustre.fail_loc=0x8000013a"
+    facet_failover mds 
+    sleep 10
+    fail mds
+    do_facet client dd if=/dev/zero of=$DIR/$tfile bs=4k count=1 || return 1
+}
+run_test 61b "test race mds llog sync vs llog cleanup"
+
+#test race  cancel cookie cb vs llog cleanup
+test_61c() {
+#   OBD_FAIL_OST_CANCEL_COOKIE_TIMEOUT 0x222 
+    touch $DIR/$tfile 
+    do_facet ost "sysctl -w lustre.fail_loc=0x80000222"
+    rm $DIR/$tfile    
+    sleep 10
+    fail ost1
+}
+run_test 61c "test race mds llog sync vs llog cleanup"
+
+
 equals_msg `basename $0`: test complete, cleaning up
-$CLEANUP
+check_and_cleanup_lustre
+[ -f "$TESTSUITELOG" ] && cat $TESTSUITELOG || true
