@@ -4,7 +4,6 @@
  *  Copyright (C) 2002 Cluster File Systems, Inc.
  *   Author: Peter J. Braam <braam@clusterfs.com>
  *   Author: Phil Schwan <phil@clusterfs.com>
- *   Author: Brian Behlendorf <behlendorf1@llnl.gov>
  *   Author: Robert Read <rread@clusterfs.com>
  *
  *   This file is part of Lustre, http://www.lustre.org.
@@ -50,14 +49,12 @@
 #include <unistd.h>
 #endif
 
-#include <lnet/lnetctl.h>
-
 #include <liblustre.h>
+#include <lnet/lnetctl.h>
 #include <obd.h>
 #include <lustre_lib.h>
 #include <lustre/liblustreapi.h>
 #include <obd_lov.h>
-#include <lustre/liblustreapi.h>
 
 static void err_msg(char *fmt, ...)
 {
@@ -70,23 +67,74 @@ static void err_msg(char *fmt, ...)
         fprintf(stderr, ": %s (%d)\n", strerror(tmp_errno), tmp_errno);
 }
 
-int llapi_file_create(const char *name, unsigned long stripe_size,
-                      int stripe_offset, int stripe_count, int stripe_pattern)
+int parse_size(char *optarg, unsigned long long *size,
+               unsigned long long *size_units)
+{
+        char *end;
+
+        *size = strtoul(optarg, &end, 0);
+
+        if (*end != '\0') {
+                if ((*end == 'b') && *(end+1) == '\0' &&
+                    (*size & (~0ULL << (64 - 9))) == 0) {
+                        *size <<= 9;
+                        *size_units = 1 << 9;
+                } else if ((*end == 'k' || *end == 'K') &&
+                           *(end+1) == '\0' && (*size &
+                           (~0ULL << (64 - 10))) == 0) {
+                        *size <<= 10;
+                        *size_units = 1 << 10;
+                } else if ((*end == 'm' || *end == 'M') &&
+                           *(end+1) == '\0' && (*size &
+                           (~0ULL << (64 - 20))) == 0) {
+                        *size <<= 20;
+                        *size_units = 1 << 20;
+                } else if ((*end == 'g' || *end == 'G') &&
+                           *(end+1) == '\0' && (*size &
+                           (~0ULL << (64 - 30))) == 0) {
+                        *size <<= 30;
+                        *size_units = 1 << 30;
+                } else if ((*end == 't' || *end == 'T') &&
+                           *(end+1) == '\0' && (*size &
+                           (~0ULL << (64 - 40))) == 0) {
+                        *size <<= 40;
+                        *size_units = 1ULL << 40;
+                } else if ((*end == 'p' || *end == 'P') &&
+                           *(end+1) == '\0' && (*size &
+                           (~0ULL << (64 - 50))) == 0) {
+                        *size <<= 50;
+                        *size_units = 1ULL << 50;
+                } else if ((*end == 'e' || *end == 'E') &&
+                           *(end+1) == '\0' && (*size &
+                           (~0ULL << (64 - 60))) == 0) {
+                        *size <<= 60;
+                        *size_units = 1ULL << 60;
+                } else {
+                        return -1;
+                }
+        }
+
+        return 0;
+}
+
+int llapi_file_open(const char *name, int flags, int mode,
+                    unsigned long stripe_size, int stripe_offset,
+                    int stripe_count, int stripe_pattern)
 {
         struct lov_user_md lum = { 0 };
         int fd, rc = 0;
         int isdir = 0;
         int page_size;
 
-        fd = open(name, O_CREAT | O_RDWR | O_LOV_DELAY_CREATE, 0644);
-        if (errno == EISDIR) {
+        fd = open(name, flags | O_LOV_DELAY_CREATE, mode);
+        if (fd < 0 && errno == EISDIR) {
                 fd = open(name, O_DIRECTORY | O_RDONLY);
                 isdir++;
         }
 
         if (fd < 0) {
                 rc = -errno;
-                err_msg("unable to open '%s'",name);
+                err_msg("unable to open '%s'", name);
                 return rc;
         }
 
@@ -134,20 +182,34 @@ int llapi_file_create(const char *name, unsigned long stripe_size,
                 if (errno != EEXIST && errno != EALREADY)
                         errmsg = strerror(errno);
 
-                rc = -errno;
                 fprintf(stderr, "error on ioctl "LPX64" for '%s' (%d): %s\n",
                         (__u64)LL_IOC_LOV_SETSTRIPE, name, fd, errmsg);
         }
 out:
-        if (close(fd) < 0) {
-                if (rc == 0)
-                        rc = -errno;
-                err_msg("error on close for '%s' (%d)", name, fd);
+        if (rc) {
+                close(fd);
+                fd = rc;
         }
-        return rc;
+
+        return fd;
 }
 
-typedef int (semantic_func_t)(char *path, DIR *parent, DIR *d, void *data);
+int llapi_file_create(const char *name, unsigned long stripe_size,
+                      int stripe_offset, int stripe_count, int stripe_pattern)
+{
+        int fd;
+
+        fd = llapi_file_open(name, O_CREAT | O_WRONLY, 0644, stripe_size,
+                             stripe_offset, stripe_count, stripe_pattern);
+        if (fd < 0)
+                return fd;
+
+        close(fd);
+        return 0;
+}
+
+typedef int (semantic_func_t)(char *path, DIR *parent, DIR *d,
+                              void *data, struct dirent64 *de);
 
 #define MAX_LOV_UUID_COUNT      max(LOV_MAX_STRIPE_COUNT, 1000)
 #define OBD_NOT_FOUND           (-1)
@@ -162,74 +224,71 @@ static int common_param_init(struct find_param *param)
         }
 
         param->got_uuids = 0;
+        param->obdindexes = NULL;
         param->obdindex = OBD_NOT_FOUND;
         return 0;
 }
 
 static void find_param_fini(struct find_param *param)
 {
+        if (param->obdindexes)
+                free(param->obdindexes);
+
         if (param->lmd)
                 free(param->lmd);
 }
 
+/*
+ * If uuidp is NULL, return the number of available obd uuids.
+ * If uuidp is non-NULL, then it will return the uuids of the obds. If
+ * there are more OSTs then allocated to uuidp, then an error is returned with
+ * the ost_count set to number of available obd uuids.
+ */
 int llapi_lov_get_uuids(int fd, struct obd_uuid *uuidp, int *ost_count)
 {
-        struct obd_ioctl_data data = { 0, };
-        struct lov_desc desc = { 0, };
-        char *buf = NULL;
-        int max_ost_count, rc;
-        __u32 *obdgens;
+        char lov_name[sizeof(struct obd_uuid)];
+        char buf[1024];
+        FILE *fp;
+        int rc = 0, index = 0;
 
-        max_ost_count = (OBD_MAX_IOCTL_BUFFER - size_round(sizeof(data)) -
-                         size_round(sizeof(desc))) /
-                        (sizeof(*uuidp) + sizeof(*obdgens));
-        if (max_ost_count > *ost_count)
-                max_ost_count = *ost_count;
-
-        obdgens = malloc(size_round(max_ost_count * sizeof(*obdgens)));
-        if (!obdgens) {
-                err_msg("error: %d generation #'s", max_ost_count);
-                return(-ENOMEM);
-        }
-
-        data.ioc_inllen1 = sizeof(desc);
-        data.ioc_inlbuf1 = (char *)&desc;
-        data.ioc_inllen2 = size_round(max_ost_count * sizeof(*uuidp));
-        data.ioc_inlbuf2 = (char *)uuidp;
-        data.ioc_inllen3 = size_round(max_ost_count * sizeof(*obdgens));
-        data.ioc_inlbuf3 = (char *)obdgens;
-
-        desc.ld_tgt_count = max_ost_count;
-
-        if (obd_ioctl_pack(&data, &buf, OBD_MAX_IOCTL_BUFFER)) {
-                fprintf(stderr, "error: %s: internal packing error\n",
-                        __FUNCTION__);
-                rc = EINVAL;
-                goto out;
-        }
-
-        rc = ioctl(fd, OBD_IOC_LOV_GET_CONFIG, buf);
+        /* Get the lov name */
+        rc = ioctl(fd, OBD_IOC_GETNAME, (void *) lov_name);
         if (rc) {
                 rc = errno;
-                err_msg("error: %s: getting LOV config", __FUNCTION__);
-                goto out;
+                err_msg("error: can't get lov name.");
+                return rc;
         }
 
-        if (obd_ioctl_unpack(&data, buf, OBD_MAX_IOCTL_BUFFER)) {
-                rc = errno = EINVAL;
-                err_msg("error: %s: internal ioctl unpack", __FUNCTION__);
-                goto out;
+        /* Now get the ost uuids from /proc */
+        snprintf(buf, sizeof(buf), "/proc/fs/lustre/lov/%s/target_obd",
+                 lov_name);
+        fp = fopen(buf, "r");
+        if (fp == NULL) {
+                rc = errno;
+                err_msg("error: opening '%s'", buf);
+                return rc;
         }
 
-        *ost_count = desc.ld_tgt_count;
-out:
-        free(buf);
-        free(obdgens);
+        while (fgets(buf, sizeof(buf), fp) != NULL) {
+                if (uuidp && (index < *ost_count)) {
+                        if (sscanf(buf, "%d: %s", &index, uuidp[index].uuid) <2)
+                                break;
+                }
+                index++;
+        }
 
+        fclose(fp);
+
+        if (uuidp && (index >= *ost_count))
+                return -EOVERFLOW;
+
+        *ost_count = index;
         return rc;
 }
 
-static int setup_obd_uuids(DIR *dir, char *dname, struct find_param *param)
+/* Here, param->obduuid points to a single obduuid, the index of which is
+ * returned in param->obdindex */
+static int setup_obd_uuid(DIR *dir, char *dname, struct find_param *param)
 {
         char uuid[sizeof(struct obd_uuid)];
         char buf[1024];
@@ -268,7 +327,7 @@ static int setup_obd_uuids(DIR *dir, char *dname, struct find_param *param)
                         break;
 
                 if (param->obduuid) {
-                        if (strncmp(param->obduuid->uuid, uuid,
+                        if (strncmp((char *)param->obduuid->uuid, uuid,
                                     sizeof(uuid)) == 0) {
                                 param->obdindex = index;
                                 break;
@@ -292,6 +351,66 @@ static int setup_obd_uuids(DIR *dir, char *dname, struct find_param *param)
         return (rc);
 }
 
+/* In this case, param->obduuid will be an array of obduuids and
+ * obd index for all these obduuids will be returned in
+ * param->obdindexes */
+static int setup_obd_indexes(DIR *dir, struct find_param *param)
+{
+        struct obd_uuid *uuids = NULL;
+        int obdcount = INIT_ALLOC_NUM_OSTS;
+        int ret, obd_valid = 0, obdnum, i;
+
+        uuids = (struct obd_uuid *)malloc(INIT_ALLOC_NUM_OSTS *
+                                          sizeof(struct obd_uuid));
+        if (uuids == NULL)
+                return -ENOMEM;
+
+retry_get_uuids:
+        ret = llapi_lov_get_uuids(dirfd(dir), uuids,
+                                  &obdcount);
+        if (ret) {
+                struct obd_uuid *uuids_temp;
+
+                if (ret == -EOVERFLOW) {
+                        uuids_temp = realloc(uuids, obdcount *
+                                             sizeof(struct obd_uuid));
+                        if (uuids_temp != NULL)
+                                goto retry_get_uuids;
+                        else
+                                ret = -ENOMEM;
+                }
+
+                fprintf(stderr, "get ost uuid failed: %s\n", strerror(errno));
+                return ret;
+        }
+
+        param->obdindexes = malloc(param->num_obds * sizeof(param->obdindex));
+        if (param->obdindexes == NULL)
+                return -ENOMEM;
+
+        for (obdnum = 0; obdnum < param->num_obds; obdnum++) {
+                for (i = 0; i <= obdcount; i++) {
+                        if (strcmp((char *)&param->obduuid[obdnum].uuid,
+                                   (char *)&uuids[i]) == 0) {
+                                param->obdindexes[obdnum] = i;
+                                obd_valid++;
+                                break;
+                        }
+                }
+                if (i == obdcount)
+                        param->obdindexes[obdnum] = OBD_NOT_FOUND;
+        }
+
+        if (obd_valid == 0)
+                param->obdindex = OBD_NOT_FOUND;
+        else
+                param->obdindex = obd_valid;
+
+        param->got_uuids = 1;
+
+        return 0;
+}
+
 void lov_dump_user_lmm_v1(struct lov_user_md_v1 *lum, char *path, int is_dir,
                           int obdindex, int quiet, int header, int body)
 {
@@ -534,9 +653,10 @@ static DIR *opendir_parent(char *path)
         return parent;
 }
 
-static int llapi_semantic_traverse(char *path, DIR *parent,
+static int llapi_semantic_traverse(char *path, int size, DIR *parent,
                                    semantic_func_t sem_init,
-                                   semantic_func_t sem_fini, void *data)
+                                   semantic_func_t sem_fini, void *data,
+                                   struct dirent64 *de)
 {
         struct dirent64 *dent;
         int len, ret;
@@ -557,7 +677,7 @@ static int llapi_semantic_traverse(char *path, DIR *parent,
                         GOTO(out, ret = -EINVAL);
         }
 
-        if (sem_init && (ret = sem_init(path, parent ?: p, d, data)))
+        if (sem_init && (ret = sem_init(path, parent ?: p, d, data, de)))
                 goto err;
 
         if (!d)
@@ -568,6 +688,12 @@ static int llapi_semantic_traverse(char *path, DIR *parent,
                         continue;
 
                 path[len] = 0;
+                if ((len + dent->d_reclen + 2) > size) {
+                        fprintf(stderr,
+                                "error: %s: string buffer is too small\n",
+                                __FUNCTION__);
+                        break;
+                }
                 strcat(path, "/");
                 strcat(path, dent->d_name);
 
@@ -584,20 +710,20 @@ static int llapi_semantic_traverse(char *path, DIR *parent,
                          * tool only makes sense for lustre filesystems. */
                         break;
                 case DT_DIR:
-                        ret = llapi_semantic_traverse(path, d, sem_init,
-                                                      sem_fini, data);
+                        ret = llapi_semantic_traverse(path, size, d, sem_init,
+                                                      sem_fini, data, dent);
                         if (ret < 0)
                                 goto out;
                         break;
                 default:
                         ret = 0;
                         if (sem_init) {
-                                ret = sem_init(path, d, NULL, data);
+                                ret = sem_init(path, d, NULL, data, dent);
                                 if (ret < 0)
                                         goto out;
                         }
                         if (sem_fini && ret == 0)
-                                sem_fini(path, d, NULL, data);
+                                sem_fini(path, d, NULL, data, dent);
                 }
         }
 
@@ -605,7 +731,7 @@ out:
         path[len] = 0;
 
         if (sem_fini)
-                sem_fini(path, parent, d, data);
+                sem_fini(path, parent, d, data, de);
 err:
         if (d)
                 closedir(d);
@@ -614,12 +740,12 @@ err:
         return ret;
 }
 
-/* Check if the file time matches 1 of the given criteria (e.g. --atime +/-N).
+/* Check if the value matches 1 of the given criteria (e.g. --atime +/-N).
  * @mds indicates if this is MDS timestamps and there are attributes on OSTs.
  *
  * The result is -1 if it does not match, 0 if not yet clear, 1 if matches.
- * The table bolow gives the answers for the specified parameters (time and
- * sign), 1st column is the answer for the MDS time, the 2nd is for the OST:
+ * The table below gives the answers for the specified parameters (value and
+ * sign), 1st column is the answer for the MDS value, the 2nd is for the OST:
  * --------------------------------------
  * 1 | file > limit; sign > 0 | -1 / -1 |
  * 2 | file = limit; sign > 0 |  ? /  1 |
@@ -631,18 +757,20 @@ err:
  * 8 | file = limit; sign < 0 |  ? / -1 |
  * 9 | file < limit; sign < 0 |  ? / -1 |
  * --------------------------------------
- * Note: 5th actually means that the file time stamp is within the interval
- * (limit - 24hours, limit]. */
-static int find_time_cmp(time_t file, time_t limit, int sign, int mds) {
+ * Note: 5th actually means that the value is within the interval
+ * (limit - margin, limit]. */
+static int find_value_cmp(unsigned int file, unsigned int limit, int sign,
+                          unsigned long long margin, int mds)
+{
         if (sign > 0) {
-                if (file <= limit)
+                if (file < limit)
                         return mds ? 0 : 1;
         }
 
         if (sign == 0) {
-                if (file <= limit && file + 24 * 60 * 60 > limit)
+                if (file <= limit && file + margin > limit)
                         return mds ? 0 : 1;
-                if (file + 24 * 60 * 60 <= limit)
+                if (file + margin <= limit)
                         return mds ? 0 : -1;
         }
 
@@ -670,16 +798,16 @@ static int find_time_check(lstat_t *st, struct find_param *param, int mds)
 
         /* Check if file is accepted. */
         if (param->atime) {
-                ret = find_time_cmp(st->st_atime, param->atime,
-                                    param->asign, mds);
+                ret = find_value_cmp(st->st_atime, param->atime,
+                                     param->asign, 24 * 60 * 60, mds);
                 if (ret < 0)
                         return ret;
                 rc = ret;
         }
 
         if (param->mtime) {
-                ret = find_time_cmp(st->st_mtime, param->mtime,
-                                    param->msign, mds);
+                ret = find_value_cmp(st->st_mtime, param->mtime,
+                                     param->msign, 24 * 60 * 60, mds);
                 if (ret < 0)
                         return ret;
 
@@ -690,8 +818,8 @@ static int find_time_check(lstat_t *st, struct find_param *param, int mds)
         }
 
         if (param->ctime) {
-                ret = find_time_cmp(st->st_ctime, param->ctime,
-                                    param->csign, mds);
+                ret = find_value_cmp(st->st_ctime, param->ctime,
+                                     param->csign, 24 * 60 * 60, mds);
                 if (ret < 0)
                         return ret;
 
@@ -704,28 +832,74 @@ static int find_time_check(lstat_t *st, struct find_param *param, int mds)
         return rc;
 }
 
-static int cb_find_init(char *path, DIR *parent, DIR *dir, void *data)
+static unsigned llapi_dir_filetype_table[] = {
+        [DT_UNKNOWN]= 0,
+        [DT_FIFO]= S_IFIFO,
+        [DT_CHR] = S_IFCHR,
+        [DT_DIR] = S_IFDIR,
+        [DT_BLK] = S_IFBLK,
+        [DT_REG] = S_IFREG,
+        [DT_LNK] = S_IFLNK,
+        [DT_SOCK]= S_IFSOCK,
+#if defined(DT_DOOR) && defined(S_IFDOOR)
+        [DT_DOOR]= S_IFDOOR,
+#endif
+};
+#if defined(DT_DOOR) && defined(S_IFDOOR)
+static const int DT_MAX = DT_DOOR;
+#else
+static const int DT_MAX = DT_SOCK;
+#endif
+
+static int cb_find_init(char *path, DIR *parent, DIR *dir,
+                        void *data, struct dirent64 *de)
 {
         struct find_param *param = (struct find_param *)data;
         int decision = 1; /* 1 is accepted; -1 is rejected. */
         lstat_t *st = &param->lmd->lmd_st;
         int lustre_fs = 1;
-        int ret = 0;
+        int checked_type = 0;
+        int ret;
 
         LASSERT(parent != NULL || dir != NULL);
 
         param->lmd->lmd_lmm.lmm_stripe_count = 0;
 
+        /* If a regular expression is presented, make the initial decision */
+        if (param->pattern != NULL) {
+                char *fname = strrchr(path, '/');
+                fname = (fname == NULL ? path : fname + 1);
+                ret = fnmatch(param->pattern, fname, 0);
+                if ((ret == FNM_NOMATCH && !param->exclude_pattern) ||
+                    (ret == 0 && param->exclude_pattern))
+                        goto decided;
+        }
+
+        /* See if we can check the file type from the dirent. */
+        if (param->type && de != NULL && de->d_type != DT_UNKNOWN &&
+            de->d_type <= DT_MAX) {
+                checked_type = 1;
+                if (llapi_dir_filetype_table[de->d_type] == param->type) {
+                        if (param->exclude_type)
+                                goto decided;
+                } else {
+                        if (!param->exclude_type)
+                                goto decided;
+                }
+        }
+
+
         /* If a time or OST should be checked, the decision is not taken yet. */
-        if (param->atime || param->ctime || param->mtime || param->obduuid)
+        if (param->atime || param->ctime || param->mtime || param->obduuid ||
+            param->size_check)
                 decision = 0;
 
         /* Request MDS for the stat info. */
-        if (!decision && dir) {
+        if (dir) {
                 /* retrieve needed file info */
                 ret = ioctl(dirfd(dir), LL_IOC_MDC_GETINFO,
                             (void *)param->lmd);
-        } else if (!decision && parent) {
+        } else /* if (parent) LASSERT() above makes always true */ {
                 char *fname = strrchr(path, '/');
                 fname = (fname == NULL ? path : fname + 1);
 
@@ -754,6 +928,28 @@ static int cb_find_init(char *path, DIR *parent, DIR *dir, void *data)
                 }
         }
 
+        /* Check the time on mds. */
+        if (!decision) {
+                int for_mds;
+
+                for_mds = lustre_fs ? (S_ISREG(st->st_mode) &&
+                                       param->lmd->lmd_lmm.lmm_stripe_count)
+                                    : 0;
+                decision = find_time_check(st, param, for_mds);
+                if (decision == -1)
+                        goto decided;
+        }
+
+        if (param->type && !checked_type) {
+                if ((st->st_mode & S_IFMT) == param->type) {
+                        if (param->exclude_type)
+                                goto decided;
+                } else {
+                        if (!param->exclude_type)
+                                goto decided;
+                }
+        }
+
         /* Prepare odb. */
         if (param->obduuid) {
                 if (lustre_fs && param->got_uuids &&
@@ -765,9 +961,10 @@ static int cb_find_init(char *path, DIR *parent, DIR *dir, void *data)
                 }
 
                 if (lustre_fs && !param->got_uuids) {
-                        ret = setup_obd_uuids(dir ? dir : parent, path, param);
+                        ret = setup_obd_indexes(dir ? dir : parent, param);
                         if (ret)
                                 return ret;
+
                         param->st_dev = st->st_dev;
                 } else if (!lustre_fs && param->got_uuids) {
                         /* A lustre/non-lustre mount point is crossed. */
@@ -776,52 +973,42 @@ static int cb_find_init(char *path, DIR *parent, DIR *dir, void *data)
                 }
         }
 
-        /* If a regular expression is presented, make the initial decision */
-        if (param->pattern != NULL) {
-                char *fname = strrchr(path, '/');
-                fname = (fname == NULL ? path : fname + 1);
-                ret = fnmatch(param->pattern, fname, 0);
-                if ((ret == FNM_NOMATCH && !param->exclude_pattern) ||
-                    (ret == 0 && param->exclude_pattern))
-                        decision = -1;
-        }
-
         /* If an OBD UUID is specified but no one matches, skip this file. */
         if (param->obduuid && param->obdindex == OBD_NOT_FOUND)
-                decision = -1;
+                goto decided;
 
         /* If a OST UUID is given, and some OST matches, check it here. */
-        if (decision != -1 && param->obdindex != OBD_NOT_FOUND) {
-                /* Only those files should be accepted, which have a strip on
-                 * the specified OST. */
+        if (param->obdindex != OBD_NOT_FOUND) {
+                if (!S_ISREG(st->st_mode))
+                        goto decided;
+
+                /* Only those files should be accepted, which have a
+                 * stripe on the specified OST. */
                 if (!param->lmd->lmd_lmm.lmm_stripe_count) {
-                        decision = -1;
+                        goto decided;
                 } else {
-                        int i;
+                        int i, j;
                         for (i = 0;
                              i < param->lmd->lmd_lmm.lmm_stripe_count; i++) {
-                               if (param->obdindex ==
-                                   param->lmd->lmd_lmm.lmm_objects[i].l_ost_idx)
-                                        break;
+                                for (j = 0; j < param->num_obds; j++) {
+                                        if (param->obdindexes[j] ==
+                                            param->lmd->lmd_lmm.lmm_objects[i].l_ost_idx)
+                                                goto obd_matches;
+                                }
                         }
 
                         if (i == param->lmd->lmd_lmm.lmm_stripe_count)
-                                decision = -1;
+                                goto decided;
                 }
         }
 
-        /* Check the time on mds. */
-        if (!decision) {
-                int for_mds;
-
-                for_mds = lustre_fs ? param->lmd->lmd_lmm.lmm_stripe_count : 0;
-                decision = find_time_check(st, param, for_mds);
-        }
+obd_matches:
 
         /* If file still fits the request, ask osd for updated info.
            The regulat stat is almost of the same speed as some new
            'glimpse-size-ioctl'. */
-        if (!decision && param->lmd->lmd_lmm.lmm_stripe_count) {
+        if (!decision && S_ISREG(st->st_mode) &&
+            (param->lmd->lmd_lmm.lmm_stripe_count || param->size_check)) {
                 if (dir) {
                         ret = ioctl(dirfd(dir), IOC_LOV_GETINFO,
                                     (void *)param->lmd);
@@ -837,10 +1024,16 @@ static int cb_find_init(char *path, DIR *parent, DIR *dir, void *data)
                 }
 
                 /* Check the time on osc. */
-                if (!decision)
-                        decision = find_time_check(st, param, 0);
+                decision = find_time_check(st, param, 0);
+                if (decision == -1)
+                        goto decided;
         }
 
+        if (param->size_check)
+                decision = find_value_cmp(st->st_size, param->size,
+                                          param->size_sign, param->size_units,
+                                          0);
+
         if (decision != -1) {
                 printf("%s", path);
                 if (param->zeroend)
@@ -849,6 +1042,7 @@ static int cb_find_init(char *path, DIR *parent, DIR *dir, void *data)
                         printf("\n");
         }
 
+decided:
         /* Do not get down anymore? */
         if (param->depth == param->maxdepth)
                 return 1;
@@ -857,7 +1051,8 @@ static int cb_find_init(char *path, DIR *parent, DIR *dir, void *data)
         return 0;
 }
 
-static int cb_common_fini(char *path, DIR *parent, DIR *d, void *data)
+static int cb_common_fini(char *path, DIR *parent, DIR *d, void *data,
+                          struct dirent64 *de)
 {
         struct find_param *param = (struct find_param *)data;
         param->depth--;
@@ -866,25 +1061,38 @@ static int cb_common_fini(char *path, DIR *parent, DIR *d, void *data)
 
 int llapi_find(char *path, struct find_param *param)
 {
-        char buf[PATH_MAX + 1];
-        int ret;
+        char *buf;
+        int ret, len = strlen(path);
+
+        if (len > PATH_MAX) {
+                fprintf(stderr, "%s: Path name '%s' is too long.\n",
+                        __FUNCTION__, path);
+                return -EINVAL;
+        }
+
+        buf = (char *)malloc(PATH_MAX + 1);
+        if (!buf)
+                return -ENOMEM;
 
         ret = common_param_init(param);
-        if (ret)
+        if (ret) {
+                free(buf);
                 return ret;
+        }
 
         param->depth = 0;
-        strncpy(buf, path, strlen(path));
-        buf[strlen(path)] = '\0';
 
-        ret = llapi_semantic_traverse(buf, NULL, cb_find_init,
-                                      cb_common_fini, param);
+        strncpy(buf, path, PATH_MAX + 1);
+        ret = llapi_semantic_traverse(buf, PATH_MAX + 1, NULL, cb_find_init,
+                                      cb_common_fini, param, NULL);
 
         find_param_fini(param);
+        free(buf);
         return ret < 0 ? ret : 0;
 }
 
-static int cb_getstripe(char *path, DIR *parent, DIR *d, void *data)
+static int cb_getstripe(char *path, DIR *parent, DIR *d, void *data,
+                        struct dirent64 *de)
 {
         struct find_param *param = (struct find_param *)data;
         int ret = 0;
@@ -893,7 +1101,7 @@ static int cb_getstripe(char *path, DIR *parent, DIR *d, void *data)
 
         /* Prepare odb. */
         if (!param->got_uuids) {
-                ret = setup_obd_uuids(d ? d : parent, path, param);
+                ret = setup_obd_uuid(d ? d : parent, path, param);
                 if (ret)
                         return ret;
         }
@@ -939,16 +1147,32 @@ out:
 
 int llapi_getstripe(char *path, struct find_param *param)
 {
-        int ret = 0;
+        char *buf;
+        int ret = 0, len = strlen(path);
+
+        if (len > PATH_MAX) {
+                fprintf(stderr, "%s: Path name '%s' is too long.\n",
+                        __FUNCTION__, path);
+                return -EINVAL;
+        }
+
+        buf = (char *)malloc(PATH_MAX + 1);
+        if (!buf)
+                return -ENOMEM;
 
         ret = common_param_init(param);
-        if (ret)
+        if (ret) {
+                free(buf);
                 return ret;
+        }
 
         param->depth = 0;
-        ret = llapi_semantic_traverse(path, NULL, cb_getstripe,
-                                      cb_common_fini, param);
+
+        strncpy(buf, path, PATH_MAX + 1);
+        ret = llapi_semantic_traverse(buf, PATH_MAX + 1, NULL, cb_getstripe,
+                                      cb_common_fini, param, NULL);
         find_param_fini(param);
+        free(buf);
         return ret < 0 ? ret : 0;
 }
 
@@ -1203,7 +1427,8 @@ int llapi_quotactl(char *mnt, struct if_quotactl *qctl)
         return rc;
 }
 
-static int cb_quotachown(char *path, DIR *parent, DIR *d, void *data)
+static int cb_quotachown(char *path, DIR *parent, DIR *d, void *data,
+                         struct dirent64 *de)
 {
         struct find_param *param = (struct find_param *)data;
         lstat_t *st;
@@ -1247,7 +1472,7 @@ static int cb_quotachown(char *path, DIR *parent, DIR *d, void *data)
          * invoke syscall directly. */
         rc = syscall(SYS_chown, path, -1, -1);
         if (rc)
-                err_msg("error: chown %s (%u,%u)", path);
+                err_msg("error: chown %s", path);
 
         rc = chmod(path, st->st_mode);
         if (rc)
@@ -1259,7 +1484,18 @@ static int cb_quotachown(char *path, DIR *parent, DIR *d, void *data)
 int llapi_quotachown(char *path, int flag)
 {
         struct find_param param;
-        int ret = 0;
+        char *buf;
+        int ret = 0, len = strlen(path);
+
+        if (len > PATH_MAX) {
+                fprintf(stderr, "%s: Path name '%s' is too long.\n",
+                        __FUNCTION__, path);
+                return -EINVAL;
+        }
+
+        buf = (char *)malloc(PATH_MAX + 1);
+        if (!buf)
+                return -ENOMEM;
 
         memset(&param, 0, sizeof(param));
         param.recursive = 1;
@@ -1270,9 +1506,11 @@ int llapi_quotachown(char *path, int flag)
         if (ret)
                 goto out;
 
-        ret = llapi_semantic_traverse(path, NULL, cb_quotachown,
-                                      NULL, &param);
+        strncpy(buf, path, PATH_MAX + 1);
+        ret = llapi_semantic_traverse(buf, PATH_MAX + 1, NULL, cb_quotachown,
+                                      NULL, &param, NULL);
 out:
         find_param_fini(&param);
+        free(buf);
         return ret;
 }
